@@ -2,9 +2,9 @@
 [styling]
 # foreground;background;bold;italic
 default=0x000000;0xffffff;false;false
-comment=0xff0000;0xffffff;false;false
-commentline=0xff0000;0xffffff;false;false
-commentdoc=0xff0000;0xffffff;false;false
+comment=0xd00000;0xffffff;false;false
+commentline=0xd00000;0xffffff;false;false
+commentdoc=0x3f5fbf;0xffffff;false;false
 number=0x007f00;0xffffff;false;false
 word=0x00007f;0xffffff;true;false
 word2=0x991111;0xffffff;true;false
@@ -17,9 +17,10 @@ identifier=0x000000;0xffffff;false;false
 stringeol=0x000000;0xe0c0e0;false;false
 verbatim=0x101030;0xffffff;false;false
 regex=0x105090;0xffffff;false;false
-commentlinedoc=0xff0000;0xffffff;true;false
-commentdockeyword=0xff0000;0xffffff;true;true
-globalclass=0x1111bb;0xffffff;true;false
+commentlinedoc=0x3f5fbf;0xffffff;true;false
+commentdockeyword=0x3f5fbf;0xffffff;true;true
+commentdockeyworderror=0x3f5fbf;0xffffff;false;false
+globalclass=0x0000d0;0xffffff;true;false
 # whether arguments of preprocessor commands should be styled (only first argument is used)
 # 1 to enable, 0 to disable
 styling_within_preprocessor=1;0;false;false
@@ -27,11 +28,12 @@ styling_within_preprocessor=1;0;false;false
 [keywords]
 # all items must be in one line
 primary=and and_eq asm auto bitand bitor bool break case catch char class compl const const_cast continue default delete do double dynamic_cast else enum explicit export extern false float for friend goto if inline int long mutable namespace new not not_eq operator or or_eq private protected public register reinterpret_cast return short signed sizeof static static_cast struct switch template this throw true try typedef typeid typename union unsigned using virtual void volatile wchar_t while xor xor_eq
-docComment=TODO FIXME
+# these are some doxygen keywords (incomplete)
+docComment=attention author brief bug class code date def enum example exception file fn namespace note param remarks return see since struct throw todo typedef var version warning union
 
 [settings]
 # the following characters are these which a "word" can contains, see documentation
-wordchars=_#&abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789
+#wordchars=_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789
 
 # if only single comment char is supported like # in this file, leave comment_close blank
 comment_open=//
