@@ -165,6 +165,8 @@ int Scintilla_LinkLexers() {
 	LINK_LEXER(lmRuby);
 	LINK_LEXER(lmSQL);
 	LINK_LEXER(lmTCL);
+	LINK_LEXER(lmVHDL);
+	LINK_LEXER(lmD);
 
 //--Autogenerated -- end of automatically generated section
 
