@@ -1,7 +1,8 @@
 /*
  *      about.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: about.c 834 2006-09-26 14:00:30Z kretek $
+ * $Id: about.c 1318 2007-02-23 12:14:14Z eht16 $
  */
 
 #include "about.h"
@@ -34,7 +35,7 @@ static GtkWidget *gb_window = NULL;
 #define INFO "<span size=\"larger\" weight=\"bold\">%s</span>"
 #define CODENAME "<span weight=\"bold\">\"" GEANY_CODENAME "\"</span>"
 #define BUILDDATE "<span size=\"smaller\">%s</span>"
-#define COPYRIGHT "Copyright (c)  2005-2006 \n Enrico Tröger \nAll rights reserved."
+#define COPYRIGHT "Copyright (c)  2005-2007 \n Enrico Tröger \nAll rights reserved."
 #define CREDITS \
 "<span size=\"larger\" weight=\"bold\">%s</span>\n\t\
 Enrico Tröger - %s\n\t\
@@ -47,14 +48,24 @@ Frank Lanitz - %s\n\t\
 \
 Yura Semashko - <language> be_BY\n\t\
 &lt;yurand2@gmail.com&gt;\n\n\t\
-Topi - <language> ca_ES\n\t\
-&lt;topi@phreaker.net&gt;\n\n\t\
+Toni Garcia-Navarro - <language> ca_ES\n\t\
+&lt;topi@elpiset.net&gt;\n\n\t\
 Petr Messner - <language> cs_CZ \n\t\
 &lt;messa@messa.cz&gt;\n\n\t\
 Frank Lanitz - <language> de_DE\n\t\
 &lt;frank@frank.uvena.de&gt;\n\n\t\
 Damián Viano - <language> es\n\t\
 &lt;debian@damianv.com.ar&gt;\n\n\t\
+Harri Koskinen  - <language> fi_FI\n\t\
+&lt;harri@fastmonkey.org&gt;\n\n\t\
+Jean-Philippe Moal - <language> fr\n\t\
+&lt;skateinmars@momo.servegame.org&gt;\n\n\t\
+Gabor Kmetyko - <language> hu\n\t\
+&lt;kg_kilo@freemail.hu&gt;\n\n\t\
+M. Baldinelli - <language> it\n\t\
+&lt;m.baldinelli@agora.it&gt;\n\n\t\
+Dario Santomarco - <language> it\n\t\
+&lt;dariello@yahoo.it&gt;\n\n\t\
 Kurt De Bree - <language> nl\n\t\
 &lt;kdebree@telenet.be&gt;\n\n\t\
 Jacek Wolszczak - <language> pl_PL\n\t\
@@ -64,7 +75,11 @@ Alexandre Moreira - <language> pt_BR\n\t\
 brahmann_ - <language> ru_RU\n\t\
 &lt;brahmann@mthr.net.ru&gt;\n\n\t\
 Clytie Siddall - <language> vi_VN\n\t\
-&lt;clytie@riverland.net.au&gt;"
+&lt;clytie@riverland.net.au&gt;\n\n\t\
+Dormouse Young - <language> zh_CN\n\t\
+&lt;dormouse.young@gmail.com&gt;\n\n\t\
+KoViCH - <language> zh_TW\n\t\
+&lt;kovich.ian@gmail.com&gt;\n\n\t"
 
 static void header_eventbox_style_set(GtkWidget *widget);
 static void header_label_style_set(GtkWidget *widget);
@@ -102,11 +117,11 @@ static GtkWidget *create_dialog(void)
 	dialog = gtk_dialog_new();
 
 	//configure dialog
-	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_CLOSE);
 	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(app->window));
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
 	gtk_window_set_title(GTK_WINDOW(dialog), _("About Geany"));
 	button = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_CLOSE);
 	gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
 	g_signal_connect(G_OBJECT(dialog), "key-press-event", G_CALLBACK(gb_on_key_pressed), NULL);
 
@@ -122,7 +137,7 @@ static GtkWidget *create_dialog(void)
 	header_label = gtk_label_new(NULL);
 	gtk_label_set_use_markup(GTK_LABEL(header_label), TRUE);
 	// print the subversion revision if it is available
-	g_snprintf(buffer, sizeof(buffer), HEADER, (utils_strcmp(REVISION, "-1")) ? "" : " (" REVISION ")");
+	g_snprintf(buffer, sizeof(buffer), HEADER, (utils_str_equal(REVISION, "-1")) ? "" : " (" REVISION ")");
 	gtk_label_set_markup(GTK_LABEL(header_label), buffer);
 	gtk_widget_show(header_label);
 	gtk_box_pack_start(GTK_BOX(header_hbox), header_label, FALSE,FALSE,0);
@@ -136,6 +151,7 @@ static GtkWidget *create_dialog(void)
 	icon = ui_new_pixbuf_from_inline(GEANY_IMAGE_LOGO, FALSE);
 	gtk_image_set_from_pixbuf(GTK_IMAGE(header_image), icon);
 	gtk_window_set_icon(GTK_WINDOW(dialog), icon);
+	g_object_unref(icon);	// free our reference
 
 	// create notebook
 	notebook = gtk_notebook_new();
