@@ -17,7 +17,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: callbacks.h 814 2006-09-12 08:32:52Z eht16 $
+ * $Id: callbacks.h 1112 2006-12-17 19:31:32Z eht16 $
  */
 
 gint
@@ -130,11 +130,6 @@ gboolean
 on_close_all1_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-gboolean
-on_tree_view_button_press_event        (GtkWidget *widget,
-                                        GdkEventButton *event,
-                                        gpointer user_data);
-
 void
 on_crlf_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -224,15 +219,6 @@ void
 on_entry1_changed                      (GtkEditable     *editable,
                                         gpointer         user_data);
 
-
-void
-on_openfiles_tree_selection_changed    (GtkTreeSelection *selection,
-                                        gpointer data);
-
-void
-on_taglist_tree_selection_changed      (GtkTreeSelection *selection,
-                                        gpointer data);
-
 void
 on_filetype_change                     (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -286,19 +272,10 @@ void
 on_goto_tag_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_compiler_treeview_copy_activate     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_message_treeview_clear_activate     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_construct_completion1_activate      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-
 void
 on_count_words1_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -319,43 +296,6 @@ void
 on_show_color_chooser1_activate        (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_build_compile_activate              (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_tex_activate                  (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_build_activate                (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_make_activate                 (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_execute_activate              (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_arguments_activate            (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_build_tex_arguments_activate        (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_make_target_dialog_response         (GtkDialog *dialog,
-                                        gint response,
-                                        gpointer user_data);
-
-void
-on_make_target_entry_activate          (GtkEntry        *entry,
-                                        gpointer         user_data);
-
 void
 on_find1_activate                      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -420,11 +360,6 @@ void
 on_website1_activate                   (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
 
-void
-on_pref_tools_button_clicked           (GtkButton       *button,
-                                        gpointer         user_data);
-
-
 void
 on_line_breaking1_toggled              (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data);
@@ -496,14 +431,6 @@ on_tv_notebook_switch_page             (GtkNotebook     *notebook,
                                         guint            page_num,
                                         gpointer         user_data);
 
-void
-on_openfiles_tree_popup_clicked        (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
-void
-on_taglist_tree_popup_clicked          (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 gboolean
 on_window_key_press_event              (GtkWidget *widget,
                                         GdkEventKey *event,
@@ -610,3 +537,28 @@ void
 on_menu_toggle_line_commentation1_activate
                                        (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+
+void
+on_next_message1_activate              (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+
+void
+on_menu_insert_special_chars1_activate (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_comments_multiline_activate    (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_comments_gpl_activate          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_insert_include_activate        (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_menu_insert_date_activate           (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
