@@ -17,7 +17,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: filetypes.c 848 2006-09-29 07:22:46Z eht16 $
+ * $Id: filetypes.c 1038 2006-11-30 15:42:52Z ntrel $
  */
 
 
@@ -30,6 +30,9 @@
 #include "callbacks.h"
 #include "templates.h"
 #include "msgwindow.h"
+#include "utils.h"
+#include "document.h"
+#include "sciwrappers.h"
 
 
 /* This is the order of unique ids used in the config file.
@@ -61,7 +64,8 @@ enum
 	FILETYPE_UID_FORTRAN,	// 22
 	FILETYPE_UID_DIFF,		// 23
 	FILETYPE_UID_FERITE,	// 24
-	FILETYPE_UID_HTML		// 25
+	FILETYPE_UID_HTML,		// 25
+	FILETYPE_UID_VHDL		// 26
 };
 
 
@@ -103,6 +107,7 @@ filetype *filetypes_get_from_uid(gint uid)
 		case FILETYPE_UID_DIFF:		return filetypes[GEANY_FILETYPES_DIFF];
 		case FILETYPE_UID_FERITE:	return filetypes[GEANY_FILETYPES_FERITE];
 		case FILETYPE_UID_HTML:		return filetypes[GEANY_FILETYPES_HTML];
+		case FILETYPE_UID_VHDL:		return filetypes[GEANY_FILETYPES_VHDL];
 		default: 					return NULL;
 	}
 }
@@ -110,7 +115,7 @@ filetype *filetypes_get_from_uid(gint uid)
 
 /* inits the filetype array and fill it with the known filetypes
  * and create the filetype menu*/
-void filetypes_init_types(void)
+void filetypes_init_types()
 {
 	GtkWidget *filetype_menu = lookup_widget(app->window, "set_filetype1_menu");
 	GtkWidget *template_menu = lookup_widget(app->window, "menu_new_with_template1_menu");
@@ -449,8 +454,9 @@ void filetypes_init_types(void)
 	filetypes[GEANY_FILETYPES_MAKE]->title = g_strdup(_("Makefile"));
 	filetypes[GEANY_FILETYPES_MAKE]->extension = g_strdup("mak");
 	{
-		gchar *pattern_make[] = {"*.mak", "*.mk", "Makefile*", "makefile*", NULL};
-		filetypes[GEANY_FILETYPES_MAKE]->pattern = g_strdupv(pattern_make);
+		gchar *patterns[] = {"*.mak", "*.mk", "GNUmakefile", "makefile", "Makefile",
+			"makefile.*", "Makefile.*", NULL};
+		filetypes[GEANY_FILETYPES_MAKE]->pattern = g_strdupv(patterns);
 	}
 	filetypes[GEANY_FILETYPES_MAKE]->style_func_ptr = styleset_makefile;
 	filetypes[GEANY_FILETYPES_MAKE]->comment_open = g_strdup("#");
@@ -468,10 +474,10 @@ void filetypes_init_types(void)
 	filetypes[GEANY_FILETYPES_XML]->has_tags = FALSE;
 	filetypes[GEANY_FILETYPES_XML]->title = g_strdup(_("XML source file"));
 	filetypes[GEANY_FILETYPES_XML]->extension = g_strdup("xml");
-	filetypes[GEANY_FILETYPES_XML]->pattern = g_new0(gchar*, 3);
-	filetypes[GEANY_FILETYPES_XML]->pattern[0] = g_strdup("*.xml");
-	filetypes[GEANY_FILETYPES_XML]->pattern[1] = g_strdup("*.sgml");
-	filetypes[GEANY_FILETYPES_XML]->pattern[2] = NULL;
+	{
+		gchar *patterns[] = {"*.xml", "*.sgml", "*.xsl", "*.xslt", NULL};
+		filetypes[GEANY_FILETYPES_XML]->pattern = g_strdupv(patterns);
+	}
 	filetypes[GEANY_FILETYPES_XML]->style_func_ptr = styleset_xml;
 	filetypes[GEANY_FILETYPES_XML]->comment_open = g_strdup("<!--");
 	filetypes[GEANY_FILETYPES_XML]->comment_close = g_strdup("-->");
@@ -507,15 +513,11 @@ void filetypes_init_types(void)
 	filetypes[GEANY_FILETYPES_HTML]->has_tags = FALSE;
 	filetypes[GEANY_FILETYPES_HTML]->title = g_strdup(_("HTML source file"));
 	filetypes[GEANY_FILETYPES_HTML]->extension = g_strdup("html");
-	filetypes[GEANY_FILETYPES_HTML]->pattern = g_new0(gchar*, 8);
-	filetypes[GEANY_FILETYPES_HTML]->pattern[0] = g_strdup("*.htm");
-	filetypes[GEANY_FILETYPES_HTML]->pattern[1] = g_strdup("*.html");
-	filetypes[GEANY_FILETYPES_HTML]->pattern[2] = g_strdup("*.shtml");
-	filetypes[GEANY_FILETYPES_HTML]->pattern[3] = g_strdup("*.hta");
-	filetypes[GEANY_FILETYPES_HTML]->pattern[4] = g_strdup("*.htd");
-	filetypes[GEANY_FILETYPES_HTML]->pattern[5] = g_strdup("*.htt");
-	filetypes[GEANY_FILETYPES_HTML]->pattern[6] = g_strdup("*.cfm");
-	filetypes[GEANY_FILETYPES_HTML]->pattern[7] = NULL;
+	{
+		gchar *patterns[] = {"*.htm", "*.html", "*.shtml", "*.hta", "*.htd", "*.htt",
+			"*.cfm", NULL};
+		filetypes[GEANY_FILETYPES_HTML]->pattern = g_strdupv(patterns);
+	}
 	filetypes[GEANY_FILETYPES_HTML]->style_func_ptr = styleset_html;
 	filetypes[GEANY_FILETYPES_HTML]->comment_open = g_strdup("<!--");
 	filetypes[GEANY_FILETYPES_HTML]->comment_close = g_strdup("-->");
@@ -600,14 +602,34 @@ void filetypes_init_types(void)
 	filetypes_init_build_programs(filetypes[GEANY_FILETYPES_OMS]);
 	filetypes_create_menu_item(filetype_menu, filetypes[GEANY_FILETYPES_OMS]->title, filetypes[GEANY_FILETYPES_OMS]);
 
+#define VHDL
+	filetypes[GEANY_FILETYPES_VHDL] = g_new0(filetype, 1);
+	filetypes[GEANY_FILETYPES_VHDL]->id = GEANY_FILETYPES_VHDL;
+	filetypes[GEANY_FILETYPES_VHDL]->uid = FILETYPE_UID_VHDL;
+	filetypes[GEANY_FILETYPES_VHDL]->item = NULL;
+	filetypes[GEANY_FILETYPES_VHDL]->lang = 21;
+	filetypes[GEANY_FILETYPES_VHDL]->name = g_strdup("VHDL");
+	filetypes[GEANY_FILETYPES_VHDL]->has_tags = TRUE;
+	filetypes[GEANY_FILETYPES_VHDL]->title = g_strdup(_("VHDL source file"));
+	filetypes[GEANY_FILETYPES_VHDL]->extension = g_strdup("vhd");
+	filetypes[GEANY_FILETYPES_VHDL]->pattern = g_new0(gchar*, 3);
+	filetypes[GEANY_FILETYPES_VHDL]->pattern[0] = g_strdup("*.vhd");
+	filetypes[GEANY_FILETYPES_VHDL]->pattern[1] = g_strdup("*.vhdl");
+	filetypes[GEANY_FILETYPES_VHDL]->pattern[2] = NULL;
+	filetypes[GEANY_FILETYPES_VHDL]->style_func_ptr = styleset_vhdl;
+	filetypes[GEANY_FILETYPES_VHDL]->comment_open = g_strdup("--");
+	filetypes[GEANY_FILETYPES_VHDL]->comment_close = NULL;
+	filetypes_init_build_programs(filetypes[GEANY_FILETYPES_VHDL]);
+	filetypes_create_menu_item(filetype_menu, filetypes[GEANY_FILETYPES_VHDL]->title, filetypes[GEANY_FILETYPES_VHDL]);
+
 #define DIFF
 	filetypes[GEANY_FILETYPES_DIFF] = g_new0(filetype, 1);
 	filetypes[GEANY_FILETYPES_DIFF]->id = GEANY_FILETYPES_DIFF;
 	filetypes[GEANY_FILETYPES_DIFF]->uid = FILETYPE_UID_DIFF;
 	filetypes[GEANY_FILETYPES_DIFF]->item = NULL;
-	filetypes[GEANY_FILETYPES_DIFF]->lang = -2;
+	filetypes[GEANY_FILETYPES_DIFF]->lang = 20;
 	filetypes[GEANY_FILETYPES_DIFF]->name = g_strdup("Diff");
-	filetypes[GEANY_FILETYPES_DIFF]->has_tags = FALSE;
+	filetypes[GEANY_FILETYPES_DIFF]->has_tags = TRUE;
 	filetypes[GEANY_FILETYPES_DIFF]->title = g_strdup(_("Diff file"));
 	filetypes[GEANY_FILETYPES_DIFF]->extension = g_strdup("diff");
 	filetypes[GEANY_FILETYPES_DIFF]->pattern = g_new0(gchar*, 3);
@@ -683,26 +705,77 @@ void filetypes_init_types(void)
 }
 
 
-/* convenient function to save code size */
+/* convenience function - NULLs and zeros struct members */
 static void filetypes_init_build_programs(filetype *ftype)
 {
 	ftype->programs = g_new0(struct build_programs, 1);
 
-	ftype->menu_items = g_new0(struct build_menu_items, 1);
+	ftype->actions = g_new0(struct build_actions, 1);
+}
+
+
+static filetype *find_shebang(gint idx)
+{
+	gchar *line = sci_get_line(doc_list[idx].sci, 0);
+	filetype *ft = NULL;
+
+	if (strlen(line) > 2 && line[0] == '#' && line[1]=='!')
+	{
+		/// TODO does g_path_get_basename() also work under Win32 for Unix filenames?
+		gchar *basename_interpreter = g_path_get_basename(line + 2);
+
+		if (strncmp(basename_interpreter, "sh", 2) == 0)
+			ft = filetypes[GEANY_FILETYPES_SH];
+		else if (strncmp(basename_interpreter, "bash", 4) == 0)
+			ft = filetypes[GEANY_FILETYPES_SH];
+		else if (strncmp(basename_interpreter, "perl", 4) == 0)
+			ft = filetypes[GEANY_FILETYPES_PERL];
+		else if (strncmp(basename_interpreter, "python", 6) == 0)
+			ft = filetypes[GEANY_FILETYPES_PYTHON];
+		else if (strncmp(basename_interpreter, "php", 3) == 0)
+			ft = filetypes[GEANY_FILETYPES_PHP];
+		else if (strncmp(basename_interpreter, "ruby", 4) == 0)
+			ft = filetypes[GEANY_FILETYPES_RUBY];
+		else if (strncmp(basename_interpreter, "tcl", 3) == 0)
+			ft = filetypes[GEANY_FILETYPES_TCL];
+		else if (strncmp(basename_interpreter, "zsh", 3) == 0)
+			ft = filetypes[GEANY_FILETYPES_SH];
+		else if (strncmp(basename_interpreter, "ksh", 3) == 0)
+			ft = filetypes[GEANY_FILETYPES_SH];
+		else if (strncmp(basename_interpreter, "csh", 3) == 0)
+			ft = filetypes[GEANY_FILETYPES_SH];
+		else if (strncmp(basename_interpreter, "dmd", 3) == 0)
+			ft = filetypes[GEANY_FILETYPES_D];
+		// what else to add?
+
+		g_free(basename_interpreter);
+	}
+
+	g_free(line);
+	return ft;
 }
 
 
 /* simple filetype selection based on the filename extension */
-filetype *filetypes_get_from_filename(const gchar *filename)
+filetype *filetypes_get_from_filename(gint idx)
 {
 	GPatternSpec *pattern;
+	filetype *ft;
+	gchar *filename;
 	gchar *base_filename, *utf8_filename;
 	gint i, j;
 
-	if (filename == NULL)
-	{
+	if (! DOC_IDX_VALID(idx))
 		return filetypes[GEANY_FILETYPES_ALL];
-	}
+
+	// try to find a shebang and if found use it prior to the filename extension
+	ft = find_shebang(idx);
+	if (ft != NULL) return ft;
+
+	if (doc_list[idx].file_name == NULL)
+		return filetypes[GEANY_FILETYPES_ALL];
+	else
+		filename = doc_list[idx].file_name;
 
 	// try to get the UTF-8 equivalent for the filename
 	utf8_filename = g_locale_to_utf8(filename, -1, NULL, NULL, NULL);
@@ -773,7 +846,7 @@ static void filetypes_create_newmenu_item(GtkWidget *menu, gchar *label, filetyp
 
 
 /* frees the array and all related pointers */
-void filetypes_free_types(void)
+void filetypes_free_types()
 {
 	gint i;
 
@@ -791,20 +864,7 @@ void filetypes_free_types(void)
 			g_free(filetypes[i]->programs->run_cmd);
 			g_free(filetypes[i]->programs->run_cmd2);
 			g_free(filetypes[i]->programs);
-			if (filetypes[i]->menu_items->menu != NULL &&
-				GTK_IS_WIDGET(filetypes[i]->menu_items->menu))
-				gtk_widget_destroy(filetypes[i]->menu_items->menu);
-			/// TODO not necessary because they got freed with the menu?
-/*			if (filetypes[i]->menu_items->item_compile != NULL &&
-				GTK_IS_WIDGET(filetypes[i]->menu_items->item_compile))
-				gtk_widget_destroy(filetypes[i]->menu_items->item_compile);
-			if (filetypes[i]->menu_items->item_link != NULL &&
-				GTK_IS_WIDGET(filetypes[i]->menu_items->item_link))
-				gtk_widget_destroy(filetypes[i]->menu_items->item_link);
-			if (filetypes[i]->menu_items->item_exec != NULL &&
-				GTK_IS_WIDGET(filetypes[i]->menu_items->item_exec))
-				gtk_widget_destroy(filetypes[i]->menu_items->item_exec);
-*/			g_free(filetypes[i]->menu_items);
+			g_free(filetypes[i]->actions);
 
 			g_strfreev(filetypes[i]->pattern);
 			g_free(filetypes[i]);
@@ -813,33 +873,6 @@ void filetypes_free_types(void)
 }
 
 
-gchar *filetypes_get_template(filetype *ft)
-{
-	if (ft == NULL) return NULL;
-
-	switch (ft->id)
-	{
-		case GEANY_FILETYPES_C:
-			return templates_get_template_generic(GEANY_TEMPLATE_FILETYPE_C); break;
-		case GEANY_FILETYPES_CPP:
-			return templates_get_template_generic(GEANY_TEMPLATE_FILETYPE_CPP); break;
-		case GEANY_FILETYPES_PHP:
-			return templates_get_template_generic(GEANY_TEMPLATE_FILETYPE_PHP); break;
-		case GEANY_FILETYPES_JAVA:
-			return templates_get_template_generic(GEANY_TEMPLATE_FILETYPE_JAVA); break;
-		case GEANY_FILETYPES_PASCAL:
-			return templates_get_template_generic(GEANY_TEMPLATE_FILETYPE_PASCAL); break;
-		case GEANY_FILETYPES_RUBY:
-			return templates_get_template_generic(GEANY_TEMPLATE_FILETYPE_RUBY); break;
-		case GEANY_FILETYPES_D:
-			return templates_get_template_generic(GEANY_TEMPLATE_FILETYPE_D); break;
-		case GEANY_FILETYPES_HTML:
-			return templates_get_template_generic(GEANY_TEMPLATE_FILETYPE_HTML); break;
-		default: return NULL;
-	}
-}
-
-
 void filetypes_get_config(GKeyFile *config, GKeyFile *configh, gint ft)
 {
 	gchar *result;
@@ -882,7 +915,7 @@ void filetypes_get_config(GKeyFile *config, GKeyFile *configh, gint ft)
 	if (result != NULL)
 	{
 		filetypes[ft]->programs->compiler = result;
-		filetypes[ft]->menu_items->can_compile = TRUE;
+		filetypes[ft]->actions->can_compile = TRUE;
 	}
 
 	result = g_key_file_get_string(configh, "build_settings", "linker", NULL);
@@ -890,7 +923,7 @@ void filetypes_get_config(GKeyFile *config, GKeyFile *configh, gint ft)
 	if (result != NULL)
 	{
 		filetypes[ft]->programs->linker = result;
-		filetypes[ft]->menu_items->can_link = TRUE;
+		filetypes[ft]->actions->can_link = TRUE;
 	}
 
 	result = g_key_file_get_string(configh, "build_settings", "run_cmd", NULL);
@@ -898,7 +931,7 @@ void filetypes_get_config(GKeyFile *config, GKeyFile *configh, gint ft)
 	if (result != NULL)
 	{
 		filetypes[ft]->programs->run_cmd = result;
-		filetypes[ft]->menu_items->can_exec = TRUE;
+		filetypes[ft]->actions->can_exec = TRUE;
 	}
 
 	result = g_key_file_get_string(configh, "build_settings", "run_cmd2", NULL);
@@ -906,11 +939,68 @@ void filetypes_get_config(GKeyFile *config, GKeyFile *configh, gint ft)
 	if (result != NULL)
 	{
 		filetypes[ft]->programs->run_cmd2 = result;
-		filetypes[ft]->menu_items->can_exec = TRUE;
+		filetypes[ft]->actions->can_exec = TRUE;
 	}
 }
 
 
+gchar *filetypes_get_conf_extension(gint filetype_idx)
+{
+	gchar *result, *tmp = g_strdup(filetypes[filetype_idx]->name);
+
+	switch (filetype_idx)
+	{
+		case GEANY_FILETYPES_ALL: result = g_strdup("common"); break;
+		case GEANY_FILETYPES_CPP: result = g_strdup("cpp"); break;
+		case GEANY_FILETYPES_MAKE: result = g_strdup("makefile"); break;
+		case GEANY_FILETYPES_OMS: result = g_strdup("oms"); break;
+		default: result = g_ascii_strdown(tmp, -1); break;
+	}
+	g_free(tmp);
+	return result;
+}
+
+
+void filetypes_save_commands()
+{
+	gchar *conf_prefix = g_strconcat(app->configdir,
+		G_DIR_SEPARATOR_S GEANY_FILEDEFS_SUBDIR G_DIR_SEPARATOR_S "filetypes.", NULL);
+	gint i;
+
+	for (i = 0; i < GEANY_FILETYPES_ALL; i++)
+	{
+		struct build_programs *bp = filetypes[i]->programs;
+		GKeyFile *config_home;
+		gchar *fname, *ext, *data;
+
+		if (! bp->modified) continue;
+
+		ext = filetypes_get_conf_extension(i);
+		fname = g_strconcat(conf_prefix, ext, NULL);
+		g_free(ext);
+
+		config_home = g_key_file_new();
+		g_key_file_load_from_file(config_home, fname, G_KEY_FILE_KEEP_COMMENTS, NULL);
+
+		if (bp->compiler && *bp->compiler)
+			g_key_file_set_string(config_home, "build_settings", "compiler", bp->compiler);
+		if (bp->linker && *bp->linker)
+			g_key_file_set_string(config_home, "build_settings", "linker", bp->linker);
+		if (bp->run_cmd && *bp->run_cmd)
+			g_key_file_set_string(config_home, "build_settings", "run_cmd", bp->run_cmd);
+		if (bp->run_cmd2 && *bp->run_cmd2)
+			g_key_file_set_string(config_home, "build_settings", "run_cmd2", bp->run_cmd2);
+
+		data = g_key_file_to_data(config_home, NULL, NULL);
+		utils_write_file(fname, data);
+		g_free(data);
+		g_key_file_free(config_home);
+		g_free(fname);
+	}
+	g_free(conf_prefix);
+}
+
+
 GtkFileFilter *filetypes_create_file_filter(filetype *ft)
 {
 	GtkFileFilter *new_filter;
