@@ -17,7 +17,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: geany.h 839 2006-09-27 12:50:29Z eht16 $
+ * $Id: geany.h 1124 2006-12-18 22:57:04Z eht16 $
  */
 
 #ifndef GEANY_H
@@ -36,13 +36,11 @@
 // listed in the documentation should not be changed ;-)
 #define GEANY_HOME_DIR					g_get_home_dir()
 #define GEANY_FILEDEFS_SUBDIR			"filedefs"
-#define GEANY_CODENAME					"Kintaro"
+#define GEANY_CODENAME					"Sander"
 #define GEANY_HOMEPAGE					"http://geany.uvena.de/"
-#define GEANY_MAX_OPEN_FILES			25
-#define GEANY_SESSION_FILES				25
-#define GEANY_MAX_TAGS_COUNT			1000
+#define GEANY_USE_WIN32_DIALOG			0
 #define GEANY_CHECK_FILE_DELAY			30
-#define GEANY_WORDCHARS					"_#&abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789\\"
+#define GEANY_WORDCHARS					"_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
 #define GEANY_MAX_WORD_LENGTH			192
 #define GEANY_MAX_AUTOCOMPLETE_WORDS	30
 #define GEANY_MAX_AUTOCOMPLETE_HEIGHT	10
@@ -90,10 +88,12 @@ typedef struct MyApp
 	gboolean			 fullscreen;
 	gboolean			 beep_on_errors;
 	gboolean			 switch_msgwin_pages;
+	gboolean			 show_notebook_tabs;
 	gboolean			 tab_order_ltr;
 	gboolean			 show_markers_margin;
 	gboolean			 show_linenumber_margin;
 	gboolean			 brace_match_ltgt;
+	gboolean			 use_tab_to_indent;
 	gboolean			 main_window_realized;
 	// I know, it is a bit confusing, but this line breaking is globally,
 	// to change the default value at startup, I think
@@ -106,6 +106,7 @@ typedef struct MyApp
 	gboolean			 pref_editor_auto_close_xml_tags;
 	gboolean			 pref_editor_auto_complete_constructs;
 	gboolean			 pref_editor_folding;
+	gboolean			 pref_editor_unfold_all_children;
 	gint				 pref_editor_tab_width;
 	gint				 pref_editor_default_encoding;
 	gboolean			 pref_editor_new_line;
@@ -114,6 +115,7 @@ typedef struct MyApp
 	gboolean			 pref_main_load_session;
 	gboolean			 pref_main_save_winpos;
 	gboolean			 pref_main_confirm_exit;
+	gboolean			 pref_main_suppress_search_dialogs;
 	gboolean			 pref_toolbar_show_search;
 	gboolean			 pref_toolbar_show_goto;
 	gboolean			 pref_toolbar_show_undo;
@@ -121,6 +123,7 @@ typedef struct MyApp
 	gboolean			 pref_toolbar_show_zoom;
 	gboolean			 pref_toolbar_show_colour;
 	gboolean			 pref_toolbar_show_fileops;
+	gboolean			 pref_toolbar_show_quit;
 	gint				 tab_pos_editor;
 	gint				 tab_pos_msgwin;
 	gint				 tab_pos_sidebar;
@@ -140,7 +143,6 @@ typedef struct MyApp
 	gchar				*configdir;
 	gchar				*datadir;
 	gchar				*docdir;
-	gchar				*build_make_custopt;
 	gchar				*custom_date_format;
 	gchar				*tools_browser_cmd;
 	gchar				*tools_make_cmd;
@@ -160,6 +162,8 @@ typedef struct MyApp
 	GtkWidget			*popup_menu;
 	GtkWidget			*toolbar_menu;
 	GtkWidget			*new_file_menu;
+	GtkWidget			*recent_files_menubar;
+	GtkWidget			*recent_files_toolbar;
 	GtkWidget			*menu_insert_include_item[2];
 	GtkWidget			*popup_goto_items[3];
 	GtkWidget			*popup_items[5];
@@ -214,4 +218,8 @@ enum {
 };
 
 
+
+// implementation in main.c; prototype is here so that all files can use it.
+void geany_debug(gchar const *format, ...) G_GNUC_PRINTF (1, 2);
+
 #endif
