@@ -1,7 +1,8 @@
 /*
  *      keybindings.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: keybindings.c 814 2006-09-12 08:32:52Z eht16 $
+ * $Id: keybindings.c 1270 2007-02-12 18:42:15Z eht16 $
  */
 
 
@@ -34,74 +35,63 @@
 #include "msgwindow.h"
 #include "sci_cb.h"
 #include "sciwrappers.h"
+#include "build.h"
 // include vte.h on non-Win32 systems, else define fake vte_init
 #ifdef HAVE_VTE
 # include "vte.h"
 #endif
 
 
+const gboolean swap_alt_tab_order = FALSE;
+
 
 /* simple convenience function to allocate and fill the struct */
-static binding *fill(void (*func) (void), guint key, GdkModifierType mod, const gchar *name, const gchar *label);
+static binding *fill(KBCallback func, guint key, GdkModifierType mod, const gchar *name,
+		const gchar *label);
 
-static void cb_func_menu_new(void);
-static void cb_func_menu_open(void);
-static void cb_func_menu_save(void);
-static void cb_func_menu_saveall(void);
-static void cb_func_menu_print(void);
-static void cb_func_menu_close(void);
-static void cb_func_menu_closeall(void);
-static void cb_func_menu_reloadfile(void);
-static void cb_func_menu_undo(void);
-static void cb_func_menu_redo(void);
-static void cb_func_menu_selectall(void);
-static void cb_func_menu_preferences(void);
-static void cb_func_menu_insert_date(void);
-static void cb_func_menu_findnext(void);
-static void cb_func_menu_findprevious(void);
-static void cb_func_menu_replace(void);
-static void cb_func_menu_findinfiles(void);
-static void cb_func_menu_gotoline(void);
-static void cb_func_menu_opencolorchooser(void);
-static void cb_func_menu_fullscreen(void);
-static void cb_func_menu_messagewindow(void);
-static void cb_func_menu_zoomin(void);
-static void cb_func_menu_zoomout(void);
-static void cb_func_menu_replacetabs(void);
-static void cb_func_menu_foldall(void);
-static void cb_func_menu_unfoldall(void);
-static void cb_func_build_compile(void);
-static void cb_func_build_link(void);
-static void cb_func_build_make(void);
-static void cb_func_build_makeowntarget(void);
-static void cb_func_build_makeobject(void);
-static void cb_func_build_run(void);
-static void cb_func_build_run2(void);
-static void cb_func_build_options(void);
-static void cb_func_reloadtaglist(void);
-static void cb_func_switch_editor(void);
-static void cb_func_switch_scribble(void);
-static void cb_func_switch_vte(void);
-static void cb_func_switch_tableft(void);
-static void cb_func_switch_tabright(void);
-static void cb_func_toggle_sidebar(void);
-static void cb_func_edit_duplicateline(void);
-static void cb_func_edit_tolowercase(void);
-static void cb_func_edit_touppercase(void);
-static void cb_func_edit_commentline(void);
-static void cb_func_edit_commentlinetoggle(void);
-static void cb_func_edit_uncommentline(void);
-static void cb_func_edit_increaseindent(void);
-static void cb_func_edit_decreaseindent(void);
-static void cb_func_edit_autocomplete(void);
-static void cb_func_edit_calltip(void);
-static void cb_func_edit_macrolist(void);
-static void cb_func_edit_suppresscompletion(void);
-static void cb_func_popup_findusage(void);
-static void cb_func_popup_gototagdefinition(void);
-static void cb_func_popup_gototagdeclaration(void);
+static void cb_func_menu_new(guint key_id);
+static void cb_func_menu_open(guint key_id);
+static void cb_func_menu_save(guint key_id);
+static void cb_func_menu_saveall(guint key_id);
+static void cb_func_menu_print(guint key_id);
+static void cb_func_menu_close(guint key_id);
+static void cb_func_menu_closeall(guint key_id);
+static void cb_func_menu_reloadfile(guint key_id);
+static void cb_func_menu_undo(guint key_id);
+static void cb_func_menu_redo(guint key_id);
+static void cb_func_menu_selectall(guint key_id);
+static void cb_func_menu_preferences(guint key_id);
+static void cb_func_menu_insert_date(guint key_id);
+static void cb_func_menu_findnext(guint key_id);
+static void cb_func_menu_findprevious(guint key_id);
+static void cb_func_menu_replace(guint key_id);
+static void cb_func_menu_findinfiles(guint key_id);
+static void cb_func_menu_nextmessage(guint key_id);
+static void cb_func_menu_gotoline(guint key_id);
+static void cb_func_menu_opencolorchooser(guint key_id);
+static void cb_func_menu_fullscreen(guint key_id);
+static void cb_func_menu_messagewindow(guint key_id);
+static void cb_func_menu_zoomin(guint key_id);
+static void cb_func_menu_zoomout(guint key_id);
+static void cb_func_menu_replacetabs(guint key_id);
+static void cb_func_menu_foldall(guint key_id);
+static void cb_func_menu_unfoldall(guint key_id);
+static void cb_func_menu_insert_specialchars(guint key_id);
+static void cb_func_build_action(guint key_id);
+static void cb_func_reloadtaglist(guint key_id);
+static void cb_func_switch_editor(guint key_id);
+static void cb_func_switch_scribble(guint key_id);
+static void cb_func_switch_vte(guint key_id);
+static void cb_func_switch_tableft(guint key_id);
+static void cb_func_switch_tabright(guint key_id);
+static void cb_func_toggle_sidebar(guint key_id);
+
+// common function for editing keybindings, only valid when scintilla has focus.
+static void cb_func_edit(guint key_id);
+
+// common function for keybindings using current word
+static void cb_func_current_word(guint key_id);
 
-static void keybindings_call_popup_item(int menuitemkey);
 static void keybindings_add_accels();
 
 
@@ -124,11 +114,11 @@ void keybindings_init(void)
 	keys[GEANY_KEYS_MENU_SAVEALL] = fill(cb_func_menu_saveall,
 		GDK_S, GDK_SHIFT_MASK | GDK_CONTROL_MASK, "menu_saveall", _("Save all"));
 	keys[GEANY_KEYS_MENU_PRINT] = fill(cb_func_menu_print,
-		GDK_p, GDK_SHIFT_MASK | GDK_CONTROL_MASK, "menu_print", _("Print"));
+		GDK_p, GDK_CONTROL_MASK, "menu_print", _("Print"));
 	keys[GEANY_KEYS_MENU_CLOSE] = fill(cb_func_menu_close,
 		GDK_w, GDK_CONTROL_MASK, "menu_close", _("Close"));
 	keys[GEANY_KEYS_MENU_CLOSEALL] = fill(cb_func_menu_closeall,
-		GDK_d, GDK_MOD1_MASK, "menu_closeall", _("Close all"));
+		GDK_w, GDK_CONTROL_MASK | GDK_SHIFT_MASK, "menu_closeall", _("Close all"));
 	keys[GEANY_KEYS_MENU_RELOADFILE] = fill(cb_func_menu_reloadfile,
 		GDK_r, GDK_CONTROL_MASK, "menu_reloadfile", _("Reload file"));
 	keys[GEANY_KEYS_MENU_UNDO] = fill(cb_func_menu_undo,
@@ -149,6 +139,8 @@ void keybindings_init(void)
 		GDK_e, GDK_CONTROL_MASK, "menu_replace", _("Replace"));
 	keys[GEANY_KEYS_MENU_FINDINFILES] = fill(cb_func_menu_findinfiles, GDK_f,
 		GDK_CONTROL_MASK | GDK_SHIFT_MASK, "menu_findinfiles", _("Find in files"));
+	keys[GEANY_KEYS_MENU_NEXTMESSAGE] = fill(cb_func_menu_nextmessage,
+		0, 0, "menu_nextmessage", _("Next Message"));
 	keys[GEANY_KEYS_MENU_GOTOLINE] = fill(cb_func_menu_gotoline,
 		GDK_l, GDK_CONTROL_MASK, "menu_gotoline", _("Go to line"));
 	keys[GEANY_KEYS_MENU_OPENCOLORCHOOSER] = fill(cb_func_menu_opencolorchooser,
@@ -169,22 +161,26 @@ void keybindings_init(void)
 		0, 0, "menu_foldall", _("Fold all"));
 	keys[GEANY_KEYS_MENU_UNFOLDALL] = fill(cb_func_menu_unfoldall,
 		0, 0, "menu_unfoldall", _("Unfold all"));
-	keys[GEANY_KEYS_BUILD_COMPILE] = fill(cb_func_build_compile,
+	keys[GEANY_KEYS_MENU_INSERTSPECIALCHARS] = fill(cb_func_menu_insert_specialchars,
+		0, 0, "menu_insert_specialchars", _("Insert Special HTML Characters"));
+	keys[GEANY_KEYS_BUILD_COMPILE] = fill(cb_func_build_action,
 		GDK_F8, 0, "build_compile", _("Compile"));
-	keys[GEANY_KEYS_BUILD_LINK] = fill(cb_func_build_link,
+	keys[GEANY_KEYS_BUILD_LINK] = fill(cb_func_build_action,
 		GDK_F9, 0, "build_link", _("Build"));
-	keys[GEANY_KEYS_BUILD_MAKE] = fill(cb_func_build_make,
+	keys[GEANY_KEYS_BUILD_MAKE] = fill(cb_func_build_action,
 		GDK_F9, GDK_SHIFT_MASK, "build_make", _("Make all"));
-	keys[GEANY_KEYS_BUILD_MAKEOWNTARGET] = fill(cb_func_build_makeowntarget,
+	keys[GEANY_KEYS_BUILD_MAKEOWNTARGET] = fill(cb_func_build_action,
 		GDK_F9, GDK_SHIFT_MASK | GDK_CONTROL_MASK, "build_makeowntarget",
 		_("Make custom target"));
-	keys[GEANY_KEYS_BUILD_MAKEOBJECT] = fill(cb_func_build_makeobject,
+	keys[GEANY_KEYS_BUILD_MAKEOBJECT] = fill(cb_func_build_action,
 		0, 0, "build_makeobject", _("Make object"));
-	keys[GEANY_KEYS_BUILD_RUN] = fill(cb_func_build_run,
+	keys[GEANY_KEYS_BUILD_NEXTERROR] = fill(cb_func_build_action,
+		0, 0, "build_nexterror", _("Next error"));
+	keys[GEANY_KEYS_BUILD_RUN] = fill(cb_func_build_action,
 		GDK_F5, 0, "build_run", _("Run"));
-	keys[GEANY_KEYS_BUILD_RUN2] = fill(cb_func_build_run2,
+	keys[GEANY_KEYS_BUILD_RUN2] = fill(cb_func_build_action,
 		0, 0, "build_run2", _("Run (alternative command)"));
-	keys[GEANY_KEYS_BUILD_OPTIONS] = fill(cb_func_build_options,
+	keys[GEANY_KEYS_BUILD_OPTIONS] = fill(cb_func_build_action,
 		0, 0, "build_options", _("Build options"));
 	keys[GEANY_KEYS_RELOADTAGLIST] = fill(cb_func_reloadtaglist,
 		GDK_r, GDK_SHIFT_MASK | GDK_CONTROL_MASK, "reloadtaglist", _("Reload symbol list"));
@@ -198,41 +194,46 @@ void keybindings_init(void)
 		GDK_Page_Up, GDK_CONTROL_MASK, "switch_tableft", _("Switch to left document"));
 	keys[GEANY_KEYS_SWITCH_TABRIGHT] = fill(cb_func_switch_tabright,
 		GDK_Page_Down, GDK_CONTROL_MASK, "switch_tabright", _("Switch to right document"));
-	keys[GEANY_KEYS_EDIT_DUPLICATELINE] = fill(cb_func_edit_duplicateline,
+
+	keys[GEANY_KEYS_EDIT_DUPLICATELINE] = fill(cb_func_edit,
 		GDK_g, GDK_CONTROL_MASK, "edit_duplicateline", _("Duplicate line or selection"));
-	keys[GEANY_KEYS_EDIT_TOLOWERCASE] = fill(cb_func_edit_tolowercase,
+	keys[GEANY_KEYS_EDIT_TOLOWERCASE] = fill(cb_func_edit,
 		GDK_u, GDK_CONTROL_MASK, "edit_tolowercase", _("Convert Selection to lower-case"));
-	keys[GEANY_KEYS_EDIT_TOUPPERCASE] = fill(cb_func_edit_touppercase,
+	keys[GEANY_KEYS_EDIT_TOUPPERCASE] = fill(cb_func_edit,
 		GDK_u, GDK_SHIFT_MASK | GDK_CONTROL_MASK, "edit_touppercase", _("Convert Selection to upper-case"));
-	keys[GEANY_KEYS_EDIT_COMMENTLINETOGGLE] = fill(cb_func_edit_commentlinetoggle,
+	keys[GEANY_KEYS_EDIT_COMMENTLINETOGGLE] = fill(cb_func_edit,
 		GDK_b, GDK_CONTROL_MASK, "edit_commentlinetoggle", _("Toggle line commentation"));
-	keys[GEANY_KEYS_EDIT_COMMENTLINE] = fill(cb_func_edit_commentline,
-		GDK_d, GDK_CONTROL_MASK, "edit_commentline", _("Comment line"));
-	keys[GEANY_KEYS_EDIT_UNCOMMENTLINE] = fill(cb_func_edit_uncommentline,
-		GDK_d, GDK_SHIFT_MASK | GDK_CONTROL_MASK, "edit_uncommentline", _("Uncomment line"));
-	keys[GEANY_KEYS_EDIT_INCREASEINDENT] = fill(cb_func_edit_increaseindent,
+	keys[GEANY_KEYS_EDIT_COMMENTLINE] = fill(cb_func_edit,
+		GDK_d, GDK_CONTROL_MASK, "edit_commentline", _("Comment line(s)"));
+	keys[GEANY_KEYS_EDIT_UNCOMMENTLINE] = fill(cb_func_edit,
+		GDK_d, GDK_SHIFT_MASK | GDK_CONTROL_MASK, "edit_uncommentline", _("Uncomment line(s)"));
+	keys[GEANY_KEYS_EDIT_INCREASEINDENT] = fill(cb_func_edit,
 		GDK_i, GDK_CONTROL_MASK, "edit_increaseindent", _("Increase indent"));
-	keys[GEANY_KEYS_EDIT_DECREASEINDENT] = fill(cb_func_edit_decreaseindent,
+	keys[GEANY_KEYS_EDIT_DECREASEINDENT] = fill(cb_func_edit,
 		GDK_i, GDK_SHIFT_MASK | GDK_CONTROL_MASK, "edit_decreaseindent", _("Decrease indent"));
-	keys[GEANY_KEYS_EDIT_AUTOCOMPLETE] = fill(cb_func_edit_autocomplete,
+	keys[GEANY_KEYS_EDIT_GOTOMATCHINGBRACE] = fill(cb_func_edit,
+		GDK_less, GDK_SHIFT_MASK | GDK_CONTROL_MASK, "edit_gotomatchingbrace",
+		_("Goto matching brace"));
+	keys[GEANY_KEYS_EDIT_AUTOCOMPLETE] = fill(cb_func_edit,
 		GDK_space, GDK_CONTROL_MASK, "edit_autocomplete", _("Complete word"));
 #ifdef G_OS_WIN32
 	// on windows alt-space is taken by the window manager
-	keys[GEANY_KEYS_EDIT_CALLTIP] = fill(cb_func_edit_calltip,
+	keys[GEANY_KEYS_EDIT_CALLTIP] = fill(cb_func_edit,
 		GDK_space, GDK_CONTROL_MASK | GDK_SHIFT_MASK, "edit_calltip", _("Show calltip"));
 #else
-	keys[GEANY_KEYS_EDIT_CALLTIP] = fill(cb_func_edit_calltip,
+	keys[GEANY_KEYS_EDIT_CALLTIP] = fill(cb_func_edit,
 		GDK_space, GDK_MOD1_MASK, "edit_calltip", _("Show calltip"));
 #endif
-	keys[GEANY_KEYS_EDIT_MACROLIST] = fill(cb_func_edit_macrolist,
+	keys[GEANY_KEYS_EDIT_MACROLIST] = fill(cb_func_edit,
 		GDK_Return, GDK_CONTROL_MASK, "edit_macrolist", _("Show macro list"));
-	keys[GEANY_KEYS_EDIT_SUPPRESSCOMPLETION] = fill(cb_func_edit_suppresscompletion,
+	keys[GEANY_KEYS_EDIT_SUPPRESSCOMPLETION] = fill(cb_func_edit,
 		GDK_space, GDK_SHIFT_MASK, "edit_suppresscompletion", _("Suppress auto completion"));
-	keys[GEANY_KEYS_POPUP_FINDUSAGE] = fill(cb_func_popup_findusage,
+
+	keys[GEANY_KEYS_POPUP_FINDUSAGE] = fill(cb_func_current_word,
 		0, 0, "popup_findusage", _("Find Usage"));
-	keys[GEANY_KEYS_POPUP_GOTOTAGDEFINITION] = fill(cb_func_popup_gototagdefinition,
+	keys[GEANY_KEYS_POPUP_GOTOTAGDEFINITION] = fill(cb_func_current_word,
 		0, 0, "popup_gototagdefinition", _("Go to tag definition"));
-	keys[GEANY_KEYS_POPUP_GOTOTAGDECLARATION] = fill(cb_func_popup_gototagdeclaration,
+	keys[GEANY_KEYS_POPUP_GOTOTAGDECLARATION] = fill(cb_func_current_word,
 		0, 0, "popup_gototagdeclaration", _("Go to tag declaration"));
 
 	// now load user defined keys
@@ -291,6 +292,7 @@ static void keybindings_add_accels()
 	GEANY_ADD_ACCEL(GEANY_KEYS_MENU_FINDPREVIOUS, find_previous1);
 	GEANY_ADD_ACCEL(GEANY_KEYS_MENU_REPLACE, replace1);
 	GEANY_ADD_ACCEL(GEANY_KEYS_MENU_FINDINFILES, find_in_files1);
+	GEANY_ADD_ACCEL(GEANY_KEYS_MENU_NEXTMESSAGE, next_message1);
 	GEANY_ADD_ACCEL(GEANY_KEYS_MENU_GOTOLINE, go_to_line1);
 	GEANY_ADD_ACCEL(GEANY_KEYS_MENU_FULLSCREEN, menu_fullscreen1);
 	GEANY_ADD_ACCEL(GEANY_KEYS_MENU_MESSAGEWINDOW, menu_show_messages_window1);
@@ -300,6 +302,7 @@ static void keybindings_add_accels()
 	GEANY_ADD_ACCEL(GEANY_KEYS_MENU_REPLACETABS, menu_replace_tabs);
 	GEANY_ADD_ACCEL(GEANY_KEYS_MENU_FOLDALL, menu_fold_all1);
 	GEANY_ADD_ACCEL(GEANY_KEYS_MENU_UNFOLDALL, menu_unfold_all1);
+	GEANY_ADD_ACCEL(GEANY_KEYS_MENU_INSERTSPECIALCHARS, menu_insert_special_chars1);
 	GEANY_ADD_ACCEL(GEANY_KEYS_EDIT_TOLOWERCASE, menu_to_lower_case2);
 	GEANY_ADD_ACCEL(GEANY_KEYS_EDIT_TOUPPERCASE, menu_to_upper_case2);
 	GEANY_ADD_ACCEL(GEANY_KEYS_EDIT_COMMENTLINE, menu_comment_line1);
@@ -376,11 +379,35 @@ void keybindings_free(void)
 }
 
 
+static gboolean check_fixed_kb(GdkEventKey *event)
+{
+	// check alt-0 to alt-9 for setting current notebook page
+	if (event->state & GDK_MOD1_MASK && event->keyval >= GDK_0 && event->keyval <= GDK_9)
+	{
+		gint page = event->keyval - GDK_0 - 1;
+		gint npages = gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook));
+
+		// alt-0 is for the rightmost tab
+		if (event->keyval == GDK_0)
+			page = npages - 1;
+		// invert the order if tabs are added on the other side
+		if (swap_alt_tab_order && ! app->tab_order_ltr)
+			page = (npages - 1) - page;
+
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(app->notebook), page);
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
 /* central keypress event handler, almost all keypress events go to this function */
 gboolean keybindings_got_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
 {
 	guint i, k;
 
+	if (check_fixed_kb(event)) return TRUE;
+
 	for (i = 0; i < GEANY_MAX_KEYS; i++)
 	{
 		// ugly hack to get around that CTRL+Shift+r results in 'R' not 'r'
@@ -399,7 +426,7 @@ gboolean keybindings_got_event(GtkWidget *widget, GdkEventKey *event, gpointer u
 		if (event->keyval == k && event->state == keys[i]->mods)
 		{
 			// call the corresponding callback function for this shortcut
-			if (keys[i]->cb_func != NULL) keys[i]->cb_func();
+			if (keys[i]->cb_func != NULL) keys[i]->cb_func(i);
 			return TRUE;
 		}
 	}
@@ -408,7 +435,8 @@ gboolean keybindings_got_event(GtkWidget *widget, GdkEventKey *event, gpointer u
 
 
 /* simple convenience function to allocate and fill the struct */
-static binding *fill(void (*func) (void), guint key, GdkModifierType mod, const gchar *name, const gchar *label)
+static binding *fill(KBCallback func, guint key, GdkModifierType mod, const gchar *name,
+		const gchar *label)
 {
 	binding *result;
 
@@ -424,197 +452,193 @@ static binding *fill(void (*func) (void), guint key, GdkModifierType mod, const
 
 
 /* these are the callback functions, each shortcut has its own function, this is only for clear code */
-static void cb_func_menu_new(void)
+static void cb_func_menu_new(G_GNUC_UNUSED guint key_id)
 {
 	document_new_file(NULL);
 }
 
-static void cb_func_menu_open(void)
+static void cb_func_menu_open(G_GNUC_UNUSED guint key_id)
 {
 	on_open1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_save(void)
+static void cb_func_menu_save(G_GNUC_UNUSED guint key_id)
 {
 	on_save1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_saveall(void)
+static void cb_func_menu_saveall(G_GNUC_UNUSED guint key_id)
 {
 	on_save_all1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_close(void)
+static void cb_func_menu_close(G_GNUC_UNUSED guint key_id)
 {
 	on_close1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_closeall(void)
+static void cb_func_menu_closeall(G_GNUC_UNUSED guint key_id)
 {
 	on_close_all1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_reloadfile(void)
+static void cb_func_menu_reloadfile(G_GNUC_UNUSED guint key_id)
 {
 	on_toolbutton23_clicked(NULL, NULL);
 }
 
-static void cb_func_menu_undo(void)
+static void cb_func_menu_undo(G_GNUC_UNUSED guint key_id)
 {
 	on_undo1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_redo(void)
+static void cb_func_menu_redo(G_GNUC_UNUSED guint key_id)
 {
 	on_redo1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_selectall(void)
+static void cb_func_menu_selectall(G_GNUC_UNUSED guint key_id)
 {
 	on_menu_select_all1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_preferences(void)
+static void cb_func_menu_preferences(G_GNUC_UNUSED guint key_id)
 {
 	on_preferences1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_findnext(void)
+static void cb_func_menu_findnext(G_GNUC_UNUSED guint key_id)
 {
 	on_find_next1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_findprevious(void)
+static void cb_func_menu_findprevious(G_GNUC_UNUSED guint key_id)
 {
 	on_find_previous1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_replace(void)
+static void cb_func_menu_replace(G_GNUC_UNUSED guint key_id)
 {
 	on_replace1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_findinfiles(void)
+static void cb_func_menu_findinfiles(G_GNUC_UNUSED guint key_id)
 {
 	on_find_in_files1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_gotoline(void)
+static void cb_func_menu_nextmessage(guint key_id)
+{
+	on_next_message1_activate(NULL, NULL);
+}
+
+static void cb_func_menu_gotoline(G_GNUC_UNUSED guint key_id)
 {
 	on_go_to_line1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_opencolorchooser(void)
+static void cb_func_menu_opencolorchooser(G_GNUC_UNUSED guint key_id)
 {
 	on_show_color_chooser1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_fullscreen(void)
+static void cb_func_menu_fullscreen(G_GNUC_UNUSED guint key_id)
 {
 	GtkCheckMenuItem *c = GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_fullscreen1"));
 
 	gtk_check_menu_item_set_active(c, ! gtk_check_menu_item_get_active(c));
 }
 
-static void cb_func_menu_messagewindow(void)
+static void cb_func_menu_messagewindow(G_GNUC_UNUSED guint key_id)
 {
 	GtkCheckMenuItem *c = GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_show_messages_window1"));
 
 	gtk_check_menu_item_set_active(c, ! gtk_check_menu_item_get_active(c));
 }
 
-static void cb_func_menu_zoomin(void)
+static void cb_func_menu_zoomin(G_GNUC_UNUSED guint key_id)
 {
 	on_zoom_in1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_zoomout(void)
+static void cb_func_menu_zoomout(G_GNUC_UNUSED guint key_id)
 {
 	on_zoom_out1_activate(NULL, NULL);
 }
 
-static void cb_func_menu_foldall(void)
+static void cb_func_menu_foldall(G_GNUC_UNUSED guint key_id)
 {
 	gint idx = document_get_cur_idx();
 	if (idx == -1 || ! doc_list[idx].is_valid) return;
 	document_fold_all(idx);
 }
 
-static void cb_func_menu_unfoldall(void)
+static void cb_func_menu_unfoldall(G_GNUC_UNUSED guint key_id)
 {
 	gint idx = document_get_cur_idx();
 	if (idx == -1 || ! doc_list[idx].is_valid) return;
 	document_unfold_all(idx);
 }
 
-static void cb_func_build_compile(void)
+static void cb_func_build_action(guint key_id)
 {
 	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-	if (doc_list[idx].file_type->menu_items->can_compile && doc_list[idx].file_name != NULL)
-		on_build_compile_activate(NULL, NULL);
+	GtkWidget *item;
+	filetype *ft;
+	BuildMenuItems *menu_items;
+
+	if (! DOC_IDX_VALID(idx)) return;
+
+	ft = doc_list[idx].file_type;
+	if (! ft) return;
+	menu_items = build_get_menu_items(ft->id);
+
+	switch (key_id)
+	{
+		case GEANY_KEYS_BUILD_COMPILE:
+		item = menu_items->item_compile;
+		break;
+
+		case GEANY_KEYS_BUILD_LINK:
+		item = menu_items->item_link;
+		break;
+
+		case GEANY_KEYS_BUILD_MAKE:
+		item = menu_items->item_make_all;
+		break;
+
+		case GEANY_KEYS_BUILD_MAKEOWNTARGET:
+		item = menu_items->item_make_custom;
+		break;
+
+		case GEANY_KEYS_BUILD_MAKEOBJECT:
+		item = menu_items->item_make_object;
+		break;
+
+		case GEANY_KEYS_BUILD_NEXTERROR:
+		item = menu_items->item_next_error;
+		break;
+
+		case GEANY_KEYS_BUILD_RUN:
+		item = menu_items->item_exec;
+		break;
+
+		case GEANY_KEYS_BUILD_RUN2:
+		item = menu_items->item_exec2;
+		break;
+
+		case GEANY_KEYS_BUILD_OPTIONS:
+		item = menu_items->item_set_args;
+		break;
+
+		default:
+		item = NULL;
+	}
+	if (item && GTK_WIDGET_IS_SENSITIVE(item))
+		gtk_menu_item_activate(GTK_MENU_ITEM(item));
 }
 
-static void cb_func_build_link(void)
-{
-	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-	if (doc_list[idx].file_type->menu_items->can_link && doc_list[idx].file_name != NULL)
-		on_build_build_activate(NULL, NULL);
-}
-
-static void cb_func_build_make(void)
-{
-	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-	if (doc_list[idx].file_name != NULL)
-		on_build_make_activate(NULL, GINT_TO_POINTER(0));
-}
-
-static void cb_func_build_makeowntarget(void)
-{
-	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-	if (doc_list[idx].file_name != NULL)
-		on_build_make_activate(NULL, GINT_TO_POINTER(1));
-}
-
-static void cb_func_build_makeobject(void)
-{
-	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-	if (doc_list[idx].file_name != NULL)
-		on_build_make_activate(NULL, GINT_TO_POINTER(2));
-}
-
-static void cb_func_build_run(void)
-{
-	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-	if (doc_list[idx].file_type->menu_items->can_exec && doc_list[idx].file_name != NULL)
-		on_build_execute_activate(NULL, GINT_TO_POINTER(0));
-}
-
-static void cb_func_build_run2(void)
-{
-	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-	if (doc_list[idx].file_type->menu_items->can_exec && doc_list[idx].file_name != NULL)
-		on_build_execute_activate(NULL, GINT_TO_POINTER(1));
-}
-
-static void cb_func_build_options(void)
-{
-	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-	if ((doc_list[idx].file_type->menu_items->can_compile ||
-		doc_list[idx].file_type->menu_items->can_link ||
-		doc_list[idx].file_type->menu_items->can_exec) &&
-		doc_list[idx].file_name != NULL)
-		on_build_arguments_activate(NULL, NULL);
-}
-
-static void cb_func_reloadtaglist(void)
+static void cb_func_reloadtaglist(G_GNUC_UNUSED guint key_id)
 {
 	gint idx = document_get_cur_idx();
 	if (idx == -1 || ! doc_list[idx].is_valid) return;
@@ -622,25 +646,8 @@ static void cb_func_reloadtaglist(void)
 }
 
 
-static void cb_func_popup_findusage(void)
-{
-	keybindings_call_popup_item(GEANY_KEYS_POPUP_FINDUSAGE);
-}
-
-
-static void cb_func_popup_gototagdefinition(void)
-{
-	keybindings_call_popup_item(GEANY_KEYS_POPUP_GOTOTAGDEFINITION);
-}
-
-
-static void cb_func_popup_gototagdeclaration(void)
-{
-	keybindings_call_popup_item(GEANY_KEYS_POPUP_GOTOTAGDECLARATION);
-}
-
-
-static void keybindings_call_popup_item(int menuitemkey)
+// common function for keybindings using current word
+static void cb_func_current_word(guint key_id)
 {
 	gint idx = document_get_cur_idx();
 	gint pos;
@@ -655,7 +662,7 @@ static void keybindings_call_popup_item(int menuitemkey)
 	if (*editor_info.current_word == 0)
 		utils_beep();
 	else
-		switch (menuitemkey)
+		switch (key_id)
 		{
 			case GEANY_KEYS_POPUP_FINDUSAGE:
 			on_find_usage1_activate(NULL, NULL);
@@ -672,38 +679,43 @@ static void keybindings_call_popup_item(int menuitemkey)
 }
 
 
-static void cb_func_switch_editor(void)
+static void cb_func_switch_editor(G_GNUC_UNUSED guint key_id)
 {
 	gint idx = document_get_cur_idx();
 	if (idx == -1 || ! doc_list[idx].is_valid) return;
 	gtk_widget_grab_focus(GTK_WIDGET(doc_list[idx].sci));
 }
 
-static void cb_func_switch_scribble(void)
+static void cb_func_switch_scribble(G_GNUC_UNUSED guint key_id)
 {
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_SCRATCH);
+	msgwin_show_hide(TRUE);
 	gtk_widget_grab_focus(lookup_widget(app->window, "textview_scribble"));
 }
 
-static void cb_func_switch_vte(void)
+static void cb_func_switch_vte(G_GNUC_UNUSED guint key_id)
 {
 #ifdef HAVE_VTE
+	msgwin_show_hide(TRUE);
+	/* the msgwin must be visible before we switch to the VTE page so that
+	 * the font settings are applied on realization */
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_VTE);
 	gtk_widget_grab_focus(vc->vte);
+	msgwin_show_hide(TRUE);
 #endif
 }
 
-static void cb_func_switch_tableft(void)
+static void cb_func_switch_tableft(G_GNUC_UNUSED guint key_id)
 {
 	utils_switch_document(LEFT);
 }
 
-static void cb_func_switch_tabright(void)
+static void cb_func_switch_tabright(G_GNUC_UNUSED guint key_id)
 {
 	utils_switch_document(RIGHT);
 }
 
-static void cb_func_toggle_sidebar(void)
+static void cb_func_toggle_sidebar(G_GNUC_UNUSED guint key_id)
 {
 	static gint active_page = -1;
 
@@ -726,87 +738,98 @@ static void cb_func_toggle_sidebar(void)
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(app->treeview_notebook), active_page);
 }
 
-static void cb_func_edit_duplicateline(void)
+
+static void goto_matching_brace(gint idx)
 {
-	on_menu_duplicate_line1_activate(NULL, NULL);
+	gint pos, new_pos;
+
+	if (! DOC_IDX_VALID(idx)) return;
+
+	pos = sci_get_current_position(doc_list[idx].sci);
+	if (! utils_isbrace(sci_get_char_at(doc_list[idx].sci, pos)))
+		pos--; // set pos to the brace
+
+	new_pos = sci_find_bracematch(doc_list[idx].sci, pos);
+	if (new_pos != -1)
+	{
+		sci_goto_pos(doc_list[idx].sci, new_pos, TRUE); // set the cursor at the brace
+		doc_list[idx].scroll_percent = 0.5F;
+	}
 }
 
-static void cb_func_edit_commentlinetoggle(void)
-{
-	on_menu_toggle_line_commentation1_activate(NULL, NULL);
-}
 
-static void cb_func_edit_commentline(void)
-{
-	on_menu_comment_line1_activate(NULL, NULL);
-}
-
-static void cb_func_edit_uncommentline(void)
-{
-	on_menu_uncomment_line1_activate(NULL, NULL);
-}
-
-static void cb_func_edit_autocomplete(void)
+// common function for editing keybindings, only valid when scintilla has focus.
+static void cb_func_edit(guint key_id)
 {
 	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-	sci_cb_start_auto_complete(idx, sci_get_current_position(doc_list[idx].sci), TRUE);
+	GtkWidget *focusw = gtk_window_get_focus(GTK_WINDOW(app->window));
+
+	// edit keybindings only valid when scintilla widget has focus
+	if (! DOC_IDX_VALID(idx) || focusw != GTK_WIDGET(doc_list[idx].sci)) return;
+
+	switch (key_id)
+	{
+		case GEANY_KEYS_EDIT_DUPLICATELINE:
+			on_menu_duplicate_line1_activate(NULL, NULL);
+			break;
+		case GEANY_KEYS_EDIT_COMMENTLINETOGGLE:
+			on_menu_toggle_line_commentation1_activate(NULL, NULL);
+			break;
+		case GEANY_KEYS_EDIT_COMMENTLINE:
+			on_menu_comment_line1_activate(NULL, NULL);
+			break;
+		case GEANY_KEYS_EDIT_UNCOMMENTLINE:
+			on_menu_uncomment_line1_activate(NULL, NULL);
+			break;
+		case GEANY_KEYS_EDIT_AUTOCOMPLETE:
+			sci_cb_start_auto_complete(idx, sci_get_current_position(doc_list[idx].sci), TRUE);
+			break;
+		case GEANY_KEYS_EDIT_CALLTIP:
+			sci_cb_show_calltip(idx, -1);
+			break;
+		case GEANY_KEYS_EDIT_MACROLIST:
+			sci_cb_show_macro_list(doc_list[idx].sci);
+			break;
+		case GEANY_KEYS_EDIT_SUPPRESSCOMPLETION:
+			sci_add_text(doc_list[idx].sci, " ");
+			break;
+		case GEANY_KEYS_EDIT_INCREASEINDENT:
+			on_menu_increase_indent1_activate(NULL, NULL);
+			break;
+		case GEANY_KEYS_EDIT_DECREASEINDENT:
+			on_menu_decrease_indent1_activate(NULL, NULL);
+			break;
+		case GEANY_KEYS_EDIT_GOTOMATCHINGBRACE:
+			goto_matching_brace(idx);
+			break;
+		case GEANY_KEYS_EDIT_TOLOWERCASE:
+			on_to_lower_case1_activate(NULL, NULL);
+			break;
+		case GEANY_KEYS_EDIT_TOUPPERCASE:
+			on_to_upper_case1_activate(NULL, NULL);
+			break;
+	}
 }
 
-static void cb_func_edit_calltip(void)
-{
-	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-	sci_cb_show_calltip(idx, -1);
-}
 
-static void cb_func_edit_macrolist(void)
-{
-	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-	sci_cb_show_macro_list(doc_list[idx].sci);
-}
-
-static void cb_func_edit_suppresscompletion(void)
-{
-	gint idx = document_get_cur_idx();
-	if (idx == -1 || ! doc_list[idx].is_valid) return;
-	sci_add_text(doc_list[idx].sci, " ");
-}
-
-static void cb_func_menu_replacetabs(void)
+static void cb_func_menu_replacetabs(G_GNUC_UNUSED guint key_id)
 {
 	on_replace_tabs_activate(NULL, NULL);
 }
 
-static void cb_func_menu_print(void)
+static void cb_func_menu_print(G_GNUC_UNUSED guint key_id)
 {
 	gint idx = document_get_cur_idx();
 	if (idx == -1 || ! doc_list[idx].is_valid) return;
 	document_print(idx);
 }
 
-static void cb_func_edit_increaseindent(void)
-{
-	on_menu_increase_indent1_activate(NULL, NULL);
-}
-
-static void cb_func_edit_decreaseindent(void)
-{
-	on_menu_decrease_indent1_activate(NULL, NULL);
-}
-
-static void cb_func_edit_tolowercase(void)
-{
-	on_to_lower_case1_activate(NULL, NULL);
-}
-
-static void cb_func_edit_touppercase(void)
-{
-	on_to_upper_case1_activate(NULL, NULL);
-}
-
-static void cb_func_menu_insert_date(void)
+static void cb_func_menu_insert_date(G_GNUC_UNUSED guint key_id)
 {
 	gtk_menu_item_activate(GTK_MENU_ITEM(lookup_widget(app->popup_menu, "insert_date_custom2")));
 }
+
+static void cb_func_menu_insert_specialchars(G_GNUC_UNUSED guint key_id)
+{
+	on_menu_insert_special_chars1_activate(NULL, NULL);
+}
