@@ -1,7 +1,8 @@
 /*
  *      keyfile.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: keyfile.c 790 2006-09-05 14:24:47Z ntrel $
+ * $Id: keyfile.c 1270 2007-02-12 18:42:15Z eht16 $
  */
 
 #include <stdlib.h>
@@ -40,27 +41,25 @@
 #include "encodings.h"
 #include "vte.h"
 #include "main.h"
+#include "msgwindow.h"
 
 
 static gchar *scribble_text = NULL;
-static gchar *session_files[GEANY_SESSION_FILES];
+static GPtrArray *session_files = NULL;
 static gint hpan_position;
 static gint vpan_position;
 
 
-static void generate_filetype_extensions(const gchar *output_dir);
-
-
 void configuration_save()
 {
 	guint i = 0, j = 0, max;
 	gint idx;
 	gboolean config_exists;
+	gboolean have_session_files;
 	GKeyFile *config = g_key_file_new();
 	gchar *configfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "geany.conf", NULL);
-	gchar *data;
+	gchar *data, *tmp;
 	gchar *entry = g_malloc(14);
-	gchar *fname = g_malloc0(256);
 	gchar **recent_files = g_new0(gchar*, app->mru_length + 1);
 	GtkTextBuffer *buffer;
 	GtkTextIter start, end;
@@ -102,7 +101,9 @@ void configuration_save()
 	g_key_file_set_boolean(config, PACKAGE, "sidebar_visible", app->sidebar_visible);
 	g_key_file_set_boolean(config, PACKAGE, "msgwindow_visible", app->msgwindow_visible);
 	g_key_file_set_boolean(config, PACKAGE, "use_folding", app->pref_editor_folding);
+	g_key_file_set_boolean(config, PACKAGE, "unfold_all_children", app->pref_editor_unfold_all_children);
 	g_key_file_set_boolean(config, PACKAGE, "use_auto_indention", app->pref_editor_use_auto_indention);
+	g_key_file_set_boolean(config, PACKAGE, "use_tab_to_indent", app->use_tab_to_indent);
 	g_key_file_set_boolean(config, PACKAGE, "use_indicators", app->pref_editor_use_indicators);
 	g_key_file_set_boolean(config, PACKAGE, "show_indent_guide", app->pref_editor_show_indent_guide);
 	g_key_file_set_boolean(config, PACKAGE, "show_white_space", app->pref_editor_show_white_space);
@@ -112,13 +113,14 @@ void configuration_save()
 	g_key_file_set_boolean(config, PACKAGE, "show_line_endings", app->pref_editor_show_line_endings);
 	g_key_file_set_boolean(config, PACKAGE, "fullscreen", app->fullscreen);
 	g_key_file_set_boolean(config, PACKAGE, "tab_order_ltr", app->tab_order_ltr);
+	g_key_file_set_boolean(config, PACKAGE, "show_notebook_tabs", app->show_notebook_tabs);
 	g_key_file_set_boolean(config, PACKAGE, "brace_match_ltgt", app->brace_match_ltgt);
 	g_key_file_set_boolean(config, PACKAGE, "switch_msgwin_pages", app->switch_msgwin_pages);
 	g_key_file_set_boolean(config, PACKAGE, "auto_close_xml_tags", app->pref_editor_auto_close_xml_tags);
 	g_key_file_set_boolean(config, PACKAGE, "auto_complete_constructs", app->pref_editor_auto_complete_constructs);
 #ifdef HAVE_VTE
 	g_key_file_set_boolean(config, "VTE", "load_vte", vte_info.load_vte);
-	if (vte_info.load_vte)
+	if (vte_info.load_vte && vc != NULL)
 	{
 		gchar *tmp_string;
 
@@ -128,6 +130,7 @@ void configuration_save()
 		g_key_file_set_boolean(config, "VTE", "scroll_on_out", vc->scroll_on_out);
 		g_key_file_set_boolean(config, "VTE", "ignore_menu_bar_accel", vc->ignore_menu_bar_accel);
 		g_key_file_set_boolean(config, "VTE", "follow_path", vc->follow_path);
+		g_key_file_set_boolean(config, "VTE", "run_in_vte", vc->run_in_vte);
 		g_key_file_set_integer(config, "VTE", "scrollback_lines", vc->scrollback_lines);
 		g_key_file_set_string(config, "VTE", "font", vc->font);
 		g_key_file_set_string(config, "VTE", "shell", vc->shell);
@@ -154,6 +157,7 @@ void configuration_save()
 	}
 	g_key_file_set_integer(config, PACKAGE, "pref_editor_tab_width", app->pref_editor_tab_width);
 	g_key_file_set_boolean(config, PACKAGE, "pref_main_confirm_exit", app->pref_main_confirm_exit);
+	g_key_file_set_boolean(config, PACKAGE, "pref_main_suppress_search_dialogs", app->pref_main_suppress_search_dialogs);
 	g_key_file_set_boolean(config, PACKAGE, "pref_main_load_session", app->pref_main_load_session);
 	g_key_file_set_boolean(config, PACKAGE, "pref_main_save_winpos", app->pref_main_save_winpos);
 	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show", app->toolbar_visible);
@@ -164,6 +168,7 @@ void configuration_save()
 	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_compile", app->pref_toolbar_show_compile);
 	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_colour", app->pref_toolbar_show_colour);
 	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_fileops", app->pref_toolbar_show_fileops);
+	g_key_file_set_boolean(config, PACKAGE, "pref_toolbar_show_quit", app->pref_toolbar_show_quit);
 	g_key_file_set_integer(config, PACKAGE, "pref_toolbar_icon_style", app->toolbar_icon_style);
 	g_key_file_set_integer(config, PACKAGE, "pref_toolbar_icon_size", app->toolbar_icon_size);
 	g_key_file_set_boolean(config, PACKAGE, "pref_editor_new_line", app->pref_editor_new_line);
@@ -205,25 +210,43 @@ void configuration_save()
 
 	if (cl_options.load_session)
 	{
-		// store the last 15(or what ever GEANY_SESSION_FILES is set to) filenames, to reopen the next time
+		// store the filenames to reopen them the next time
 		max = gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook));
-		for(i = 0; (i < max) && (j < GEANY_SESSION_FILES); i++)
+		for(i = 0; i < max; i++)
 		{
 			idx = document_get_n_idx(i);
 			if (idx >= 0 && doc_list[idx].file_name)
 			{
+				gchar *fname;
+				filetype *ft = doc_list[idx].file_type;
+
+				if (ft == NULL)	// can happen when saving a new file when quitting
+					ft = filetypes[GEANY_FILETYPES_ALL];
 				g_snprintf(entry, 13, "FILE_NAME_%d", j);
-				g_snprintf(fname, 255, "%d:%d:%s", sci_get_current_position(doc_list[idx].sci),
-					doc_list[idx].file_type->uid, doc_list[idx].file_name);
+				fname = g_strdup_printf("%d:%d:%s", sci_get_current_position(doc_list[idx].sci),
+					ft->uid, doc_list[idx].file_name);
 				g_key_file_set_string(config, "files", entry, fname);
+				g_free(fname);
 				j++;
 			}
 		}
-		// if open filenames less than GEANY_SESSION_FILES, delete existing saved entries in the list
-		for(i = j; i < GEANY_SESSION_FILES; i++)
+		// if open filenames less than saved session files, delete existing entries in the list
+		have_session_files = TRUE;
+		i = j;
+		while (have_session_files)
 		{
 			g_snprintf(entry, 13, "FILE_NAME_%d", i);
-			g_key_file_set_string(config, "files", entry, "");
+			tmp = g_key_file_get_string(config, "files", entry, NULL);
+			if (tmp == NULL)
+			{
+				have_session_files = FALSE;
+			}
+			else
+			{
+				g_key_file_remove_key(config, "files", entry, NULL);
+				g_free(tmp);
+				i++;
+			}
 		}
 	}
 
@@ -235,7 +258,6 @@ void configuration_save()
 	g_key_file_free(config);
 	g_free(configfile);
 	g_free(entry);
-	g_free(fname);
 	g_free(scribble_text);
 }
 
@@ -251,6 +273,7 @@ void configuration_save()
 gboolean configuration_load()
 {
 	gboolean config_exists;
+	gboolean have_session_files;
 	guint i, geo_len;
 	gint *geo;
 	gsize len = 0;
@@ -268,8 +291,7 @@ gboolean configuration_load()
 		GtkIconSize tb_iconsize;
 		GtkToolbarStyle tb_style;
 		GEANY_GET_SETTING("gtk-toolbar-style", tb_style, GTK_TOOLBAR_ICONS);
-		GEANY_GET_SETTING("gtk-toolbar-icon-size", tb_iconsize,
-			GTK_ICON_SIZE_LARGE_TOOLBAR);
+		GEANY_GET_SETTING("gtk-toolbar-icon-size", tb_iconsize, GTK_ICON_SIZE_LARGE_TOOLBAR);
 		app->toolbar_icon_style = utils_get_setting_integer(config, PACKAGE, "pref_toolbar_icon_style", tb_style);
 		app->toolbar_icon_size = utils_get_setting_integer(config, PACKAGE, "pref_toolbar_icon_size", tb_iconsize);
 	}
@@ -288,6 +310,7 @@ gboolean configuration_load()
 	app->msgwindow_visible = utils_get_setting_boolean(config, PACKAGE, "msgwindow_visible", TRUE);
 	app->pref_editor_line_breaking = utils_get_setting_boolean(config, PACKAGE, "line_breaking", FALSE); //default is off for better performance
 	app->pref_editor_use_auto_indention = utils_get_setting_boolean(config, PACKAGE, "use_auto_indention", TRUE);
+	app->use_tab_to_indent = utils_get_setting_boolean(config, PACKAGE, "use_tab_to_indent", FALSE);
 	app->pref_editor_use_indicators = utils_get_setting_boolean(config, PACKAGE, "use_indicators", TRUE);
 	app->pref_editor_show_indent_guide = utils_get_setting_boolean(config, PACKAGE, "show_indent_guide", FALSE);
 	app->pref_editor_show_white_space = utils_get_setting_boolean(config, PACKAGE, "show_white_space", FALSE);
@@ -295,10 +318,12 @@ gboolean configuration_load()
 	app->pref_editor_auto_close_xml_tags = utils_get_setting_boolean(config, PACKAGE, "auto_close_xml_tags", TRUE);
 	app->pref_editor_auto_complete_constructs = utils_get_setting_boolean(config, PACKAGE, "auto_complete_constructs", TRUE);
 	app->pref_editor_folding = utils_get_setting_boolean(config, PACKAGE, "use_folding", TRUE);
+	app->pref_editor_unfold_all_children = utils_get_setting_boolean(config, PACKAGE, "unfold_all_children", FALSE);
 	app->show_markers_margin = utils_get_setting_boolean(config, PACKAGE, "show_markers_margin", TRUE);
 	app->show_linenumber_margin = utils_get_setting_boolean(config, PACKAGE, "show_linenumber_margin", TRUE);
 	app->fullscreen = utils_get_setting_boolean(config, PACKAGE, "fullscreen", FALSE);
-	app->tab_order_ltr = utils_get_setting_boolean(config, PACKAGE, "tab_order_ltr", FALSE);
+	app->tab_order_ltr = utils_get_setting_boolean(config, PACKAGE, "tab_order_ltr", TRUE);
+	app->show_notebook_tabs = utils_get_setting_boolean(config, PACKAGE, "show_notebook_tabs", TRUE);
 	app->brace_match_ltgt = utils_get_setting_boolean(config, PACKAGE, "brace_match_ltgt", FALSE);
 	app->switch_msgwin_pages = utils_get_setting_boolean(config, PACKAGE, "switch_msgwin_pages", FALSE);
 	app->custom_date_format = utils_get_setting_string(config, PACKAGE, "custom_date_format", "");
@@ -341,7 +366,8 @@ gboolean configuration_load()
 
 		g_free(tmp_string);
 	}
-	app->pref_main_confirm_exit = utils_get_setting_boolean(config, PACKAGE, "pref_main_confirm_exit", TRUE);
+	app->pref_main_confirm_exit = utils_get_setting_boolean(config, PACKAGE, "pref_main_confirm_exit", FALSE);
+	app->pref_main_suppress_search_dialogs = utils_get_setting_boolean(config, PACKAGE, "pref_main_suppress_search_dialogs", FALSE);
 	app->pref_main_load_session = utils_get_setting_boolean(config, PACKAGE, "pref_main_load_session", TRUE);
 	app->pref_main_save_winpos = utils_get_setting_boolean(config, PACKAGE, "pref_main_save_winpos", TRUE);
 	app->pref_toolbar_show_search = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_show_search", TRUE);
@@ -351,6 +377,7 @@ gboolean configuration_load()
 	app->pref_toolbar_show_undo = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_show_undo", FALSE);
 	app->pref_toolbar_show_colour = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_show_colour", TRUE);
 	app->pref_toolbar_show_fileops = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_show_fileops", TRUE);
+	app->pref_toolbar_show_quit = utils_get_setting_boolean(config, PACKAGE, "pref_toolbar_show_quit", TRUE);
 #ifdef HAVE_VTE
 	vte_info.load_vte = utils_get_setting_boolean(config, "VTE", "load_vte", TRUE);
 	if (vte_info.load_vte)
@@ -360,7 +387,7 @@ gboolean configuration_load()
 
 		vc = g_new0(VteConfig, 1);
 		vte_info.dir = utils_get_setting_string(config, "VTE", "last_dir", NULL);
-		if ((vte_info.dir == NULL || utils_strcmp(vte_info.dir, "")) && pw != NULL)
+		if ((vte_info.dir == NULL || utils_str_equal(vte_info.dir, "")) && pw != NULL)
 			// last dir is not set, fallback to user's home directory
 			vte_info.dir = g_strdup(pw->pw_dir);
 		else if (vte_info.dir == NULL && pw == NULL)
@@ -374,6 +401,7 @@ gboolean configuration_load()
 		vc->scroll_on_out = utils_get_setting_boolean(config, "VTE", "scroll_on_out", TRUE);
 		vc->ignore_menu_bar_accel = utils_get_setting_boolean(config, "VTE", "ignore_menu_bar_accel", FALSE);
 		vc->follow_path = utils_get_setting_boolean(config, "VTE", "follow_path", FALSE);
+		vc->run_in_vte = utils_get_setting_boolean(config, "VTE", "run_in_vte", FALSE);
 		vc->scrollback_lines = utils_get_setting_integer(config, "VTE", "scrollback_lines", 500);
 		vc->colour_fore = g_new0(GdkColor, 1);
 		vc->colour_back = g_new0(GdkColor, 1);
@@ -436,16 +464,21 @@ gboolean configuration_load()
 	}
 	g_strfreev(recent_files);
 
-	for(i = 0; i < GEANY_SESSION_FILES; i++)
+	session_files = g_ptr_array_new();
+	have_session_files = TRUE;
+	i = 0;
+	while (have_session_files)
 	{
 		g_snprintf(entry, 13, "FILE_NAME_%d", i);
-		session_files[i] = g_key_file_get_string(config, "files", entry, &error);
-		if (! session_files[i] || error)
+		tmp_string = g_key_file_get_string(config, "files", entry, &error);
+		if (! tmp_string || error)
 		{
 			g_error_free(error);
 			error = NULL;
-			session_files[i] = NULL;
+			have_session_files = FALSE;
 		}
+		g_ptr_array_add(session_files, tmp_string);
+		i++;
 	}
 
 	g_key_file_free(config);
@@ -460,20 +493,23 @@ gboolean configuration_open_files()
 {
 	gint i;
 	guint x, pos, y, len;
-	gchar *file, *locale_filename, **array;
-	gboolean ret = FALSE;
+	gboolean ret = FALSE, failure = FALSE;
 
-	i = app->tab_order_ltr ? 0 : GEANY_SESSION_FILES - 1;
-	while(TRUE)
+	i = app->tab_order_ltr ? 0 : (session_files->len - 1);
+	while (TRUE)
 	{
-		if (session_files[i] && *session_files[i])
+		gchar *tmp = g_ptr_array_index(session_files, i);
+
+		if (tmp && *tmp)
 		{
+			const gchar *file;
+			gchar *locale_filename, **array;
 			gint uid = -1;
 			x = 0;
 			y = 0;
 
 			// yes it is :, it should be a ;, but now it is too late to change it
-			array = g_strsplit(session_files[i], ":", 3);
+			array = g_strsplit(tmp, ":", 3);
 			len = g_strv_length(array);
 
 			// read position
@@ -490,21 +526,28 @@ gboolean configuration_open_files()
 
 			// try to get the locale equivalent for the filename, fallback to filename if error
 			locale_filename = utils_get_locale_from_utf8(file);
+			g_strfreev(array);
 
-			if (g_file_test(locale_filename, G_FILE_TEST_IS_REGULAR || G_FILE_TEST_IS_SYMLINK))
+			if (g_file_test(locale_filename, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
 			{
 				filetype *ft = filetypes_get_from_uid(uid);
 				document_open_file(-1, locale_filename, pos, FALSE, ft, NULL);
 				ret = TRUE;
 			}
+			else
+			{
+				failure = TRUE;
+				geany_debug("Could not find file '%s'.", file);
+			}
+
 			g_free(locale_filename);
 		}
-		g_free(session_files[i]);
+		g_free(tmp);
 
 		if (app->tab_order_ltr)
 		{
 			i++;
-			if (i >= GEANY_SESSION_FILES) break;
+			if (i >= (gint)session_files->len) break;
 		}
 		else
 		{
@@ -513,6 +556,9 @@ gboolean configuration_open_files()
 		}
 	}
 
+	g_ptr_array_free(session_files, TRUE);
+	if (failure)
+		msgwin_status_add(_("Failed to load one or more session files."));
 	return ret;
 }
 
@@ -536,21 +582,6 @@ void configuration_apply_settings()
 		gtk_paned_set_position(GTK_PANED(lookup_widget(app->window, "vpaned1")), vpan_position);
 	}
 
-	// now the scintilla widget pages may need scrolling in view
-	if (app->pref_main_load_session)
-	{
-		gint idx;
-		guint tabnum = 0;
-
-		while (tabnum < (guint) gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)))
-		{
-			idx = document_get_n_idx(tabnum);
-			if (idx < 0) break;
-			sci_scroll_caret(doc_list[idx].sci);
-			tabnum++;
-		}
-	}
-
 	// set fullscreen after initial draw so that returning to normal view is the right size.
 	// fullscreen mode is disabled by default, so act only if it is true
 	if (app->fullscreen)
@@ -562,6 +593,12 @@ void configuration_apply_settings()
 }
 
 
+#ifdef GEANY_DEBUG
+// Geany data file generation is only available with a debug build of Geany.
+
+static void generate_filetype_extensions(const gchar *output_dir);
+
+
 /* Generate the config files in "data/" from defaults */
 void configuration_generate_data_files()
 {
@@ -577,6 +614,7 @@ void configuration_generate_data_files()
 		return;
 	}
 	g_print("Generating system files in %s:\n", gen_dir);
+	// currently only filetype extensions are auto-generated.
 	generate_filetype_extensions(gen_dir);
 	g_free(gen_dir);
 }
@@ -618,6 +656,8 @@ static void generate_filetype_extensions(const gchar *output_dir)
 	g_key_file_free(config);
 }
 
+#endif
+
 
 void configuration_read_filetype_extensions()
 {
@@ -649,6 +689,8 @@ void configuration_read_filetype_extensions()
 		else g_strfreev(list);
 	}
 
+	g_free(sysconfigfile);
+	g_free(userconfigfile);
 	g_key_file_free(sysconfig);
 	g_key_file_free(userconfig);
 }
