@@ -1,7 +1,8 @@
 /*
  *      main.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: main.c 830 2006-09-22 11:38:14Z ntrel $
+ * $Id: main.c 1269 2007-02-12 17:21:57Z eht16 $
  */
 
 
@@ -50,6 +51,9 @@
 #include "keybindings.h"
 #include "sci_cb.h"
 #include "search.h"
+#include "build.h"
+#include "highlighting.h"
+#include "symbols.h"
 
 #ifdef HAVE_SOCKET
 # include "socket.h"
@@ -81,20 +85,21 @@ static gboolean generate_datafiles = FALSE;
 
 static GOptionEntry entries[] =
 {
+	{ "config", 'c', 0, G_OPTION_ARG_FILENAME, &alternate_config, N_("use an alternate configuration directory"), NULL },
 	{ "debug", 'd', 0, G_OPTION_ARG_NONE, &debug_mode, N_("runs in debug mode (means being verbose)"), NULL },
-	{ "no-session", 's', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &cl_options.load_session, N_("don't load the previous session's files"), NULL },
-	{ "no-ctags", 'n', 0, G_OPTION_ARG_NONE, &ignore_global_tags, N_("don't load auto completion data (see documentation)"), NULL },
+	{ "generate-data-files", 'g', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &generate_datafiles, "", NULL },
 #ifdef HAVE_SOCKET
 	{ "new-instance", 'i', 0, G_OPTION_ARG_NONE, &ignore_socket, N_("don't open files in a running instance, force opening a new instance"), NULL },
 #endif
-	{ "config", 'c', 0, G_OPTION_ARG_FILENAME, &alternate_config, N_("use an alternate configuration directory"), NULL },
+	{ "line", 'l', 0, G_OPTION_ARG_INT, &cl_options.goto_line, N_("set initial line number for the first opened file"), NULL },
 	{ "no-msgwin", 'm', 0, G_OPTION_ARG_NONE, &no_msgwin, N_("don't show message window at startup"), NULL },
+	{ "no-ctags", 'n', 0, G_OPTION_ARG_NONE, &ignore_global_tags, N_("don't load auto completion data (see documentation)"), NULL },
+	{ "no-session", 's', G_OPTION_FLAG_REVERSE, G_OPTION_ARG_NONE, &cl_options.load_session, N_("don't load the previous session's files"), NULL },
 #ifdef HAVE_VTE
 	{ "no-terminal", 't', 0, G_OPTION_ARG_NONE, &no_vte, N_("don't load terminal support"), NULL },
-	{ "vte-lib", 'l', 0, G_OPTION_ARG_FILENAME, &lib_vte, N_("filename of libvte.so"), NULL },
+	{ "vte-lib", 0, 0, G_OPTION_ARG_FILENAME, &lib_vte, N_("filename of libvte.so"), NULL },
 #endif
 	{ "version", 'v', 0, G_OPTION_ARG_NONE, &show_version, N_("show version and exit"), NULL },
-	{ "generate-data-files", 'g', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_NONE, &generate_datafiles, "", NULL },
 	{ NULL, 0, 0, 0, NULL, NULL, NULL }
 };
 
@@ -201,8 +206,11 @@ static void apply_settings(void)
 				lookup_widget(app->window, "menu_use_auto_indention1")), app->pref_editor_use_auto_indention);
 	app->ignore_callback = FALSE;
 
+	// connect the toolbar dropdown menus
 	gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(
 			lookup_widget(app->window, "menutoolbutton1")), app->new_file_menu);
+	gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(
+			lookup_widget(app->window, "toolbutton9")), app->recent_files_toolbar);
 
 	// set the tab placements of the notebooks
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(app->notebook), app->tab_pos_editor);
@@ -210,6 +218,9 @@ static void apply_settings(void)
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(app->treeview_notebook), app->tab_pos_sidebar);
 
 	ui_update_toolbar_items();
+
+	// whether to show notebook tabs or not
+	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(app->notebook), app->show_notebook_tabs);
 }
 
 
@@ -230,10 +241,13 @@ static void main_init(void)
 	app->tm_workspace				= tm_get_workspace();
 	app->recent_queue				= g_queue_new();
 	app->opening_session_files		= FALSE;
-	html_entities					= NULL;
 
 	app->window = create_window1();
 	app->new_file_menu = gtk_menu_new();
+	app->recent_files_toolbar = gtk_menu_new();
+	app->recent_files_menubar = gtk_menu_new();
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(lookup_widget(app->window, "recent_files1")),
+		app->recent_files_menubar);
 
 	// store important pointers in the MyApp structure
 	app->toolbar = lookup_widget(app->window, "toolbar1");
@@ -307,9 +321,6 @@ static void main_init(void)
 	app->undo_items[0] = lookup_widget(app->popup_menu, "undo1");
 	app->undo_items[1] = lookup_widget(app->window, "menu_undo2");
 	app->undo_items[2] = lookup_widget(app->window, "toolbutton_undo");
-
-	msgwin_init();
-	search_init();
 }
 
 
@@ -353,8 +364,12 @@ static void setup_paths()
 	doc_dir = g_strdup(PACKAGE_DATA_DIR "/doc/" PACKAGE "/html/");
 #endif
 
-	app->datadir = data_dir;
-	app->docdir = doc_dir;
+	// convert path names to locale encoding
+	app->datadir = utils_get_locale_from_utf8(data_dir);
+	app->docdir = utils_get_locale_from_utf8(doc_dir);
+
+	g_free(data_dir);
+	g_free(doc_dir);
 }
 
 
@@ -384,6 +399,7 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 
 	// first initialise cl_options fields with default values
 	cl_options.load_session = TRUE;
+	cl_options.goto_line = -1;
 
 	context = g_option_context_new(_(" - A fast and lightweight IDE"));
 	g_option_context_add_main_entries(context, entries, GETTEXT_PACKAGE);
@@ -407,8 +423,6 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 #ifdef GEANY_DEBUG
 	geany_debug("debug mode built in (can't be disabled)");
 #endif
-	geany_debug("GTK+ runtime version: %u.%u.%u",
-		gtk_major_version, gtk_minor_version, gtk_micro_version);
 
 	if (alternate_config)
 	{
@@ -433,6 +447,12 @@ static gint setup_config_dir()
 {
 	gint mkdir_result = 0;
 
+	gchar *tmp = app->configdir;
+
+	// convert configdir to locale encoding to avoid troubles
+	app->configdir = utils_get_locale_from_utf8(app->configdir);
+	g_free(tmp);
+
 	mkdir_result = utils_make_settings_dir(app->configdir, app->datadir, app->docdir);
 	if (mkdir_result != 0)
 	{
@@ -472,7 +492,9 @@ gint main(gint argc, gchar **argv)
 
 	signal(SIGTERM, signal_cb);
 #ifdef G_OS_UNIX
-	/* ignore SIGPIPE signal for preventing sudden death of program */
+	// SIGQUIT is used to kill spawned children and we get also this signal, so ignore
+	signal(SIGQUIT, SIG_IGN);
+	// ignore SIGPIPE signal for preventing sudden death of program
 	signal(SIGPIPE, SIG_IGN);
 #endif
 
@@ -502,6 +524,9 @@ gint main(gint argc, gchar **argv)
 	}
 #endif
 
+	geany_debug("GTK+ %u.%u.%u, GLib %u.%u.%u",
+		gtk_major_version, gtk_minor_version, gtk_micro_version,
+		glib_major_version, glib_minor_version, glib_micro_version);
 	gtk_init(&argc, &argv);
 
 	// inits
@@ -517,6 +542,8 @@ gint main(gint argc, gchar **argv)
 #endif
 	if (no_msgwin) app->msgwindow_visible = FALSE;
 
+	msgwin_init();
+	search_init();
 	ui_create_insert_menu_items();
 	ui_create_insert_date_menu_items();
 	keybindings_init();
@@ -524,14 +551,23 @@ gint main(gint argc, gchar **argv)
 	templates_init();
 	document_init_doclist();
 	filetypes_init_types();
+#ifdef GEANY_DEBUG
 	if (generate_datafiles)
 	{
 		configuration_generate_data_files();
 		exit(0);
 	}
+#endif
 	configuration_read_filetype_extensions();
 
-	gtk_window_set_icon(GTK_WINDOW(app->window), ui_new_pixbuf_from_inline(GEANY_IMAGE_LOGO, FALSE));
+	// set window icon
+	{
+		GdkPixbuf *pb;
+
+		pb = ui_new_pixbuf_from_inline(GEANY_IMAGE_LOGO, FALSE);
+		gtk_window_set_icon(GTK_WINDOW(app->window), pb);
+		g_object_unref(pb);	// free our reference
+	}
 
 	// registering some basic events
 	g_signal_connect(G_OBJECT(app->window), "delete_event", G_CALLBACK(on_exit_clicked), NULL);
@@ -542,12 +578,6 @@ gint main(gint argc, gchar **argv)
 	treeviews_prepare_openfiles();
 	treeviews_create_taglist_popup_menu();
 	treeviews_create_openfiles_popup_menu();
-	msgwin_prepare_status_tree_view();
-	msgwin_prepare_msg_tree_view();
-	msgwin_prepare_compiler_tree_view();
-	msgwindow.popup_status_menu = msgwin_create_message_popup_menu(3);
-	msgwindow.popup_msg_menu = msgwin_create_message_popup_menu(4);
-	msgwindow.popup_compiler_menu = msgwin_create_message_popup_menu(5);
 #ifdef HAVE_VTE
 	vte_init();
 #endif
@@ -565,26 +595,22 @@ gint main(gint argc, gchar **argv)
 	app->opening_session_files = TRUE;
 	if (argc > 1)
 	{
-		gint i, opened = 0;
+		gint i;
 		for(i = 1; i < argc; i++)
 		{
-			if (argv[i] && g_file_test(argv[i], G_FILE_TEST_IS_REGULAR || G_FILE_TEST_IS_SYMLINK))
+			if (argv[i] && g_file_test(argv[i], G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
 			{
-				if (opened < GEANY_MAX_OPEN_FILES)
-				{
-					gchar *filename = get_argv_filename(argv[i]);
-					document_open_file(-1, filename, 0, FALSE, NULL, NULL);
-					g_free(filename);
-					opened++;
-				}
-				else
-				{
-					dialogs_show_error(
-			_("You have opened too many files. There is a limit of %d concurrent open files."),
-			GEANY_MAX_OPEN_FILES);
-					break;
-				}
+				gchar *filename = get_argv_filename(argv[i]);
+				gint idx;
+
+				idx = document_open_file(-1, filename, 0, FALSE, NULL, NULL);
+				// add recent file manually because opening_session_files is set
+				if (DOC_IDX_VALID(idx))
+					ui_add_recent_file(doc_list[idx].file_name);
+				g_free(filename);
 			}
+			else
+				geany_debug("Could not load file '%s'.", argv[i]);
 		}
 	}
 	else if (app->pref_main_load_session && cl_options.load_session)
@@ -606,7 +632,7 @@ gint main(gint argc, gchar **argv)
 	idx = document_get_cur_idx();
 	gtk_widget_grab_focus(GTK_WIDGET(doc_list[idx].sci));
 	gtk_tree_model_foreach(GTK_TREE_MODEL(tv.store_openfiles), treeviews_find_node, GINT_TO_POINTER(idx));
-	ui_build_show_hide(idx);
+	build_menu_update(idx);
 	ui_update_tag_list(idx, FALSE);
 
 #ifdef G_OS_WIN32
@@ -641,3 +667,86 @@ gint main(gint argc, gchar **argv)
 	return 0;
 }
 
+
+void main_quit()
+{
+	geany_debug("Quitting...");
+
+#ifdef HAVE_SOCKET
+	socket_finalize();
+#endif
+
+	keybindings_free();
+	filetypes_save_commands();
+	filetypes_free_types();
+	styleset_free_styles();
+	templates_free_templates();
+	msgwin_finalize();
+	search_finalize();
+	build_finalize();
+	document_finalize();
+	symbols_finalize();
+
+	tm_workspace_free(TM_WORK_OBJECT(app->tm_workspace));
+	g_free(app->configdir);
+	g_free(app->datadir);
+	g_free(app->docdir);
+	g_free(app->custom_date_format);
+	g_free(app->editor_font);
+	g_free(app->tagbar_font);
+	g_free(app->msgwin_font);
+	g_free(app->long_line_color);
+	g_free(app->pref_template_developer);
+	g_free(app->pref_template_company);
+	g_free(app->pref_template_mail);
+	g_free(app->pref_template_initial);
+	g_free(app->pref_template_version);
+	g_free(app->tools_make_cmd);
+	g_free(app->tools_term_cmd);
+	g_free(app->tools_browser_cmd);
+	g_free(app->tools_print_cmd);
+	g_free(app->tools_grep_cmd);
+	while (! g_queue_is_empty(app->recent_queue))
+	{
+		g_free(g_queue_pop_tail(app->recent_queue));
+	}
+	g_queue_free(app->recent_queue);
+
+	if (app->prefs_dialog && GTK_IS_WIDGET(app->prefs_dialog)) gtk_widget_destroy(app->prefs_dialog);
+	if (app->save_filesel && GTK_IS_WIDGET(app->save_filesel)) gtk_widget_destroy(app->save_filesel);
+	if (app->open_filesel && GTK_IS_WIDGET(app->open_filesel)) gtk_widget_destroy(app->open_filesel);
+	if (app->open_fontsel && GTK_IS_WIDGET(app->open_fontsel)) gtk_widget_destroy(app->open_fontsel);
+	if (app->open_colorsel && GTK_IS_WIDGET(app->open_colorsel)) gtk_widget_destroy(app->open_colorsel);
+	if (app->default_tag_tree && GTK_IS_WIDGET(app->default_tag_tree))
+	{
+		g_object_unref(app->default_tag_tree);
+		gtk_widget_destroy(app->default_tag_tree);
+	}
+	scintilla_release_resources();
+#ifdef HAVE_VTE
+	if (vte_info.have_vte) vte_close();
+	g_free(vte_info.lib_vte);
+	g_free(vte_info.dir);
+#endif
+	gtk_widget_destroy(app->window);
+
+	// destroy popup menus
+	if (app->popup_menu && GTK_IS_WIDGET(app->popup_menu))
+					gtk_widget_destroy(app->popup_menu);
+	if (app->toolbar_menu && GTK_IS_WIDGET(app->toolbar_menu))
+					gtk_widget_destroy(app->toolbar_menu);
+	if (tv.popup_taglist && GTK_IS_WIDGET(tv.popup_taglist))
+					gtk_widget_destroy(tv.popup_taglist);
+	if (tv.popup_openfiles && GTK_IS_WIDGET(tv.popup_openfiles))
+					gtk_widget_destroy(tv.popup_openfiles);
+	if (msgwindow.popup_status_menu && GTK_IS_WIDGET(msgwindow.popup_status_menu))
+					gtk_widget_destroy(msgwindow.popup_status_menu);
+	if (msgwindow.popup_msg_menu && GTK_IS_WIDGET(msgwindow.popup_msg_menu))
+					gtk_widget_destroy(msgwindow.popup_msg_menu);
+	if (msgwindow.popup_compiler_menu && GTK_IS_WIDGET(msgwindow.popup_compiler_menu))
+					gtk_widget_destroy(msgwindow.popup_compiler_menu);
+
+	g_free(app);
+
+	gtk_main_quit();
+}
