@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: main.h 798 2006-09-07 15:51:24Z eht16 $
+ * $Id: main.h 913 2006-10-21 11:34:18Z ntrel $
  */
 
 
@@ -28,13 +28,14 @@
 typedef struct
 {
 	gboolean load_session;
+	gint goto_line;
 } CommandLineOptions;
 
 extern CommandLineOptions cl_options;
 
 
-void geany_debug(gchar const *format, ...);
-
 gchar *get_argv_filename(const gchar *filename);
 
+void main_quit();
+
 #endif
