@@ -1,7 +1,8 @@
 /*
  *      msgwindow.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: msgwindow.c 820 2006-09-17 20:40:15Z ntrel $
+ * $Id: msgwindow.c 1269 2007-02-12 17:21:57Z eht16 $
  */
 
 
@@ -31,16 +32,12 @@
 #include "ui_utils.h"
 #include "utils.h"
 #include "document.h"
+#include "build.h"
 
 #include <string.h>
 #include <stdlib.h>
 
 
-static struct
-{
-	gchar	*dir;
-	guint	file_type_id;
-} build_info = {NULL, GEANY_FILETYPES_ALL};
 
 // used for parse_file_line
 typedef struct
@@ -53,13 +50,20 @@ typedef struct
 	gint file_idx;			// idx of the field where the filename is or -1
 } ParseData;
 
-static GdkColor dark = {0, 58832, 58832, 58832};
-static GdkColor white = {0, 65535, 65535, 65535};
+static const GdkColor dark = {0, 58832, 58832, 58832};
+static const GdkColor white = {0, 65535, 65535, 65535};
 
 MessageWindow msgwindow;
 
 
+static void prepare_msg_tree_view(void);
+static void prepare_status_tree_view(void);
+static void prepare_compiler_tree_view(void);
+static GtkWidget *create_message_popup_menu(gint type);
 static void msgwin_parse_grep_line(const gchar *string, gchar **filename, gint *line);
+static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *event,
+																			gpointer user_data);
+static void on_scribble_populate(GtkTextView *textview, GtkMenu *arg1, gpointer user_data);
 
 
 void msgwin_init()
@@ -69,29 +73,33 @@ void msgwin_init()
 	msgwindow.tree_msg = lookup_widget(app->window, "treeview4");
 	msgwindow.tree_compiler = lookup_widget(app->window, "treeview5");
 	msgwindow.find_in_files_dir = NULL;
+
+	gtk_widget_set_sensitive(lookup_widget(app->window, "next_message1"), FALSE);
+
+	prepare_status_tree_view();
+	prepare_msg_tree_view();
+	prepare_compiler_tree_view();
+	msgwindow.popup_status_menu = create_message_popup_menu(MSG_STATUS);
+	msgwindow.popup_msg_menu = create_message_popup_menu(MSG_MESSAGE);
+	msgwindow.popup_compiler_menu = create_message_popup_menu(MSG_COMPILER);
+
+	g_signal_connect(G_OBJECT(lookup_widget(app->window, "textview_scribble")),
+		"populate-popup", G_CALLBACK(on_scribble_populate), NULL);
 }
 
 
 void msgwin_finalize()
 {
 	g_free(msgwindow.find_in_files_dir);
-	g_free(build_info.dir);
-}
-
-
-void msgwin_set_build_info(const gchar *dir, guint file_type_id)
-{
-	g_free(build_info.dir);
-	build_info.dir = g_strdup(dir);
-	build_info.file_type_id = file_type_id;
 }
 
 
 /* does some preparing things to the status message list widget */
-void msgwin_prepare_status_tree_view(void)
+static void prepare_status_tree_view(void)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
+	PangoFontDescription *pfd;
 
 	msgwindow.store_status = gtk_list_store_new(2, GDK_TYPE_COLOR, G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(msgwindow.tree_status), GTK_TREE_MODEL(msgwindow.store_status));
@@ -102,20 +110,23 @@ void msgwin_prepare_status_tree_view(void)
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(msgwindow.tree_status), FALSE);
 
-	gtk_widget_modify_font(msgwindow.tree_status, pango_font_description_from_string(app->msgwin_font));
-	g_signal_connect(G_OBJECT(msgwindow.tree_status), "button-press-event",
-				G_CALLBACK(on_tree_view_button_press_event), GINT_TO_POINTER(3));
+	pfd = pango_font_description_from_string(app->msgwin_font);
+	gtk_widget_modify_font(msgwindow.tree_status, pfd);
+	pango_font_description_free(pfd);
 
+	g_signal_connect(G_OBJECT(msgwindow.tree_status), "button-press-event",
+				G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_STATUS));
 }
 
 
 /* does some preparing things to the message list widget
  * (currently used for showing results of 'Find usage') */
-void msgwin_prepare_msg_tree_view(void)
+static void prepare_msg_tree_view(void)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *select;
+	PangoFontDescription *pfd;
 
 	msgwindow.store_msg = gtk_list_store_new(4, G_TYPE_INT, G_TYPE_INT, GDK_TYPE_COLOR, G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(msgwindow.tree_msg), GTK_TREE_MODEL(msgwindow.store_msg));
@@ -126,9 +137,13 @@ void msgwin_prepare_msg_tree_view(void)
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(msgwindow.tree_msg), FALSE);
 
-	gtk_widget_modify_font(msgwindow.tree_msg, pango_font_description_from_string(app->msgwin_font));
-	g_signal_connect(G_OBJECT(msgwindow.tree_msg), "button-press-event",
-					G_CALLBACK(on_tree_view_button_press_event), GINT_TO_POINTER(4));
+	pfd = pango_font_description_from_string(app->msgwin_font);
+	gtk_widget_modify_font(msgwindow.tree_msg, pfd);
+	pango_font_description_free(pfd);
+
+	// use button-release-event so the selection has changed (connect_after button-press-event doesn't work)
+	g_signal_connect(G_OBJECT(msgwindow.tree_msg), "button-release-event",
+					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_MESSAGE));
 
 	// selection handling
 	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgwindow.tree_msg));
@@ -138,11 +153,12 @@ void msgwin_prepare_msg_tree_view(void)
 
 
 /* does some preparing things to the compiler list widget */
-void msgwin_prepare_compiler_tree_view(void)
+static void prepare_compiler_tree_view(void)
 {
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *select;
+	PangoFontDescription *pfd;
 
 	msgwindow.store_compiler = gtk_list_store_new(2, GDK_TYPE_COLOR, G_TYPE_STRING);
 	gtk_tree_view_set_model(GTK_TREE_VIEW(msgwindow.tree_compiler), GTK_TREE_MODEL(msgwindow.store_compiler));
@@ -153,9 +169,13 @@ void msgwin_prepare_compiler_tree_view(void)
 
 	gtk_tree_view_set_enable_search(GTK_TREE_VIEW(msgwindow.tree_compiler), FALSE);
 
-	gtk_widget_modify_font(msgwindow.tree_compiler, pango_font_description_from_string(app->msgwin_font));
-	g_signal_connect(G_OBJECT(msgwindow.tree_compiler), "button-press-event",
-					G_CALLBACK(on_tree_view_button_press_event), GINT_TO_POINTER(5));
+	pfd = pango_font_description_from_string(app->msgwin_font);
+	gtk_widget_modify_font(msgwindow.tree_compiler, pfd);
+	pango_font_description_free(pfd);
+
+	// use button-release-event so the selection has changed (connect_after button-press-event doesn't work)
+	g_signal_connect(G_OBJECT(msgwindow.tree_compiler), "button-release-event",
+					G_CALLBACK(on_msgwin_button_press_event), GINT_TO_POINTER(MSG_COMPILER));
 
 	// selection handling
 	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgwindow.tree_compiler));
@@ -164,85 +184,97 @@ void msgwin_prepare_compiler_tree_view(void)
 }
 
 
-// adds string to the compiler textview
-void msgwin_compiler_add(gint msg_color, gboolean scroll, gchar const *format, ...)
+static const GdkColor color_error = {0, 65535, 0, 0};
+
+void msgwin_compiler_add_fmt(gint msg_color, const gchar *format, ...)
 {
-	GtkTreeIter iter;
-	GtkTreePath *path;
-	GdkColor *color;
-	static GdkColor red = {0, 65535, 0, 0};
-	static GdkColor dark_red = {0, 65535 / 2, 0, 0};
-	static GdkColor blue = {0, 0, 0, 65535};
-	static GdkColor black = {0, 0, 0, 0};
-	static gchar string[512];
+	gchar string[512];
 	va_list args;
 
 	va_start(args, format);
-	g_vsnprintf(string, 511, format, args);
+	g_vsnprintf(string, 512, format, args);
 	va_end(args);
+	msgwin_compiler_add(msg_color, string);
+}
+
+
+// adds string to the compiler textview
+void msgwin_compiler_add(gint msg_color, const gchar *msg)
+{
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	const GdkColor *color;
+	const GdkColor dark_red = {0, 65535 / 2, 0, 0};
+	const GdkColor blue = {0, 0, 0, 0xD000};	// not too bright ;-)
+	const GdkColor black = {0, 0, 0, 0};
 
 	switch (msg_color)
 	{
-		case COLOR_RED: color = &red; break;
+		case COLOR_RED: color = &color_error; break;
 		case COLOR_DARK_RED: color = &dark_red; break;
 		case COLOR_BLUE: color = &blue; break;
 		default: color = &black;
 	}
 
 	gtk_list_store_append(msgwindow.store_compiler, &iter);
-	gtk_list_store_set(msgwindow.store_compiler, &iter, 0, color, 1, string, -1);
+	gtk_list_store_set(msgwindow.store_compiler, &iter, 0, color, 1, msg, -1);
 
 	if (app->msgwindow_visible)
 	{
 		path = gtk_tree_model_get_path(
 			gtk_tree_view_get_model(GTK_TREE_VIEW(msgwindow.tree_compiler)), &iter);
 		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(msgwindow.tree_compiler), path, NULL, TRUE, 0.5, 0.5);
-
-		if (scroll)
-		{
-			gtk_tree_view_set_cursor(GTK_TREE_VIEW(msgwindow.tree_compiler), path, NULL, FALSE);
-		}
 		gtk_tree_path_free(path);
 	}
+
+	// calling build_menu_update for every build message would be overkill
+	gtk_widget_set_sensitive(build_get_menu_items(-1)->item_next_error, TRUE);
 }
 
 
-void msgwin_show()
+void msgwin_show_hide(gboolean show)
 {
-	app->msgwindow_visible = TRUE;
-	gtk_widget_show(lookup_widget(app->window, "scrolledwindow1"));
+	app->msgwindow_visible = show;
+	app->ignore_callback = TRUE;
+	gtk_check_menu_item_set_active(
+		GTK_CHECK_MENU_ITEM(lookup_widget(app->window, "menu_show_messages_window1")),
+		show);
+	app->ignore_callback = FALSE;
+	ui_widget_show_hide(lookup_widget(app->window, "scrolledwindow1"), show);
 }
 
 
 // adds string to the msg treeview
-void msgwin_msg_add(gint line, gint idx, gchar *string)
+void msgwin_msg_add(gint line, gint idx, const gchar *string)
 {
 	GtkTreeIter iter;
 	static gint state = 0;
 
-	if (! app->msgwindow_visible) msgwin_show();
+	if (! app->msgwindow_visible) msgwin_show_hide(TRUE);
 
 	gtk_list_store_append(msgwindow.store_msg, &iter);
 	gtk_list_store_set(msgwindow.store_msg, &iter, 0, line, 1, idx, 2,
 		((state++ % 2) == 0) ? &white : &dark, 3, string, -1);
+
+	gtk_widget_set_sensitive(lookup_widget(app->window, "next_message1"), TRUE);
 }
 
 
-// adds a status message
-void msgwin_status_add(gchar const *format, ...)
+// logs a status message (use ui_set_statusbar() to just display text on the statusbar)
+void msgwin_status_add(const gchar *format, ...)
 {
 	GtkTreeIter iter;
 	static gint state = 0;
-	static gchar string[512];
+	gchar string[512];
 	gchar *statusmsg, *time_str;
 	va_list args;
 
 	va_start(args, format);
-	g_vsnprintf(string, 511, format, args);
+	g_vsnprintf(string, 512, format, args);
 	va_end(args);
 
 	// display status message in status bar
-	ui_set_statusbar(string, FALSE);
+	ui_set_statusbar("%s", string);
 
 	// add a timestamp to status messages
 	time_str = utils_get_current_time_string();
@@ -269,34 +301,128 @@ void msgwin_status_add(gchar const *format, ...)
 }
 
 
-GtkWidget *msgwin_create_message_popup_menu(gint type)
+static void
+on_message_treeview_clear_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
 {
-	GtkWidget *message_popup_menu, *clear;
+	GtkListStore *store;
+
+	switch (GPOINTER_TO_INT(user_data))
+	{
+		case MSG_MESSAGE:
+		gtk_widget_set_sensitive(lookup_widget(app->window, "next_message1"), FALSE);
+		store = msgwindow.store_msg;
+		break;
+
+		case MSG_COMPILER:
+		gtk_widget_set_sensitive(build_get_menu_items(-1)->item_next_error, FALSE);
+		store = msgwindow.store_compiler;
+		break;
+
+		default: // MSG_STATUS
+		store = msgwindow.store_status;
+	}
+	gtk_list_store_clear(store);
+}
+
+
+static void
+on_compiler_treeview_copy_activate     (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	GtkWidget *tv = NULL;
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gint str_idx = 1;
+
+	switch (GPOINTER_TO_INT(user_data))
+	{
+		case MSG_STATUS:
+		tv = msgwindow.tree_status;
+		break;
+
+		case MSG_COMPILER:
+		tv = msgwindow.tree_compiler;
+		break;
+
+		case MSG_MESSAGE:
+		tv = msgwindow.tree_msg;
+		str_idx = 3;
+		break;
+	}
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv));
+
+	if (gtk_tree_selection_get_selected(selection, &model, &iter))
+	{
+		gchar *string;
+
+		gtk_tree_model_get(model, &iter, str_idx, &string, -1);
+		if (string && *string)
+		{
+			gtk_clipboard_set_text(gtk_clipboard_get(gdk_atom_intern("CLIPBOARD", FALSE)),
+				string, -1);
+		}
+		g_free(string);
+	}
+}
+
+
+static void
+on_hide_message_window                 (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	msgwin_show_hide(FALSE);
+}
+
+
+static GtkWidget *create_message_popup_menu(gint type)
+{
+	GtkWidget *message_popup_menu, *clear, *copy;
 
 	message_popup_menu = gtk_menu_new();
 
 	clear = gtk_image_menu_item_new_from_stock("gtk-clear", NULL);
 	gtk_widget_show(clear);
 	gtk_container_add(GTK_CONTAINER(message_popup_menu), clear);
+	g_signal_connect((gpointer)clear, "activate",
+		G_CALLBACK(on_message_treeview_clear_activate), GINT_TO_POINTER(type));
 
-	if (type == 3)
-		g_signal_connect((gpointer)clear, "activate", G_CALLBACK(on_message_treeview_clear_activate), msgwindow.store_status);
-	else if (type == 4)
-		g_signal_connect((gpointer)clear, "activate", G_CALLBACK(on_message_treeview_clear_activate), msgwindow.store_msg);
-	else if (type == 5)
-	{
-		GtkWidget *copy = gtk_image_menu_item_new_from_stock("gtk-copy", NULL);
-		gtk_widget_show(copy);
-		gtk_container_add(GTK_CONTAINER(message_popup_menu), copy);
+	copy = gtk_image_menu_item_new_from_stock("gtk-copy", NULL);
+	gtk_widget_show(copy);
+	gtk_container_add(GTK_CONTAINER(message_popup_menu), copy);
+	g_signal_connect((gpointer)copy, "activate",
+		G_CALLBACK(on_compiler_treeview_copy_activate), GINT_TO_POINTER(type));
 
-		g_signal_connect((gpointer)copy, "activate", G_CALLBACK(on_compiler_treeview_copy_activate), NULL);
-		g_signal_connect((gpointer)clear, "activate", G_CALLBACK(on_message_treeview_clear_activate), msgwindow.store_compiler);
-	}
+	msgwin_menu_add_common_items(GTK_MENU(message_popup_menu));
 
 	return message_popup_menu;
 }
 
 
+static void on_scribble_populate(GtkTextView *textview, GtkMenu *arg1, gpointer user_data)
+{
+	msgwin_menu_add_common_items(arg1);
+}
+
+
+/* Menu items that should be on all message window popup menus */
+void msgwin_menu_add_common_items(GtkMenu *menu)
+{
+	GtkWidget *item;
+
+	item = gtk_separator_menu_item_new();
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
+
+	item = gtk_menu_item_new_with_mnemonic(_("_Hide Message Window"));
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect((gpointer)item, "activate",
+		G_CALLBACK(on_hide_message_window), NULL);
+}
+
+
 gboolean msgwin_goto_compiler_file_line()
 {
 	GtkTreeIter iter;
@@ -304,10 +430,20 @@ gboolean msgwin_goto_compiler_file_line()
 	GtkTreeSelection *selection;
 	gchar *string;
 	gboolean ret = FALSE;
+	GdkColor *color;
 
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgwindow.tree_compiler));
 	if (gtk_tree_selection_get_selected(selection, &model, &iter))
 	{
+		// if the item is not coloured red, it's not an error line
+		gtk_tree_model_get(model, &iter, 0, &color, -1);
+		if (! gdk_color_equal(color, &color_error))
+		{
+			gdk_color_free(color);
+			return FALSE;
+		}
+		gdk_color_free(color);
+
 		gtk_tree_model_get(model, &iter, 1, &string, -1);
 		if (string != NULL)
 		{
@@ -326,7 +462,8 @@ gboolean msgwin_goto_compiler_file_line()
 
 				if (idx >= 0 && doc_list[idx].is_valid)
 				{
-					document_set_indicator(idx, line - 1);
+					if (! doc_list[idx].changed)	// if modified, line may be wrong
+						document_set_indicator(idx, line - 1);
 					ret = utils_goto_line(idx, line);
 				}
 			}
@@ -422,10 +559,24 @@ void msgwin_parse_compiler_error_line(const gchar *string, gchar **filename, gin
 		case GEANY_FILETYPES_PHP:
 		{
 			// Parse error: parse error, unexpected T_CASE in brace_bug.php on line 3
-			data.pattern = " ";
-			data.min_fields = 11;
-			data.line_idx = 10;
-			data.file_idx = 7;
+			// Parse error: syntax error, unexpected T_LNUMBER, expecting T_FUNCTION in bob.php on line 16
+			gchar *tmp = strstr(string, " in ");
+
+			if(tmp != NULL)
+			{
+				data.string = tmp;
+				data.pattern = " ";
+				data.min_fields = 6;
+				data.line_idx = 5;
+				data.file_idx = 2;
+			}
+			else
+			{
+				data.pattern = " ";
+				data.min_fields = 11;
+				data.line_idx = 10;
+				data.file_idx = 7;
+			}
 			break;
 		}
 		case GEANY_FILETYPES_PERL:
@@ -526,11 +677,15 @@ void msgwin_parse_compiler_error_line(const gchar *string, gchar **filename, gin
 		case GEANY_FILETYPES_MAKE:	// Assume makefile is building with gcc
 		default:	// The default is a GNU gcc type error
 		{
-			data.pattern = ":";
-			data.min_fields = 3;
-			data.line_idx = 1;
-			data.file_idx = 0;
-			break;
+			// don't accidently find libtool versions x:y:x and think it is a file name
+			if (strstr(string, "libtool --mode=link") == NULL)
+			{
+				data.pattern = ":";
+				data.min_fields = 3;
+				data.line_idx = 1;
+				data.file_idx = 0;
+				break;
+			}
 		}
 	}
 
@@ -554,7 +709,9 @@ gboolean msgwin_goto_messages_file_line()
 
 		gtk_tree_model_get(model, &iter, 0, &line, 1, &idx, 3, &string, -1);
 		if (line >= 0 && idx >= 0)
-			utils_goto_line(idx, line); //checks valid idx
+		{
+			ret = utils_goto_line(idx, line);	// checks valid idx
+		}
 		else if (line < 0 && string != NULL)
 		{
 			gchar *filename;
@@ -601,3 +758,53 @@ static void msgwin_parse_grep_line(const gchar *string, gchar **filename, gint *
 }
 
 
+static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *event,
+																			gpointer user_data)
+{
+	// user_data might be NULL, GPOINTER_TO_INT returns 0 if called with NULL
+
+	if (event->button == 1)
+	{
+		switch (GPOINTER_TO_INT(user_data))
+		{
+			case MSG_COMPILER:
+			{	// single click in the compiler treeview
+				msgwin_goto_compiler_file_line();
+				break;
+			}
+			case MSG_MESSAGE:
+			{	// single click in the message treeview (results of 'Find usage')
+				msgwin_goto_messages_file_line();
+				break;
+			}
+		}
+	}
+
+	if (event->button == 3)
+	{	// popupmenu to hide or clear the active treeview
+		switch (GPOINTER_TO_INT(user_data))
+		{
+			case MSG_STATUS:
+			{
+				gtk_menu_popup(GTK_MENU(msgwindow.popup_status_menu), NULL, NULL, NULL, NULL,
+																	event->button, event->time);
+				break;
+			}
+			case MSG_MESSAGE:
+			{
+				gtk_menu_popup(GTK_MENU(msgwindow.popup_msg_menu), NULL, NULL, NULL, NULL,
+																	event->button, event->time);
+				break;
+			}
+			case MSG_COMPILER:
+			{
+				gtk_menu_popup(GTK_MENU(msgwindow.popup_compiler_menu), NULL, NULL, NULL, NULL,
+																	event->button, event->time);
+				break;
+			}
+		}
+	}
+	return FALSE;
+}
+
+
