@@ -1,7 +1,8 @@
 /*
  *      msgwindow.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: msgwindow.h 802 2006-09-08 13:39:32Z ntrel $
+ * $Id: msgwindow.h 1269 2007-02-12 17:21:57Z eht16 $
  */
 
 
@@ -35,14 +36,15 @@ enum
 
 enum
 {
-	MSG_STATUS = 0,
+	MSG_STATUS = 0, // force it to start at 0 to keep in sync with the notebook page numbers
 	MSG_COMPILER,
 	MSG_MESSAGE,
 	MSG_SCRATCH,
-	MSG_VTE
+	MSG_VTE,
 };
 
 
+
 typedef struct
 {
 	GtkListStore	*store_status;
@@ -65,31 +67,20 @@ void msgwin_init();
 
 void msgwin_finalize();
 
-void msgwin_prepare_msg_tree_view(void);
+void msgwin_show_hide(gboolean show);
 
-void msgwin_prepare_status_tree_view(void);
+void msgwin_msg_add(gint line, gint idx, const gchar *string);
 
-void msgwin_prepare_compiler_tree_view(void);
+void msgwin_compiler_add_fmt(gint msg_color, const gchar *format, ...) G_GNUC_PRINTF (2, 3);
 
-void msgwin_show();
+void msgwin_compiler_add(gint msg_color, const gchar *msg);
 
-void msgwin_msg_add(gint line, gint idx, gchar *string);
+void msgwin_status_add(const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 
-void msgwin_compiler_add(gint msg_color, gboolean scroll, gchar const *format, ...);
-
-void msgwin_status_add(gchar const *format, ...);
-
-void msgwin_set_build_info(const gchar *dir, guint file_type_id);
-
-GtkWidget *msgwin_create_message_popup_menu(gint type);
+void msgwin_menu_add_common_items(GtkMenu *menu);
 
 gboolean msgwin_goto_compiler_file_line();
 
-/* try to parse the file and line number where the error occured described in line
- * and when something useful is found, it stores the line number in *line and the
- * relevant file with the error in filename.
- * *line will be -1 if no error was found in string.
- * filename must be freed unless it is NULL. */
 void msgwin_parse_compiler_error_line(const gchar *string, gchar **filename, gint *line);
 
 gboolean msgwin_goto_messages_file_line();
