@@ -17,7 +17,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: prefs.c 811 2006-09-11 11:13:36Z eht16 $
+ * $Id: prefs.c 1252 2007-02-06 17:39:19Z ntrel $
  */
 
 
@@ -38,12 +38,15 @@
 #include "keybindings.h"
 #include "interface.h"
 #include "encodings.h"
-#include "callbacks.h"
 
 #ifdef HAVE_VTE
 # include "vte.h"
 #endif
 
+#ifdef G_OS_WIN32
+# include "win32.h"
+#endif
+
 
 gchar *dialog_key_name;
 static GtkListStore *store = NULL;
@@ -57,6 +60,9 @@ static void on_cell_edited(GtkCellRendererText *cellrenderertext, gchar *path, g
 static gboolean on_keytype_dialog_response(GtkWidget *dialog, GdkEventKey *event, gpointer user_data);
 static void on_dialog_response(GtkWidget *dialog, gint response, gpointer user_data);
 static gboolean find_duplicate(guint idx, guint key, GdkModifierType mods, const gchar *action);
+static void on_pref_toolbar_show_toggled(GtkToggleButton *togglebutton, gpointer user_data);
+static void on_pref_show_notebook_tabs_toggled(GtkToggleButton *togglebutton, gpointer user_data);
+static void on_pref_use_folding_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 
 
 void prefs_init_dialog(void)
@@ -86,6 +92,15 @@ void prefs_init_dialog(void)
 	widget = lookup_widget(app->prefs_dialog, "check_ask_for_quit");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), app->pref_main_confirm_exit);
 
+	widget = lookup_widget(app->prefs_dialog, "check_ask_suppress_search_dialogs");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), app->pref_main_suppress_search_dialogs);
+
+	widget = lookup_widget(app->prefs_dialog, "check_show_notebook_tabs");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), app->show_notebook_tabs);
+	// disable following setting if notebook tabs are hidden
+	on_pref_show_notebook_tabs_toggled(GTK_TOGGLE_BUTTON(
+					lookup_widget(app->prefs_dialog, "check_show_notebook_tabs")), NULL);
+
 	if (app->tab_order_ltr)
 		widget = lookup_widget(app->prefs_dialog, "radio_tab_right");
 	else
@@ -161,6 +176,9 @@ void prefs_init_dialog(void)
 	widget = lookup_widget(app->prefs_dialog, "check_toolbar_fileops");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), app->pref_toolbar_show_fileops);
 
+	widget = lookup_widget(app->prefs_dialog, "check_toolbar_quit");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), app->pref_toolbar_show_quit);
+
 
 	switch (app->toolbar_icon_style)
 	{
@@ -224,6 +242,11 @@ void prefs_init_dialog(void)
 	widget = lookup_widget(app->prefs_dialog, "check_folding");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), app->pref_editor_folding);
 
+	widget = lookup_widget(app->prefs_dialog, "check_unfold_children");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), app->pref_editor_unfold_all_children);
+	on_pref_use_folding_toggled(GTK_TOGGLE_BUTTON(
+					lookup_widget(app->prefs_dialog, "check_folding")), NULL);
+
 	widget = lookup_widget(app->prefs_dialog, "check_indicators");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), app->pref_editor_use_indicators);
 
@@ -352,21 +375,23 @@ void prefs_init_dialog(void)
 
 		widget = lookup_widget(app->prefs_dialog, "check_follow_path");
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), vc->follow_path);
+
+		widget = lookup_widget(app->prefs_dialog, "check_run_in_vte");
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), vc->run_in_vte);
 	}
 #endif
 }
 
-//gtk_notebook_set_tab_pos
 
 /*
  * callbacks
  */
 void on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 {
-	if (response == GTK_RESPONSE_OK)
+	if (response == GTK_RESPONSE_OK || response == GTK_RESPONSE_APPLY)
 	{
 		GtkWidget *widget;
-		gint i;
+		guint i;
 
 		// General settings
 		widget = lookup_widget(app->prefs_dialog, "spin_mru");
@@ -384,12 +409,18 @@ void on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_dat
 		widget = lookup_widget(app->prefs_dialog, "check_ask_for_quit");
 		app->pref_main_confirm_exit = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+		widget = lookup_widget(app->prefs_dialog, "check_ask_suppress_search_dialogs");
+		app->pref_main_suppress_search_dialogs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 		widget = lookup_widget(app->prefs_dialog, "check_switch_pages");
 		app->switch_msgwin_pages = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
 		widget = lookup_widget(app->prefs_dialog, "radio_tab_right");
 		app->tab_order_ltr = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+		widget = lookup_widget(app->prefs_dialog, "check_show_notebook_tabs");
+		app->show_notebook_tabs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 
 		// Interface settings
 		widget = lookup_widget(app->prefs_dialog, "check_list_symbol");
@@ -446,6 +477,9 @@ void on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_dat
 		widget = lookup_widget(app->prefs_dialog, "check_toolbar_fileops");
 		app->pref_toolbar_show_fileops = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+		widget = lookup_widget(app->prefs_dialog, "check_toolbar_quit");
+		app->pref_toolbar_show_quit = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 		widget = lookup_widget(app->prefs_dialog, "radio_toolbar_imagetext");
 		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) app->toolbar_icon_style = 2;
 		else
@@ -489,6 +523,9 @@ void on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_dat
 		app->pref_editor_folding = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 		ui_update_fold_items();
 
+		widget = lookup_widget(app->prefs_dialog, "check_unfold_children");
+		app->pref_editor_unfold_all_children = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 		widget = lookup_widget(app->prefs_dialog, "check_indent");
 		app->pref_editor_show_indent_guide = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
@@ -594,6 +631,9 @@ void on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_dat
 			widget = lookup_widget(app->prefs_dialog, "check_follow_path");
 			vc->follow_path = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+			widget = lookup_widget(app->prefs_dialog, "check_run_in_vte");
+			vc->run_in_vte = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 			vte_apply_user_settings();
 		}
 #endif
@@ -603,13 +643,14 @@ void on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_dat
 		ui_update_toolbar_icons(app->toolbar_icon_size);
 		gtk_toolbar_set_style(GTK_TOOLBAR(app->toolbar), app->toolbar_icon_style);
 		ui_treeviews_show_hide(FALSE);
+		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(app->notebook), app->show_notebook_tabs);
 
 		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(app->notebook), app->tab_pos_editor);
 		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(msgwindow.notebook), app->tab_pos_msgwin);
 		gtk_notebook_set_tab_pos(GTK_NOTEBOOK(app->treeview_notebook), app->tab_pos_sidebar);
 
 		// re-colourise all open documents, if tab width or long line settings have changed
-		for (i = 0; i < GEANY_MAX_OPEN_FILES; i++)
+		for (i = 0; i < doc_array->len; i++)
 		{
 			if (doc_list[i].is_valid)
 			{
@@ -621,16 +662,12 @@ void on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_dat
 		// store all settings
 		configuration_save();
 	}
-	gtk_list_store_clear(store);
-	gtk_widget_hide(GTK_WIDGET(dialog));
-}
 
-
-gboolean on_prefs_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
-{
-	gtk_widget_hide(widget);
-
-	return TRUE;
+	if (response != GTK_RESPONSE_APPLY)
+	{
+		gtk_list_store_clear(store);
+		gtk_widget_hide(GTK_WIDGET(dialog));
+	}
 }
 
 
@@ -669,7 +706,7 @@ void on_prefs_color_choosed(GtkColorButton *widget, gpointer user_data)
 void on_prefs_font_choosed(GtkFontButton *widget, gpointer user_data)
 {
 	const gchar *fontbtn = gtk_font_button_get_font_name(widget);
-	gint i;
+	guint i;
 
 	switch (GPOINTER_TO_INT(user_data))
 	{
@@ -678,17 +715,16 @@ void on_prefs_font_choosed(GtkFontButton *widget, gpointer user_data)
 			if (strcmp(fontbtn, app->tagbar_font) == 0) break;
 			g_free(app->tagbar_font);
 			app->tagbar_font = g_strdup(fontbtn);
-			for (i = 0; i < GEANY_MAX_OPEN_FILES; i++)
+			for (i = 0; i < doc_array->len; i++)
 			{
 				if (doc_list[i].is_valid && GTK_IS_WIDGET(doc_list[i].tag_tree))
-					gtk_widget_modify_font(doc_list[i].tag_tree,
-						pango_font_description_from_string(app->tagbar_font));
+					ui_widget_modify_font_from_string(doc_list[i].tag_tree,
+						app->tagbar_font);
 			}
 			if (GTK_IS_WIDGET(app->default_tag_tree))
-				gtk_widget_modify_font(app->default_tag_tree,
-					pango_font_description_from_string(app->tagbar_font));
-			gtk_widget_modify_font(lookup_widget(app->window, "entry1"),
-				pango_font_description_from_string(app->tagbar_font));
+				ui_widget_modify_font_from_string(app->default_tag_tree, app->tagbar_font);
+			ui_widget_modify_font_from_string(lookup_widget(app->window, "entry1"),
+				app->tagbar_font);
 			break;
 		}
 		case 2:
@@ -696,12 +732,9 @@ void on_prefs_font_choosed(GtkFontButton *widget, gpointer user_data)
 			if (strcmp(fontbtn, app->msgwin_font) == 0) break;
 			g_free(app->msgwin_font);
 			app->msgwin_font = g_strdup(fontbtn);
-			gtk_widget_modify_font(msgwindow.tree_compiler,
-				pango_font_description_from_string(app->msgwin_font));
-			gtk_widget_modify_font(msgwindow.tree_msg,
-				pango_font_description_from_string(app->msgwin_font));
-			gtk_widget_modify_font(msgwindow.tree_status,
-				pango_font_description_from_string(app->msgwin_font));
+			ui_widget_modify_font_from_string(msgwindow.tree_compiler, app->msgwin_font);
+			ui_widget_modify_font_from_string(msgwindow.tree_msg, app->msgwin_font);
+			ui_widget_modify_font_from_string(msgwindow.tree_status, app->msgwin_font);
 			break;
 		}
 		case 3:
@@ -724,7 +757,6 @@ void on_prefs_font_choosed(GtkFontButton *widget, gpointer user_data)
 }
 
 
-
 static gboolean on_prefs_tree_view_button_press_event(
 						GtkWidget *widget, GdkEventButton *event, gpointer user_data)
 {
@@ -834,7 +866,7 @@ static void on_dialog_response(GtkWidget *dialog, gint response, gpointer user_d
 
 		for (idx = 0; idx < GEANY_MAX_KEYS; idx++)
 		{
-			if (utils_strcmp(dialog_key_name, keys[idx]->label)) break;
+			if (utils_str_equal(dialog_key_name, keys[idx]->label)) break;
 		}
 
 		gtk_accelerator_parse(gtk_label_get_text(GTK_LABEL(dialog_label)), &lkey, &lmods);
@@ -873,7 +905,7 @@ static gboolean find_duplicate(guint idx, guint key, GdkModifierType mods, const
 		if (keys[i]->key == key && keys[i]->mods == mods
 			&& ! (keys[i]->key == keys[idx]->key && keys[i]->mods == keys[idx]->mods))
 		{
-			dialogs_show_error(
+			dialogs_show_msgbox(GTK_MESSAGE_ERROR,
 				_("The combination '%s' is already used for \"%s\". Please choose another one."),
 				action, keys[i]->label);
 			return TRUE;
@@ -883,7 +915,7 @@ static gboolean find_duplicate(guint idx, guint key, GdkModifierType mods, const
 }
 
 
-void on_pref_toolbar_show_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+static void on_pref_toolbar_show_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 {
 	gboolean sens = gtk_toggle_button_get_active(togglebutton);
 
@@ -892,6 +924,23 @@ void on_pref_toolbar_show_toggled(GtkToggleButton *togglebutton, gpointer user_d
 }
 
 
+static void on_pref_show_notebook_tabs_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+{
+	gboolean sens = gtk_toggle_button_get_active(togglebutton);
+
+	// tab placement only enabled when tabs are visible
+	gtk_widget_set_sensitive(lookup_widget(app->prefs_dialog, "combo_tab_editor"), sens);
+}
+
+
+static void on_pref_use_folding_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+{
+	gboolean sens = gtk_toggle_button_get_active(togglebutton);
+
+	gtk_widget_set_sensitive(lookup_widget(app->prefs_dialog, "check_unfold_children"), sens);
+}
+
+
 void dialogs_show_prefs_dialog(void)
 {
 	if (app->prefs_dialog == NULL)
@@ -914,190 +963,13 @@ void dialogs_show_prefs_dialog(void)
 		}
 
 #ifdef HAVE_VTE
-		if (vte_info.have_vte)
-		{
-			GtkWidget *notebook, *vbox, *label, *alignment, *table;
-			GtkWidget *font_term, *color_fore, *color_back, *spin_scrollback, *entry_emulation;
-			GtkWidget *check_scroll_key, *check_scroll_out, *check_follow_path, *check_ignore_menu_key;
-			GtkWidget *entry_shell, *button_shell, *image_shell;
-			GtkTooltips *tooltips;
-			GtkObject *spin_scrollback_adj;
-
-			tooltips = GTK_TOOLTIPS(lookup_widget(app->prefs_dialog, "tooltips"));
-			notebook = lookup_widget(app->prefs_dialog, "notebook2");
-			vbox = gtk_vbox_new(FALSE, 0);
-			gtk_container_add(GTK_CONTAINER(notebook), vbox);
-
-			label = gtk_label_new(_("These are settings for the virtual terminal emulator widget (VTE). They only apply, if the VTE library could be loaded."));
-			gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-			gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_FILL);
-			gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-			gtk_misc_set_alignment(GTK_MISC(label), 0.14, 0.19);
-			gtk_misc_set_padding(GTK_MISC(label), 0, 8);
-
-			alignment = gtk_alignment_new(0.5, 0.5, 1, 1);
-			gtk_box_pack_start(GTK_BOX(vbox), alignment, FALSE, FALSE, 0);
-			gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 0, 0, 12, 6);
-
-			table = gtk_table_new(10, 3, FALSE);
-			gtk_container_add(GTK_CONTAINER(alignment), table);
-			gtk_table_set_row_spacings(GTK_TABLE(table), 3);
-			gtk_table_set_col_spacings(GTK_TABLE(table), 10);
-
-			label = gtk_label_new(_("Terminal font"));
-			gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
-						(GtkAttachOptions) (GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-
-			font_term = gtk_font_button_new();
-			gtk_table_attach(GTK_TABLE(table), font_term, 1, 2, 0, 1,
-						(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_tooltips_set_tip(tooltips, font_term, _("Sets the font for the terminal widget."), NULL);
-
-			label = gtk_label_new(_("Foreground color"));
-			gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
-						(GtkAttachOptions) (GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-
-			label = gtk_label_new(_("Background color"));
-			gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
-						(GtkAttachOptions) (GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-
-			color_fore = gtk_color_button_new();
-			gtk_table_attach(GTK_TABLE(table), color_fore, 1, 2, 1, 2,
-						(GtkAttachOptions) (GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_tooltips_set_tip(tooltips, color_fore, _("Sets the foreground color of the text in the terminal widget."), NULL);
-			gtk_color_button_set_title(GTK_COLOR_BUTTON(color_fore), _("Color Chooser"));
-
-			color_back = gtk_color_button_new();
-			gtk_table_attach(GTK_TABLE(table), color_back, 1, 2, 2, 3,
-						(GtkAttachOptions) (GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_tooltips_set_tip(tooltips, color_back, _("Sets the background color of the text in the terminal widget."), NULL);
-			gtk_color_button_set_title(GTK_COLOR_BUTTON(color_back), _("Color Chooser"));
-
-			label = gtk_label_new(_("Scrollback lines"));
-			gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
-						(GtkAttachOptions) (GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-
-			spin_scrollback_adj = gtk_adjustment_new(500, 0, 5000, 1, 10, 10);
-			spin_scrollback = gtk_spin_button_new(GTK_ADJUSTMENT(spin_scrollback_adj), 1, 0);
-			gtk_table_attach(GTK_TABLE(table), spin_scrollback, 1, 2, 3, 4,
-						(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_tooltips_set_tip(tooltips, spin_scrollback, _("Specifies the history in lines, which you can scroll back in the terminal widget."), NULL);
-			gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin_scrollback), TRUE);
-			gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin_scrollback), TRUE);
-
-			label = gtk_label_new(_("Terminal emulation"));
-			gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5,
-						(GtkAttachOptions) (GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-
-			entry_emulation = gtk_entry_new();
-			gtk_table_attach(GTK_TABLE(table), entry_emulation, 1, 2, 4, 5,
-						(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_tooltips_set_tip(tooltips, entry_emulation, _("Controls how the terminal emulator should behave. xterm is a good start."), NULL);
-
-			label = gtk_label_new(_("Shell"));
-			gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6,
-						(GtkAttachOptions) (GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-
-			entry_shell = gtk_entry_new();
-			gtk_table_attach(GTK_TABLE(table), entry_shell, 1, 2, 5, 6,
-						(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_tooltips_set_tip(tooltips, entry_shell, _("Sets the path to the shell which should be started inside the terminal emulation."), NULL);
-
-			button_shell = gtk_button_new();
-			gtk_widget_show(button_shell);
-			gtk_table_attach(GTK_TABLE(table), button_shell, 2, 3, 5, 6,
-						(GtkAttachOptions) (GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-
-			image_shell = gtk_image_new_from_stock("gtk-open", GTK_ICON_SIZE_BUTTON);
-			gtk_widget_show(image_shell);
-			gtk_container_add(GTK_CONTAINER(button_shell), image_shell);
-
-			check_scroll_key = gtk_check_button_new_with_mnemonic(_("Scroll on keystroke"));
-			gtk_table_attach(GTK_TABLE(table), check_scroll_key, 1, 2, 6, 7,
-						(GtkAttachOptions) (GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_tooltips_set_tip(tooltips, check_scroll_key, _("Whether to scroll to the bottom if a key was pressed."), NULL);
-			gtk_button_set_focus_on_click(GTK_BUTTON(check_scroll_key), FALSE);
-
-			check_scroll_out = gtk_check_button_new_with_mnemonic(_("Scroll on output"));
-			gtk_table_attach(GTK_TABLE(table), check_scroll_out, 1, 2, 7, 8,
-						(GtkAttachOptions) (GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_tooltips_set_tip(tooltips, check_scroll_out, _("Whether to scroll to the bottom if an output was generated."), NULL);
-			gtk_button_set_focus_on_click(GTK_BUTTON(check_scroll_out), FALSE);
-
-			check_ignore_menu_key = gtk_check_button_new_with_mnemonic(_("Disable menu shortcut key (F10 by default)"));
-			gtk_table_attach(GTK_TABLE(table), check_ignore_menu_key, 1, 2, 8, 9,
-						(GtkAttachOptions) (GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_tooltips_set_tip(tooltips, check_ignore_menu_key, _("This option disables the keybinding to popup the menu bar (default is F10). Disabling it can be useful if you use, for example, Midnight Commander within the VTE."), NULL);
-			gtk_button_set_focus_on_click(GTK_BUTTON(check_ignore_menu_key), FALSE);
-
-			check_follow_path = gtk_check_button_new_with_mnemonic(_("Follow the path of the current file"));
-			gtk_table_attach(GTK_TABLE(table), check_follow_path, 1, 2, 9, 10,
-						(GtkAttachOptions) (GTK_FILL),
-						(GtkAttachOptions) (0), 0, 0);
-			gtk_tooltips_set_tip(tooltips, check_follow_path, _("Whether to execute \"cd $path\" when you switch between opened files."), NULL);
-			gtk_button_set_focus_on_click(GTK_BUTTON(check_follow_path), FALSE);
-
-			label = gtk_label_new(_("Terminal"));
-			gtk_notebook_set_tab_label(GTK_NOTEBOOK(notebook), gtk_notebook_get_nth_page(
-						GTK_NOTEBOOK(notebook), 7), label);
-
-			g_object_set_data_full(G_OBJECT(app->prefs_dialog), "font_term",
-					gtk_widget_ref(font_term),	(GDestroyNotify) gtk_widget_unref);
-			g_object_set_data_full(G_OBJECT(app->prefs_dialog), "color_fore",
-					gtk_widget_ref(color_fore),	(GDestroyNotify) gtk_widget_unref);
-			g_object_set_data_full(G_OBJECT(app->prefs_dialog), "color_back",
-					gtk_widget_ref(color_back),	(GDestroyNotify) gtk_widget_unref);
-			g_object_set_data_full(G_OBJECT(app->prefs_dialog), "spin_scrollback",
-					gtk_widget_ref(spin_scrollback),	(GDestroyNotify) gtk_widget_unref);
-			g_object_set_data_full(G_OBJECT(app->prefs_dialog), "entry_emulation",
-					gtk_widget_ref(entry_emulation),	(GDestroyNotify) gtk_widget_unref);
-			g_object_set_data_full(G_OBJECT(app->prefs_dialog), "entry_shell",
-					gtk_widget_ref(entry_shell),	(GDestroyNotify) gtk_widget_unref);
-			g_object_set_data_full(G_OBJECT(app->prefs_dialog), "check_scroll_key",
-					gtk_widget_ref(check_scroll_key),	(GDestroyNotify) gtk_widget_unref);
-			g_object_set_data_full(G_OBJECT(app->prefs_dialog), "check_scroll_out",
-					gtk_widget_ref(check_scroll_out),	(GDestroyNotify) gtk_widget_unref);
-			g_object_set_data_full(G_OBJECT(app->prefs_dialog), "check_ignore_menu_key",
-					gtk_widget_ref(check_ignore_menu_key),	(GDestroyNotify) gtk_widget_unref);
-			g_object_set_data_full(G_OBJECT(app->prefs_dialog), "check_follow_path",
-					gtk_widget_ref(check_follow_path),	(GDestroyNotify) gtk_widget_unref);
-
-			gtk_widget_show_all(vbox);
-
-			g_signal_connect((gpointer) font_term, "font-set", G_CALLBACK(on_prefs_font_choosed),
-															   GINT_TO_POINTER(4));
-			g_signal_connect((gpointer) color_fore, "color-set", G_CALLBACK(on_prefs_color_choosed),
-																 GINT_TO_POINTER(2));
-			g_signal_connect((gpointer) color_back, "color-set", G_CALLBACK(on_prefs_color_choosed),
-																 GINT_TO_POINTER(3));
-			g_signal_connect((gpointer) button_shell, "clicked",
-					G_CALLBACK(on_pref_tools_button_clicked), entry_shell);
-		}
+		vte_append_preferences_tab();
 #endif
-		g_signal_connect((gpointer) app->prefs_dialog, "response", G_CALLBACK(on_prefs_button_clicked), NULL);
-		g_signal_connect((gpointer) app->prefs_dialog, "delete_event", G_CALLBACK(on_prefs_delete_event), NULL);
+
+		g_signal_connect((gpointer) app->prefs_dialog, "response",
+			G_CALLBACK(on_prefs_button_clicked), NULL);
+		g_signal_connect((gpointer) app->prefs_dialog, "delete_event",
+			G_CALLBACK(gtk_widget_hide_on_delete), NULL);
 		g_signal_connect((gpointer) lookup_widget(app->prefs_dialog, "tagbar_font"),
 				"font-set", G_CALLBACK(on_prefs_font_choosed), GINT_TO_POINTER(1));
 		g_signal_connect((gpointer) lookup_widget(app->prefs_dialog, "msgwin_font"),
@@ -1108,21 +980,75 @@ void dialogs_show_prefs_dialog(void)
 				"color-set", G_CALLBACK(on_prefs_color_choosed), GINT_TO_POINTER(1));
 		// file chooser buttons in the tools tab
 		g_signal_connect((gpointer) lookup_widget(app->prefs_dialog, "button_make"),
-				"clicked", G_CALLBACK(on_pref_tools_button_clicked), lookup_widget(app->prefs_dialog, "entry_com_make"));
+				"clicked", G_CALLBACK(on_prefs_tools_button_clicked), lookup_widget(app->prefs_dialog, "entry_com_make"));
 		g_signal_connect((gpointer) lookup_widget(app->prefs_dialog, "button_term"),
-				"clicked", G_CALLBACK(on_pref_tools_button_clicked), lookup_widget(app->prefs_dialog, "entry_com_term"));
+				"clicked", G_CALLBACK(on_prefs_tools_button_clicked), lookup_widget(app->prefs_dialog, "entry_com_term"));
 		g_signal_connect((gpointer) lookup_widget(app->prefs_dialog, "button_browser"),
-				"clicked", G_CALLBACK(on_pref_tools_button_clicked), lookup_widget(app->prefs_dialog, "entry_browser"));
+				"clicked", G_CALLBACK(on_prefs_tools_button_clicked), lookup_widget(app->prefs_dialog, "entry_browser"));
 		g_signal_connect((gpointer) lookup_widget(app->prefs_dialog, "button_print"),
-				"clicked", G_CALLBACK(on_pref_tools_button_clicked), lookup_widget(app->prefs_dialog, "entry_print"));
+				"clicked", G_CALLBACK(on_prefs_tools_button_clicked), lookup_widget(app->prefs_dialog, "entry_print"));
 		g_signal_connect((gpointer) lookup_widget(app->prefs_dialog, "button_grep"),
-				"clicked", G_CALLBACK(on_pref_tools_button_clicked), lookup_widget(app->prefs_dialog, "entry_grep"));
+				"clicked", G_CALLBACK(on_prefs_tools_button_clicked), lookup_widget(app->prefs_dialog, "entry_grep"));
 
 		g_signal_connect((gpointer) lookup_widget(app->prefs_dialog, "check_toolbar_show"),
 				"toggled", G_CALLBACK(on_pref_toolbar_show_toggled), NULL);
-
+		g_signal_connect((gpointer) lookup_widget(app->prefs_dialog, "check_show_notebook_tabs"),
+				"toggled", G_CALLBACK(on_pref_show_notebook_tabs_toggled), NULL);
+		g_signal_connect((gpointer) lookup_widget(app->prefs_dialog, "check_folding"),
+				"toggled", G_CALLBACK(on_pref_use_folding_toggled), NULL);
 	}
 
 	prefs_init_dialog();
 	gtk_widget_show(app->prefs_dialog);
 }
+
+
+void
+on_prefs_tools_button_clicked           (GtkButton       *button,
+                                        gpointer         item)
+{
+#ifdef G_OS_WIN32
+	win32_show_pref_file_dialog(item);
+#else
+	GtkWidget *dialog;
+	gchar *filename, *tmp, **field;
+
+	// initialize the dialog
+	dialog = gtk_file_chooser_dialog_new(_("Open File"), GTK_WINDOW(app->prefs_dialog),
+					GTK_FILE_CHOOSER_ACTION_OPEN,
+					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
+	// cut the options from the command line
+	field = g_strsplit(gtk_entry_get_text(GTK_ENTRY(item)), " ", 2);
+	if (field[0])
+	{
+		filename = g_find_program_in_path(field[0]);
+		if (filename)
+		{
+			gtk_file_chooser_select_filename(GTK_FILE_CHOOSER(dialog), filename);
+			g_free(filename);
+		}
+	}
+
+	// run it
+	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
+	{
+		tmp = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+		if (g_strv_length(field) > 1)
+			filename = g_strconcat(tmp, " ", field[1], NULL);
+		else
+		{
+			filename = tmp;
+			tmp = NULL;
+		}
+		gtk_entry_set_text(GTK_ENTRY(item), filename);
+		g_free(filename);
+		g_free(tmp);
+	}
+
+	g_strfreev(field);
+	gtk_widget_destroy(dialog);
+#endif
+}
+
+
