@@ -29,6 +29,7 @@
 #include "Scintilla.h"
 #include "ScintillaWidget.h"
 
+// Note: Avoid using SSM in files not related to scintilla, use sciwrappers.h instead.
 #define SSM(s, m, w, l) scintilla_send_message(s, m, w, l)
 
 
@@ -40,8 +41,6 @@ typedef struct
 
 extern EditorInfo editor_info;
 
-gchar **html_entities;
-
 
 gboolean
 on_editor_button_press_event           (GtkWidget *widget,
@@ -53,8 +52,6 @@ void on_editor_notification(GtkWidget* editor, gint scn, gpointer lscn, gpointer
 
 gboolean sci_cb_start_auto_complete(gint idx, gint pos, gboolean force);
 
-void sci_cb_get_indent(ScintillaObject *sci, gint pos, gboolean use_this_line);
-
 void sci_cb_close_block(gint idx, gint pos);
 
 void sci_cb_auto_forif(gint idx, gint pos);
@@ -63,8 +60,9 @@ void sci_cb_auto_latex(gint idx, gint pos);
 
 void sci_cb_show_macro_list(ScintillaObject *sci);
 
-gboolean sci_cb_handle_xml(ScintillaObject *sci, gchar ch);
-
+/* Reads the word at given cursor position and writes it into the given buffer. The buffer will be
+ * NULL terminated in any case, even when the word is truncated because wordlen is too small.
+ * position can be -1, then the current position is used. */
 void sci_cb_find_current_word(ScintillaObject *sci, gint pos, gchar *word, size_t wordlen);
 
 gboolean sci_cb_show_calltip(gint idx, gint pos);
@@ -77,10 +75,12 @@ void sci_cb_do_uncomment(gint idx, gint line);
 
 void sci_cb_highlight_braces(ScintillaObject *sci, gint cur_pos);
 
-void sci_cb_auto_multiline(ScintillaObject *sci, gint pos);
-
 void sci_cb_auto_table(ScintillaObject *sci, gint pos);
 
 void sci_cb_auto_close_bracket(ScintillaObject *sci, gint pos, gchar c);
 
+gboolean sci_cb_lexer_is_c_like(gint lexer);
+
+gint sci_cb_lexer_get_type_keyword_idx(gint lexer);
+
 #endif
