@@ -1,7 +1,8 @@
 /*
  *      sciwrappers.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: sciwrappers.c 829 2006-09-21 10:31:06Z ntrel $
+ * $Id: sciwrappers.c 1270 2007-02-12 18:42:15Z eht16 $
  */
 
 #include <string.h>
@@ -273,6 +274,12 @@ void sci_empty_undo_buffer( ScintillaObject* sci )
 }
 
 
+gboolean sci_is_modified(ScintillaObject *sci)
+{
+	return (SSM(sci, SCI_GETMODIFY, 0, 0) != 0);
+}
+
+
 void sci_zoom_in( ScintillaObject* sci )
 {
 	SSM( sci, SCI_ZOOMIN,0,0);
@@ -373,9 +380,15 @@ gint sci_get_current_position(ScintillaObject* sci )
 }
 
 
-void sci_set_current_position(ScintillaObject* sci, gint position )
+void sci_set_current_position(ScintillaObject* sci, gint position, gboolean scroll_to_caret)
 {
-	SSM(sci, SCI_GOTOPOS, position, 0);
+	if (scroll_to_caret)
+		SSM(sci, SCI_GOTOPOS, position, 0);
+	else
+	{
+		SSM(sci, SCI_SETCURRENTPOS, position, 0);
+		SSM(sci, SCI_SETANCHOR, position, 0); // to avoid creation of a selection
+	}
 }
 
 
@@ -469,11 +482,18 @@ gint sci_get_line_length(ScintillaObject* sci,gint line)
 }
 
 
-void sci_get_line(ScintillaObject* sci, gint line, gchar* text)
+// Returns: a NULL-terminated copy of the line text
+gchar *sci_get_line(ScintillaObject* sci, gint line_num)
 {
-	SSM(sci,SCI_GETLINE, line, (sptr_t) text);
+	gint len = sci_get_line_length(sci, line_num);
+	gchar *linebuf = g_malloc(len + 1);
+
+	SSM(sci, SCI_GETLINE, line_num, (sptr_t) linebuf);
+	linebuf[len] = '\0';
+	return linebuf;
 }
 
+
 // the last char will be null terminated
 void sci_get_text(ScintillaObject* sci, gint len, gchar* text)
 {
@@ -526,6 +546,20 @@ gint sci_get_fold_level(ScintillaObject* sci, gint line)
 }
 
 
+/* Get the next line after start_line with fold level <= level */
+gint sci_get_last_child(ScintillaObject* sci, gint start_line, gint level)
+{
+	return SSM( sci, SCI_GETLASTCHILD, start_line, level);
+}
+
+
+/* Get the line number of the fold point before start_line, or -1 if there isn't one */
+gint sci_get_fold_parent(ScintillaObject* sci, gint start_line)
+{
+	return SSM( sci, SCI_GETFOLDPARENT, start_line, 0);
+}
+
+
 void sci_toggle_fold(ScintillaObject* sci, gint line)
 {
 	SSM( sci, SCI_TOGGLEFOLD, line, 1);
@@ -614,9 +648,9 @@ gboolean sci_can_copy(ScintillaObject *sci)
 }
 
 
-void sci_goto_pos(ScintillaObject *sci, gint pos, gboolean ensure_visibility)
+void sci_goto_pos(ScintillaObject *sci, gint pos, gboolean unfold)
 {
-	if (ensure_visibility) SSM(sci,SCI_ENSUREVISIBLE,SSM(sci, SCI_LINEFROMPOSITION, pos, 0),0);
+	if (unfold) SSM(sci,SCI_ENSUREVISIBLE,SSM(sci, SCI_LINEFROMPOSITION, pos, 0),0);
 	SSM(sci, SCI_GOTOPOS, pos, 0);
 }
 
@@ -639,6 +673,12 @@ void sci_scroll_caret(ScintillaObject *sci)
 }
 
 
+void sci_scroll_lines(ScintillaObject *sci, gint lines)
+{
+	SSM(sci, SCI_LINESCROLL, 0, lines);
+}
+
+
 gint sci_search_next(ScintillaObject *sci, gint flags, const gchar *text)
 {
 	return SSM(sci, SCI_SEARCHNEXT, flags, (sptr_t) text );
@@ -671,21 +711,6 @@ void sci_goto_line(ScintillaObject *sci, gint line, gboolean ensure_visibility)
 }
 
 
-void sci_goto_line_scroll(ScintillaObject *sci, gint line, gdouble percent_of_view)
-{
-	gint vis1, los, delta;
-
-	sci_goto_line(sci, line, TRUE);
-	// sci 'visible line' != file line number
-	vis1 = SSM(sci, SCI_GETFIRSTVISIBLELINE, 0, 0);
-	vis1 = SSM(sci, SCI_DOCLINEFROMVISIBLE, vis1, 0);
-	los = SSM(sci, SCI_LINESONSCREEN, 0, 0);
-	delta = (line - vis1) - los * percent_of_view;
-	sci_scroll_lines(sci, delta);
-	sci_scroll_caret(sci); //ensure visible, in case of excessive folding/wrapping
-}
-
-
 void sci_marker_delete_all(ScintillaObject *sci, gint marker)
 {
 	SSM(sci, SCI_MARKERDELETEALL, marker, 0);
@@ -716,6 +741,7 @@ void sci_clear_cmdkey(ScintillaObject *sci, gint key)
 }
 
 
+/* text will be zero terminated and must be allocated (end - start + 1) bytes */
 void sci_get_text_range(ScintillaObject *sci, gint start, gint end, gchar *text)
 {
 	struct TextRange tr;
@@ -785,11 +811,6 @@ void sci_set_keywords(ScintillaObject *sci, gint k, gchar *text)
 	SSM(sci, SCI_SETKEYWORDS, k, (sptr_t) text);
 }
 
-void sci_scroll_lines(ScintillaObject *sci, gint lines)
-{
-	SSM(sci, SCI_LINESCROLL, 0, lines);
-}
-
 void sci_set_readonly(ScintillaObject *sci, gboolean readonly)
 {
 	SSM(sci, SCI_SETREADONLY, readonly, 0);
@@ -806,6 +827,7 @@ gboolean sci_get_readonly(ScintillaObject *sci)
 	SSM(sci, cmd, 0, 0);
 }
 
+
 gint sci_get_current_line(ScintillaObject *sci, gint pos)
 {
 	if (pos >= 0)
@@ -832,6 +854,12 @@ gint sci_get_lines_selected(ScintillaObject *sci)
 	return SSM(sci, SCI_LINEFROMPOSITION, end, 0) - SSM(sci, SCI_LINEFROMPOSITION, start, 0) + 1;
 }
 
+gint sci_get_first_visible_line(ScintillaObject *sci)
+{
+	return SSM(sci, SCI_GETFIRSTVISIBLELINE, 0, 0);
+}
+
+
 void sci_set_styling(ScintillaObject *sci, gint len, gint style)
 {
 	if (len < 0 || style < 0) return;
@@ -858,3 +886,19 @@ void sci_set_autoc_max_height(ScintillaObject *sci, gint val)
 {
 	SSM(sci, SCI_AUTOCSETMAXHEIGHT, val, 0);
 }
+
+gint sci_find_bracematch(ScintillaObject *sci, gint pos)
+{
+	return SSM(sci, SCI_BRACEMATCH, pos, 0);
+}
+
+gint sci_get_overtype(ScintillaObject *sci)
+{
+	return SSM(sci, SCI_GETOVERTYPE, 0, 0);
+}
+
+void sci_set_tab_indents(ScintillaObject *sci, gboolean set)
+{
+	SSM(sci, SCI_SETTABINDENTS, set, 0);
+}
+
