@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: search.c 825 2006-09-20 11:03:33Z ntrel $
+ * $Id: search.c 1252 2007-02-06 17:39:19Z ntrel $
  */
 
 #include <gdk/gdkkeysyms.h>
@@ -30,7 +30,8 @@
 #include "msgwindow.h"
 #include "document.h"
 #include "sciwrappers.h"
-#include "main.h"
+#include "ui_utils.h"
+#include "sci_cb.h"
 
 #include <unistd.h>
 #include <string.h>
@@ -42,10 +43,16 @@
 
 
 enum {
-	GEANY_RESPONSE_REPLACE = 1,
-	GEANY_RESPONSE_REPLACE_ALL,
-	GEANY_RESPONSE_REPLACE_SEL,
-	GEANY_RESPONSE_FIND
+	GEANY_RESPONSE_FIND = 1,
+	GEANY_RESPONSE_FIND_PREVIOUS,
+	GEANY_RESPONSE_FIND_IN_FILE,
+	GEANY_RESPONSE_FIND_IN_SESSION,
+	GEANY_RESPONSE_MARK,
+	GEANY_RESPONSE_REPLACE,
+	GEANY_RESPONSE_REPLACE_AND_FIND,
+	GEANY_RESPONSE_REPLACE_IN_SESSION,
+	GEANY_RESPONSE_REPLACE_IN_FILE,
+	GEANY_RESPONSE_REPLACE_IN_SEL
 };
 
 
@@ -85,8 +92,6 @@ static gchar **search_get_argv(const gchar **argv_prefix, const gchar *dir);
 static GSList *search_get_file_list(const gchar *path, guint *length);
 
 
-static void add_find_checkboxes(GtkDialog *dialog);
-
 static void
 on_find_replace_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 
@@ -103,7 +108,7 @@ static void
 on_replace_entry_activate(GtkEntry *entry, gpointer user_data);
 
 static gboolean
-on_combo_entry_key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
+on_widget_key_pressed_set_focus(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
 
 static void
 on_find_in_files_dialog_response(GtkDialog *dialog, gint response, gpointer user_data);
@@ -136,9 +141,9 @@ void search_finalize()
 }
 
 
-static void add_find_checkboxes(GtkDialog *dialog)
+static GtkWidget *add_find_checkboxes(GtkDialog *dialog)
 {
-	GtkWidget *checkbox1, *checkbox2, *check_regexp, *checkbox4, *checkbox5,
+	GtkWidget *checkbox1, *checkbox2, *check_regexp, *check_back, *checkbox5,
 			  *checkbox7, *hbox, *fbox, *mbox;
 	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
 
@@ -151,11 +156,22 @@ static void add_find_checkboxes(GtkDialog *dialog)
 	g_signal_connect(G_OBJECT(check_regexp), "toggled",
 		G_CALLBACK(on_find_replace_checkbutton_toggled), GTK_WIDGET(dialog));
 
-	checkbox4 = gtk_check_button_new_with_mnemonic(_("_Search backwards"));
-	g_object_set_data_full(G_OBJECT(dialog), "check_back",
-					gtk_widget_ref(checkbox4), (GDestroyNotify)gtk_widget_unref);
-	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox4), FALSE);
-
+	if (dialog != GTK_DIALOG(widgets.find_dialog))
+	{
+		check_back = gtk_check_button_new_with_mnemonic(_("_Search backwards"));
+		g_object_set_data_full(G_OBJECT(dialog), "check_back",
+						gtk_widget_ref(check_back), (GDestroyNotify)gtk_widget_unref);
+		gtk_button_set_focus_on_click(GTK_BUTTON(check_back), FALSE);
+	}
+	else
+	{	// align the two checkboxes at the top of the hbox
+		GtkSizeGroup *label_size;
+		check_back = gtk_label_new(NULL);
+		label_size = gtk_size_group_new(GTK_SIZE_GROUP_VERTICAL);
+		gtk_size_group_add_widget(GTK_SIZE_GROUP(label_size), check_back);
+		gtk_size_group_add_widget(GTK_SIZE_GROUP(label_size), check_regexp);
+		g_object_unref(label_size);
+	}
 	checkbox7 = gtk_check_button_new_with_mnemonic(_("Use _escape sequences"));
 	g_object_set_data_full(G_OBJECT(dialog), "check_escape",
 					gtk_widget_ref(checkbox7), (GDestroyNotify)gtk_widget_unref);
@@ -168,7 +184,7 @@ static void add_find_checkboxes(GtkDialog *dialog)
 	fbox = gtk_vbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(fbox), check_regexp);
 	gtk_container_add(GTK_CONTAINER(fbox), checkbox7);
-	gtk_container_add(GTK_CONTAINER(fbox), checkbox4);
+	gtk_container_add(GTK_CONTAINER(fbox), check_back);
 
 	checkbox1 = gtk_check_button_new_with_mnemonic(_("_Case sensitive"));
 	g_object_set_data_full(G_OBJECT(dialog), "check_case",
@@ -180,7 +196,7 @@ static void add_find_checkboxes(GtkDialog *dialog)
 					gtk_widget_ref(checkbox2), (GDestroyNotify)gtk_widget_unref);
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox2), FALSE);
 
-	checkbox5 = gtk_check_button_new_with_mnemonic(_("Match only word s_tart"));
+	checkbox5 = gtk_check_button_new_with_mnemonic(_("Match from s_tart of word"));
 	g_object_set_data_full(G_OBJECT(dialog), "check_wordstart",
 					gtk_widget_ref(checkbox5), (GDestroyNotify)gtk_widget_unref);
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox5), FALSE);
@@ -194,7 +210,33 @@ static void add_find_checkboxes(GtkDialog *dialog)
 	hbox = gtk_hbox_new(TRUE, 6);
 	gtk_container_add(GTK_CONTAINER(hbox), fbox);
 	gtk_container_add(GTK_CONTAINER(hbox), mbox);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, TRUE, TRUE, 6);
+	return hbox;
+}
+
+
+static void send_find_dialog_response(GtkButton *button, gpointer user_data)
+{
+	gtk_dialog_response(GTK_DIALOG(widgets.find_dialog), GPOINTER_TO_INT(user_data));
+}
+
+
+static gchar *get_default_text(gint idx)
+{
+	gchar *s = NULL;
+
+	if (sci_get_lines_selected(doc_list[idx].sci) == 1)
+	{
+		s = g_malloc(sci_get_selected_text_length(doc_list[idx].sci));
+		sci_get_selected_text(doc_list[idx].sci, s);
+	}
+	else if (sci_get_lines_selected(doc_list[idx].sci) == 0)
+	{	// use the word at current cursor position
+		gchar word[GEANY_MAX_WORD_LENGTH];
+
+		sci_cb_find_current_word(doc_list[idx].sci, -1, word, sizeof(word));
+		if (word[0] != '\0') s = g_strdup(word);
+	}
+	return s;
 }
 
 
@@ -205,20 +247,29 @@ void search_show_find_dialog()
 
 	if (idx == -1 || ! doc_list[idx].is_valid) return;
 
-	if (sci_get_lines_selected(doc_list[idx].sci) == 1)
-	{
-		sel = g_malloc(sci_get_selected_text_length(doc_list[idx].sci));
-		sci_get_selected_text(doc_list[idx].sci, sel);
-	}
+	sel = get_default_text(idx);
 
 	if (widgets.find_dialog == NULL)
 	{
-		GtkWidget *label, *entry, *sbox, *align;
+		GtkWidget *label, *entry, *sbox, *vbox;
+		GtkWidget *exp, *bbox, *button, *check_close;
+		GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
 
 		widgets.find_dialog = gtk_dialog_new_with_buttons(_("Find"),
 			GTK_WINDOW(app->window), GTK_DIALOG_DESTROY_WITH_PARENT,
-			GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
-			GTK_STOCK_FIND, GTK_RESPONSE_ACCEPT, NULL);
+			GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL, NULL);
+		vbox = ui_dialog_vbox_new(GTK_DIALOG(widgets.find_dialog));
+		gtk_box_set_spacing(GTK_BOX(vbox), 9);
+
+		button = ui_button_new_with_image(GTK_STOCK_GO_BACK, _("_Previous"));
+		gtk_dialog_add_action_widget(GTK_DIALOG(widgets.find_dialog), button,
+			GEANY_RESPONSE_FIND_PREVIOUS);
+		g_object_set_data_full(G_OBJECT(widgets.find_dialog), "btn_previous",
+						gtk_widget_ref(button), (GDestroyNotify)gtk_widget_unref);
+
+		button = ui_button_new_with_image(GTK_STOCK_GO_FORWARD, _("_Next"));
+		gtk_dialog_add_action_widget(GTK_DIALOG(widgets.find_dialog), button,
+			GEANY_RESPONSE_FIND);
 
 		label = gtk_label_new(_("Search for:"));
 		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
@@ -235,33 +286,77 @@ void search_show_find_dialog()
 		g_signal_connect((gpointer) widgets.find_dialog, "response",
 				G_CALLBACK(on_find_dialog_response), entry);
 		g_signal_connect((gpointer) widgets.find_dialog, "delete_event",
-				G_CALLBACK(gtk_widget_hide), NULL);
+				G_CALLBACK(gtk_widget_hide_on_delete), NULL);
 
 		sbox = gtk_hbox_new(FALSE, 6);
 		gtk_box_pack_start(GTK_BOX(sbox), label, FALSE, FALSE, 0);
 		gtk_box_pack_start(GTK_BOX(sbox), entry, TRUE, TRUE, 0);
-		align = gtk_alignment_new(0, 0, 1, 0);
-		gtk_alignment_set_padding(GTK_ALIGNMENT(align), 6, 0, 0, 0);
-		gtk_container_add(GTK_CONTAINER(align), sbox);
-		gtk_container_add(GTK_CONTAINER(GTK_DIALOG(widgets.find_dialog)->vbox), align);
+		gtk_box_pack_start(GTK_BOX(vbox), sbox, TRUE, FALSE, 0);
 
-		add_find_checkboxes(GTK_DIALOG(widgets.find_dialog));
+		gtk_container_add(GTK_CONTAINER(vbox),
+			add_find_checkboxes(GTK_DIALOG(widgets.find_dialog)));
 
-		gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(widgets.find_dialog)->vbox), 3);
+		// Now add the multiple match options
+		exp = gtk_expander_new(_("Find All"));
+		bbox = gtk_hbutton_box_new();
+
+#if 0
+		button = gtk_button_new_with_mnemonic(_("_Mark"));
+		gtk_container_add(GTK_CONTAINER(bbox), button);
+		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(send_find_dialog_response),
+			GINT_TO_POINTER(GEANY_RESPONSE_MARK));
+#endif	// not implemented yet
+
+		button = gtk_button_new_with_mnemonic(_("In Sessi_on"));
+		gtk_container_add(GTK_CONTAINER(bbox), button);
+		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(send_find_dialog_response),
+			GINT_TO_POINTER(GEANY_RESPONSE_FIND_IN_SESSION));
+
+		button = gtk_button_new_with_mnemonic(_("_In Document"));
+		gtk_container_add(GTK_CONTAINER(bbox), button);
+		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(send_find_dialog_response),
+			GINT_TO_POINTER(GEANY_RESPONSE_FIND_IN_FILE));
+
+		// close window checkbox
+		check_close = gtk_check_button_new_with_mnemonic(_("Close _dialog"));
+		g_object_set_data_full(G_OBJECT(widgets.find_dialog), "check_close",
+						gtk_widget_ref(check_close), (GDestroyNotify) gtk_widget_unref);
+		gtk_button_set_focus_on_click(GTK_BUTTON(check_close), FALSE);
+		gtk_tooltips_set_tip(tooltips, check_close,
+				_("Disable this option to keep the dialog open."), NULL);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_close), TRUE);
+		gtk_container_add(GTK_CONTAINER(bbox), check_close);
+		gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(bbox), check_close, TRUE);
+
+		ui_hbutton_box_copy_layout(
+			GTK_BUTTON_BOX(GTK_DIALOG(widgets.find_dialog)->action_area),
+			GTK_BUTTON_BOX(bbox));
+		gtk_container_add(GTK_CONTAINER(exp), bbox);
+		gtk_container_add(GTK_CONTAINER(vbox), exp);
 
 		gtk_widget_show_all(widgets.find_dialog);
 	}
 	else
 	{
-		if (sel) gtk_entry_set_text(GTK_ENTRY(GTK_BIN(
+		// only set selection if the dialog is not already visible
+		if (! GTK_WIDGET_VISIBLE(widgets.find_dialog) && sel)
+			gtk_entry_set_text(GTK_ENTRY(GTK_BIN(
 							lookup_widget(widgets.find_dialog, "entry"))->child), sel);
 		gtk_widget_grab_focus(GTK_WIDGET(GTK_BIN(lookup_widget(widgets.find_dialog, "entry"))->child));
 		gtk_widget_show(widgets.find_dialog);
+		// bring the dialog back in the foreground in case it is already open but the focus is away
+		gtk_window_present(GTK_WINDOW(widgets.find_dialog));
 	}
 	g_free(sel);
 }
 
 
+static void send_replace_dialog_response(GtkButton *button, gpointer user_data)
+{
+	gtk_dialog_response(GTK_DIALOG(widgets.replace_dialog), GPOINTER_TO_INT(user_data));
+}
+
+
 void search_show_replace_dialog()
 {
 	gint idx = document_get_cur_idx();
@@ -269,41 +364,30 @@ void search_show_replace_dialog()
 
 	if (idx == -1 || ! doc_list[idx].is_valid) return;
 
-	if (sci_get_lines_selected(doc_list[idx].sci) == 1)
-	{
-		sel = g_malloc(sci_get_selected_text_length(doc_list[idx].sci));
-		sci_get_selected_text(doc_list[idx].sci, sel);
-	}
+	sel = get_default_text(idx);
 
 	if (widgets.replace_dialog == NULL)
 	{
 		GtkWidget *label_find, *label_replace, *entry_find, *entry_replace,
-			*checkbox6, *checkbox8, *button, *align, *rbox, *fbox, *hbox;
-		GtkSizeGroup *size_group;
+			*check_close, *button, *rbox, *fbox, *vbox, *exp, *bbox;
+		GtkSizeGroup *label_size;
 		GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
 
 		widgets.replace_dialog = gtk_dialog_new_with_buttons(_("Replace"),
 			GTK_WINDOW(app->window), GTK_DIALOG_DESTROY_WITH_PARENT,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL, NULL);
+		vbox = ui_dialog_vbox_new(GTK_DIALOG(widgets.replace_dialog));
+		gtk_box_set_spacing(GTK_BOX(vbox), 9);
 
-		button = gtk_button_new_with_mnemonic(_("_In Selection"));
-		gtk_tooltips_set_tip(tooltips, button,
-			_("Replace all matches found in the currently selected text"), NULL);
-		gtk_widget_show(button);
-		gtk_dialog_add_action_widget(GTK_DIALOG(widgets.replace_dialog), button,
-			GEANY_RESPONSE_REPLACE_SEL);
-		button = gtk_button_new_with_mnemonic(_("Replace _All"));
-		gtk_widget_show(button);
-		gtk_dialog_add_action_widget(GTK_DIALOG(widgets.replace_dialog), button,
-			GEANY_RESPONSE_REPLACE_ALL);
 		button = gtk_button_new_from_stock(GTK_STOCK_FIND);
-		gtk_widget_show(button);
 		gtk_dialog_add_action_widget(GTK_DIALOG(widgets.replace_dialog), button,
 			GEANY_RESPONSE_FIND);
 		button = gtk_button_new_with_mnemonic(_("_Replace"));
-		gtk_widget_show(button);
 		gtk_dialog_add_action_widget(GTK_DIALOG(widgets.replace_dialog), button,
 			GEANY_RESPONSE_REPLACE);
+		button = gtk_button_new_with_mnemonic(_("Re_place & Find"));
+		gtk_dialog_add_action_widget(GTK_DIALOG(widgets.replace_dialog), button,
+			GEANY_RESPONSE_REPLACE_AND_FIND);
 
 		label_find = gtk_label_new(_("Search for:"));
 		gtk_misc_set_alignment(GTK_MISC(label_find), 0, 0.5);
@@ -324,66 +408,84 @@ void search_show_replace_dialog()
 		g_object_set_data_full(G_OBJECT(widgets.replace_dialog), "entry_replace",
 						gtk_widget_ref(entry_replace), (GDestroyNotify)gtk_widget_unref);
 
-		g_signal_connect((gpointer) gtk_bin_get_child(GTK_BIN(entry_find)), "key-press-event",
-				G_CALLBACK(on_combo_entry_key_pressed), gtk_bin_get_child(GTK_BIN(entry_replace)));
+		g_signal_connect((gpointer) gtk_bin_get_child(GTK_BIN(entry_find)),
+				"key-press-event", G_CALLBACK(on_widget_key_pressed_set_focus),
+				gtk_bin_get_child(GTK_BIN(entry_replace)));
 		g_signal_connect((gpointer) gtk_bin_get_child(GTK_BIN(entry_replace)), "activate",
 				G_CALLBACK(on_replace_entry_activate), NULL);
 		g_signal_connect((gpointer) widgets.replace_dialog, "response",
 				G_CALLBACK(on_replace_dialog_response), entry_replace);
 		g_signal_connect((gpointer) widgets.replace_dialog, "delete_event",
-				G_CALLBACK(gtk_widget_hide), NULL);
+				G_CALLBACK(gtk_widget_hide_on_delete), NULL);
 
 		fbox = gtk_hbox_new(FALSE, 6);
 		gtk_box_pack_start(GTK_BOX(fbox), label_find, FALSE, FALSE, 0);
 		gtk_box_pack_start(GTK_BOX(fbox), entry_find, TRUE, TRUE, 0);
-		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(widgets.replace_dialog)->vbox), fbox,
-			FALSE, FALSE, 6);
 
 		rbox = gtk_hbox_new(FALSE, 6);
 		gtk_box_pack_start(GTK_BOX(rbox), label_replace, FALSE, FALSE, 0);
 		gtk_box_pack_start(GTK_BOX(rbox), entry_replace, TRUE, TRUE, 0);
-		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(widgets.replace_dialog)->vbox), rbox,
-			FALSE, FALSE, 3);
 
-		size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
-		gtk_size_group_add_widget(size_group, label_find);
-		gtk_size_group_add_widget(size_group, label_replace);
-		g_object_unref(G_OBJECT(size_group));	// auto destroy the size group
+		label_size = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+		gtk_size_group_add_widget(label_size, label_find);
+		gtk_size_group_add_widget(label_size, label_replace);
+		g_object_unref(G_OBJECT(label_size));	// auto destroy the size group
 
-		add_find_checkboxes(GTK_DIALOG(widgets.replace_dialog));
+		gtk_box_pack_start(GTK_BOX(vbox), fbox, TRUE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(vbox), rbox, TRUE, FALSE, 0);
+		gtk_container_add(GTK_CONTAINER(vbox),
+			add_find_checkboxes(GTK_DIALOG(widgets.replace_dialog)));
 
-		checkbox6 = gtk_check_button_new_with_mnemonic(_("Replace in all _open files"));
-		g_object_set_data_full(G_OBJECT(widgets.replace_dialog), "check_all_buffers",
-						gtk_widget_ref(checkbox6), (GDestroyNotify)gtk_widget_unref);
-		gtk_tooltips_set_tip(tooltips, checkbox6,
-			_("Replaces the search text in all opened files. This option is only useful(and used) if you click on \"Replace All\"."), NULL);
-		gtk_button_set_focus_on_click(GTK_BUTTON(checkbox6), FALSE);
+		// Now add the multiple replace options
+		exp = gtk_expander_new(_("Replace All"));
+		bbox = gtk_hbutton_box_new();
 
-		// Don't close window checkbox
-		checkbox8 = gtk_check_button_new_with_mnemonic(_("_Don't close this dialog"));
-		g_object_set_data_full(G_OBJECT(widgets.replace_dialog), "check_dont_close_window",
-						gtk_widget_ref(checkbox8), (GDestroyNotify) gtk_widget_unref);
-		gtk_button_set_focus_on_click(GTK_BUTTON(checkbox8), FALSE);
-		gtk_tooltips_set_tip(tooltips, checkbox8,
-				_("The dialog window won't be closed when you start the operation."), NULL);
+		button = gtk_button_new_with_mnemonic(_("In Se_lection"));
+		gtk_tooltips_set_tip(tooltips, button,
+			_("Replace all matches found in the currently selected text"), NULL);
+		gtk_container_add(GTK_CONTAINER(bbox), button);
+		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(send_replace_dialog_response),
+			GINT_TO_POINTER(GEANY_RESPONSE_REPLACE_IN_SEL));
 
-		hbox = gtk_hbox_new(TRUE, 6);
-		gtk_container_add(GTK_CONTAINER(hbox), checkbox6);
-		gtk_container_add(GTK_CONTAINER(hbox), checkbox8);
+		button = gtk_button_new_with_mnemonic(_("In Sessi_on"));
+		gtk_container_add(GTK_CONTAINER(bbox), button);
+		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(send_replace_dialog_response),
+			GINT_TO_POINTER(GEANY_RESPONSE_REPLACE_IN_SESSION));
 
-		align = gtk_alignment_new(0, 0, 1, 1);
-		gtk_alignment_set_padding(GTK_ALIGNMENT(align), 0, 9, 0, 0);
-		gtk_container_add(GTK_CONTAINER(align), hbox);
-		gtk_container_add(GTK_CONTAINER(GTK_DIALOG(widgets.replace_dialog)->vbox), align);
+		button = gtk_button_new_with_mnemonic(_("_In Document"));
+		gtk_container_add(GTK_CONTAINER(bbox), button);
+		g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(send_replace_dialog_response),
+			GINT_TO_POINTER(GEANY_RESPONSE_REPLACE_IN_FILE));
+
+		// close window checkbox
+		check_close = gtk_check_button_new_with_mnemonic(_("Close _dialog"));
+		g_object_set_data_full(G_OBJECT(widgets.replace_dialog), "check_close",
+						gtk_widget_ref(check_close), (GDestroyNotify) gtk_widget_unref);
+		gtk_button_set_focus_on_click(GTK_BUTTON(check_close), FALSE);
+		gtk_tooltips_set_tip(tooltips, check_close,
+				_("Disable this option to keep the dialog open."), NULL);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_close), TRUE);
+		gtk_container_add(GTK_CONTAINER(bbox), check_close);
+		gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(bbox), check_close, TRUE);
+
+		ui_hbutton_box_copy_layout(
+			GTK_BUTTON_BOX(GTK_DIALOG(widgets.replace_dialog)->action_area),
+			GTK_BUTTON_BOX(bbox));
+		gtk_container_add(GTK_CONTAINER(exp), bbox);
+		gtk_container_add(GTK_CONTAINER(vbox), exp);
 
 		gtk_widget_show_all(widgets.replace_dialog);
 	}
 	else
 	{
-		if (sel) gtk_entry_set_text(GTK_ENTRY(GTK_BIN(
+		// only set selection if the dialog is not already visible
+		if (! GTK_WIDGET_VISIBLE(widgets.replace_dialog) && sel)
+			gtk_entry_set_text(GTK_ENTRY(GTK_BIN(
 							lookup_widget(widgets.replace_dialog, "entry_find"))->child), sel);
 		gtk_widget_grab_focus(GTK_WIDGET(GTK_BIN(lookup_widget(widgets.replace_dialog, "entry_find"))->child));
 		gtk_widget_show(widgets.replace_dialog);
+		// bring the dialog back in the foreground in case it is already open but the focus is away
+		gtk_window_present(GTK_WINDOW(widgets.replace_dialog));
 	}
 	g_free(sel);
 }
@@ -392,8 +494,8 @@ void search_show_replace_dialog()
 void search_show_find_in_files_dialog()
 {
 	static GtkWidget *combo = NULL;
-	static GtkWidget *entry1;
-	GtkWidget *entry2; // the child GtkEntry of combo
+	static GtkWidget *dir_combo;
+	GtkWidget *entry; // the child GtkEntry of combo (or dir_combo)
 	GtkWidget *dirbtn, *openimg;
 	gint idx = document_get_cur_idx();
 	gchar *sel = NULL;
@@ -401,18 +503,18 @@ void search_show_find_in_files_dialog()
 
 	if (idx == -1 || ! doc_list[idx].is_valid) return;
 
-	cur_dir = utils_get_current_file_dir();
-
 	if (widgets.find_in_files_dialog == NULL)
 	{
 		GtkWidget *label, *label1, *checkbox1, *checkbox2, *check_wholeword;
-		GtkWidget *dbox, *sbox, *cbox, *rbox, *rbtn, *hbox;
+		GtkWidget *dbox, *sbox, *cbox, *rbox, *rbtn, *hbox, *vbox;
 		GtkSizeGroup *size_group;
 		GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(app->window, "tooltips"));
 
 		widgets.find_in_files_dialog = gtk_dialog_new_with_buttons(
 			_("Find in files"), GTK_WINDOW(app->window), GTK_DIALOG_DESTROY_WITH_PARENT,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
+		vbox = ui_dialog_vbox_new(GTK_DIALOG(widgets.find_in_files_dialog));
+		gtk_box_set_spacing(GTK_BOX(vbox), 9);
 
 		gtk_dialog_add_button(GTK_DIALOG(widgets.find_in_files_dialog), "gtk-find", GTK_RESPONSE_ACCEPT);
 		gtk_dialog_set_default_response(GTK_DIALOG(widgets.find_in_files_dialog),
@@ -421,11 +523,12 @@ void search_show_find_in_files_dialog()
 		label1 = gtk_label_new(_("Directory:"));
 		gtk_misc_set_alignment(GTK_MISC(label1), 0, 0.5);
 
-		entry1 = gtk_entry_new();
-		gtk_entry_set_max_length(GTK_ENTRY(entry1), 248);
-		gtk_entry_set_width_chars(GTK_ENTRY(entry1), 50);
-		g_object_set_data_full(G_OBJECT(widgets.find_in_files_dialog), "entry_dir",
-						gtk_widget_ref(entry1), (GDestroyNotify)gtk_widget_unref);
+		dir_combo = gtk_combo_box_entry_new_text();
+		entry = gtk_bin_get_child(GTK_BIN(dir_combo));
+		gtk_entry_set_max_length(GTK_ENTRY(entry), 248);
+		gtk_entry_set_width_chars(GTK_ENTRY(entry), 50);
+		g_object_set_data_full(G_OBJECT(widgets.find_in_files_dialog), "dir_combo",
+						gtk_widget_ref(dir_combo), (GDestroyNotify)gtk_widget_unref);
 
 		dirbtn = gtk_button_new();
 		openimg = gtk_image_new_from_stock(GTK_STOCK_OPEN, GTK_ICON_SIZE_BUTTON);
@@ -435,17 +538,17 @@ void search_show_find_in_files_dialog()
 
 		dbox = gtk_hbox_new(FALSE, 6);
 		gtk_box_pack_start(GTK_BOX(dbox), label1, FALSE, FALSE, 0);
-		gtk_box_pack_start(GTK_BOX(dbox), entry1, TRUE, TRUE, 0);
+		gtk_box_pack_start(GTK_BOX(dbox), dir_combo, TRUE, TRUE, 0);
 		gtk_box_pack_start(GTK_BOX(dbox), dirbtn, FALSE, FALSE, 0);
 
 		label = gtk_label_new(_("Search for:"));
 		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
 		combo = gtk_combo_box_entry_new_text();
-		entry2 = gtk_bin_get_child(GTK_BIN(combo));
-		gtk_entry_set_max_length(GTK_ENTRY(entry2), 248);
-		gtk_entry_set_width_chars(GTK_ENTRY(entry2), 50);
-		gtk_entry_set_activates_default(GTK_ENTRY(entry2), TRUE);
+		entry = gtk_bin_get_child(GTK_BIN(combo));
+		gtk_entry_set_max_length(GTK_ENTRY(entry), 248);
+		gtk_entry_set_width_chars(GTK_ENTRY(entry), 50);
+		gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
 
 		sbox = gtk_hbox_new(FALSE, 6);
 		gtk_box_pack_start(GTK_BOX(sbox), label, FALSE, FALSE, 0);
@@ -508,45 +611,46 @@ void search_show_find_in_files_dialog()
 		gtk_container_add(GTK_CONTAINER(hbox), rbox);
 		gtk_container_add(GTK_CONTAINER(hbox), cbox);
 
-		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(widgets.find_in_files_dialog)->vbox),
-			dbox, TRUE, FALSE, 6);
-		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(widgets.find_in_files_dialog)->vbox),
-			sbox, TRUE, FALSE, 0);
-		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(widgets.find_in_files_dialog)->vbox),
-			hbox, TRUE, TRUE, 6);
+		gtk_box_pack_start(GTK_BOX(vbox), dbox, TRUE, FALSE, 0);
+		gtk_box_pack_start(GTK_BOX(vbox), sbox, TRUE, FALSE, 0);
+		gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
-		gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(widgets.find_in_files_dialog)->vbox), 6);
-
-		g_signal_connect((gpointer) entry1, "key-press-event",
-				G_CALLBACK(on_combo_entry_key_pressed), gtk_bin_get_child(GTK_BIN(combo)));
+		g_signal_connect((gpointer) dir_combo, "key-press-event",
+				G_CALLBACK(on_widget_key_pressed_set_focus), combo);
 		g_signal_connect((gpointer) widgets.find_in_files_dialog, "response",
 				G_CALLBACK(on_find_in_files_dialog_response), combo);
 		g_signal_connect((gpointer) widgets.find_in_files_dialog, "delete_event",
-				G_CALLBACK(gtk_widget_hide), NULL);
+				G_CALLBACK(gtk_widget_hide_on_delete), NULL);
 
 		gtk_widget_show_all(widgets.find_in_files_dialog);
+		sel = get_default_text(idx);
 	}
 
-	if (sci_get_lines_selected(doc_list[idx].sci) == 1)
-	{
-		sel = g_malloc(sci_get_selected_text_length(doc_list[idx].sci));
-		sci_get_selected_text(doc_list[idx].sci, sel);
-	}
-
-	entry2 = GTK_BIN(combo)->child;
-	if (sel) gtk_entry_set_text(GTK_ENTRY(entry2), sel);
+	entry = GTK_BIN(combo)->child;
+	// only set selection if the dialog is not already visible, or has just been created
+	if (! sel && ! GTK_WIDGET_VISIBLE(widgets.find_in_files_dialog))
+		sel = get_default_text(idx);
+	if (sel)
+		gtk_entry_set_text(GTK_ENTRY(entry), sel);
 	g_free(sel);
 
-	if (cur_dir) gtk_entry_set_text(GTK_ENTRY(entry1), cur_dir);
-	g_free(cur_dir);
+	entry = GTK_BIN(dir_combo)->child;
+	cur_dir = utils_get_current_file_dir();
+	if (cur_dir)
+	{
+		gtk_entry_set_text(GTK_ENTRY(entry), cur_dir);
+		g_free(cur_dir);
+	}
 
 	// put the focus to the directory entry if it is empty
-	if (utils_strcmp(gtk_entry_get_text(GTK_ENTRY(entry1)), ""))
-		gtk_widget_grab_focus(entry1);
+	if (utils_str_equal(gtk_entry_get_text(GTK_ENTRY(entry)), ""))
+		gtk_widget_grab_focus(dir_combo);
 	else
-		gtk_widget_grab_focus(entry2);
+		gtk_widget_grab_focus(combo);
 
 	gtk_widget_show(widgets.find_in_files_dialog);
+	// bring the dialog back in the foreground in case it is already open but the focus is away
+	gtk_window_present(GTK_WINDOW(widgets.find_in_files_dialog));
 }
 
 
@@ -556,7 +660,9 @@ static void on_open_dir_dialog_clicked(GtkButton *button, gpointer user_data)
 		GTK_WINDOW(app->window), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 		GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
-	GtkWidget *entry_dir = lookup_widget(widgets.find_in_files_dialog, "entry_dir");
+	GtkWidget *dir_combo =
+		lookup_widget(widgets.find_in_files_dialog, "dir_combo");
+	GtkWidget *entry_dir = gtk_bin_get_child(GTK_BIN(dir_combo));
 	gchar *dir_locale;
 	const gchar *entry_text;
 
@@ -589,7 +695,6 @@ on_find_replace_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user
 	if (togglebutton == chk_regexp)
 	{
 		gboolean regex_set = gtk_toggle_button_get_active(chk_regexp);
-		GtkWidget *check_back = lookup_widget(dialog, "check_back");
 		GtkWidget *check_word = lookup_widget(dialog, "check_word");
 		GtkWidget *check_wordstart = lookup_widget(dialog, "check_wordstart");
 		GtkToggleButton *check_case = GTK_TOGGLE_BUTTON(
@@ -597,7 +702,11 @@ on_find_replace_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user
 		static gboolean case_state = FALSE; // state before regex enabled
 
 		// hide options that don't apply to regex searches
-		gtk_widget_set_sensitive(check_back, ! regex_set);
+		if (dialog == widgets.find_dialog)
+			gtk_widget_set_sensitive(lookup_widget(dialog, "btn_previous"), ! regex_set);
+		else
+			gtk_widget_set_sensitive(lookup_widget(dialog, "check_back"), ! regex_set);
+
 		gtk_widget_set_sensitive(check_word, ! regex_set);
 		gtk_widget_set_sensitive(check_wordstart, ! regex_set);
 
@@ -617,10 +726,20 @@ on_find_replace_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user
 }
 
 
+static void
+on_find_entry_activate(GtkEntry *entry, gpointer user_data)
+{
+	on_find_dialog_response(NULL, GEANY_RESPONSE_FIND,
+				lookup_widget(GTK_WIDGET(entry), "entry"));
+}
+
+
 static void
 on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 {
-	if (response == GTK_RESPONSE_ACCEPT)
+	if (response == GTK_RESPONSE_CANCEL)
+		gtk_widget_hide(widgets.find_dialog);
+	else
 	{
 		gint idx = document_get_cur_idx();
 		gboolean search_replace_escape;
@@ -632,11 +751,14 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 			fl3 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
 						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_regexp"))),
 			fl4 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_wordstart")));
+						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_wordstart"))),
+			check_close = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_close")));
 		search_replace_escape = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
 						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_escape")));
-		search_data.backwards = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-						lookup_widget(GTK_WIDGET(widgets.find_dialog), "check_back")));
+		search_data.backwards = FALSE;
+
+		if (! DOC_IDX_VALID(idx)) return;
 
 		g_free(search_data.text);
 		search_data.text = g_strdup(gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(user_data)))));
@@ -648,22 +770,47 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 			return;
 		}
 
-		gtk_combo_box_prepend_text(GTK_COMBO_BOX(user_data), search_data.text);
+		ui_combo_box_add_to_history(GTK_COMBO_BOX(user_data), search_data.text);
 		search_data.flags = (fl1 ? SCFIND_MATCHCASE : 0) |
 				(fl2 ? SCFIND_WHOLEWORD : 0) |
 				(fl3 ? SCFIND_REGEXP | SCFIND_POSIX: 0) |
 				(fl4 ? SCFIND_WORDSTART : 0);
-		document_find_text(idx, search_data.text, search_data.flags, search_data.backwards);
+
+		switch (response)
+		{
+			case GEANY_RESPONSE_FIND:
+			case GEANY_RESPONSE_FIND_PREVIOUS:
+			document_find_text(idx, search_data.text, search_data.flags,
+				(response == GEANY_RESPONSE_FIND_PREVIOUS), TRUE);
+			check_close = FALSE;
+			if (app->pref_main_suppress_search_dialogs)
+				check_close = TRUE;
+			break;
+
+			case GEANY_RESPONSE_FIND_IN_FILE:
+			search_find_usage(search_data.text, search_data.flags, FALSE);
+			break;
+
+			case GEANY_RESPONSE_FIND_IN_SESSION:
+			search_find_usage(search_data.text, search_data.flags, TRUE);
+			break;
+
+			case GEANY_RESPONSE_MARK:
+			// TODO
+			break;
+
+			// Note: we can get GTK_RESPONSE_DELETE_EVENT responses when ESC is pressed
+		}
+		if (check_close)
+			gtk_widget_hide(widgets.find_dialog);
 	}
-	else gtk_widget_hide(widgets.find_dialog);
 }
 
 
 static void
-on_find_entry_activate(GtkEntry *entry, gpointer user_data)
+on_replace_entry_activate(GtkEntry *entry, gpointer user_data)
 {
-	on_find_dialog_response(NULL, GTK_RESPONSE_ACCEPT,
-				lookup_widget(GTK_WIDGET(entry), "entry"));
+	on_replace_dialog_response(NULL, GEANY_RESPONSE_REPLACE, NULL);
 }
 
 
@@ -674,7 +821,7 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 	GtkWidget *entry_find = lookup_widget(GTK_WIDGET(widgets.replace_dialog), "entry_find");
 	GtkWidget *entry_replace = lookup_widget(GTK_WIDGET(widgets.replace_dialog), "entry_replace");
 	gint search_flags_re;
-	gboolean search_backwards_re, search_replace_escape_re, search_in_all_buffers_re;
+	gboolean search_backwards_re, search_replace_escape_re;
 	gboolean fl1, fl2, fl3, fl4;
 	gboolean close_window;
 	gchar *find, *replace;
@@ -693,14 +840,12 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 				lookup_widget(GTK_WIDGET(widgets.replace_dialog), "check_regexp")));
 	fl4 = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
 				lookup_widget(GTK_WIDGET(widgets.replace_dialog), "check_wordstart")));
-	close_window = !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-				lookup_widget(GTK_WIDGET(widgets.replace_dialog), "check_dont_close_window")));
+	close_window = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
+				lookup_widget(GTK_WIDGET(widgets.replace_dialog), "check_close")));
 	search_backwards_re = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
 				lookup_widget(GTK_WIDGET(widgets.replace_dialog), "check_back")));
 	search_replace_escape_re = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
 				lookup_widget(GTK_WIDGET(widgets.replace_dialog), "check_escape")));
-	search_in_all_buffers_re = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-				lookup_widget(GTK_WIDGET(widgets.replace_dialog), "check_all_buffers")));
 	find = g_strdup(gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_find)))));
 	replace = g_strdup(gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_replace)))));
 
@@ -711,8 +856,8 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		return;
 	}
 
-	gtk_combo_box_prepend_text(GTK_COMBO_BOX(entry_find), find);
-	gtk_combo_box_prepend_text(GTK_COMBO_BOX(entry_replace), replace);
+	ui_combo_box_add_to_history(GTK_COMBO_BOX(entry_find), find);
+	ui_combo_box_add_to_history(GTK_COMBO_BOX(entry_replace), replace);
 
 	if (search_replace_escape_re &&
 		(! utils_str_replace_escape(find) || ! utils_str_replace_escape(replace)))
@@ -727,60 +872,69 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 					  (fl3 ? SCFIND_REGEXP | SCFIND_POSIX : 0) |
 					  (fl4 ? SCFIND_WORDSTART : 0);
 
-	if (search_in_all_buffers_re && response == GEANY_RESPONSE_REPLACE_ALL)
+	switch (response)
 	{
-		gint i;
-		for (i = 0; i < GEANY_MAX_OPEN_FILES; i++)
+		case GEANY_RESPONSE_REPLACE_AND_FIND:
 		{
-			if (! doc_list[i].is_valid) continue;
+			gint rep = document_replace_text(idx, find, replace, search_flags_re,
+				search_backwards_re);
+			if (rep != -1)
+				document_find_text(idx, find, search_flags_re, search_backwards_re,
+					TRUE);
+			break;
+		}
+		case GEANY_RESPONSE_REPLACE:
+		{
+			document_replace_text(idx, find, replace, search_flags_re,
+				search_backwards_re);
+			break;
+		}
+		case GEANY_RESPONSE_FIND:
+		{
+			document_find_text(idx, find, search_flags_re, search_backwards_re, TRUE);
+			break;
+		}
+		case GEANY_RESPONSE_REPLACE_IN_FILE:
+		{
+			document_replace_all(idx, find, replace, search_flags_re, search_replace_escape_re);
+			if (close_window) gtk_widget_hide(widgets.replace_dialog);
+			break;
+		}
+		case GEANY_RESPONSE_REPLACE_IN_SESSION:
+		{
+			guint n, count = 0;
 
-			document_replace_all(i, find, replace, search_flags_re, search_replace_escape_re);
+			// replace in all documents following notebook tab order
+			for (n = 0; (gint) n < gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)); n++)
+			{
+				gint idx = document_get_n_idx(n);
+
+				if (! doc_list[idx].is_valid) continue;
+
+				if (document_replace_all(idx, find, replace, search_flags_re,
+					search_replace_escape_re)) count++;
+			}
+			ui_set_statusbar(_("Replaced text in %u files."), count);
+
+			ui_save_buttons_toggle(doc_list[idx].changed);	// update save all
+			if (close_window) gtk_widget_hide(widgets.replace_dialog);
+			break;
 		}
-		if (close_window) gtk_widget_hide(widgets.replace_dialog);
-	}
-	else
-	{
-		switch (response)
+		case GEANY_RESPONSE_REPLACE_IN_SEL:
 		{
-			case GEANY_RESPONSE_REPLACE:
-			{
-				document_replace_text(idx, find, replace, search_flags_re,
-					search_backwards_re);
-				break;
-			}
-			case GEANY_RESPONSE_FIND:
-			{
-				document_find_text(idx, find, search_flags_re, search_backwards_re);
-				break;
-			}
-			case GEANY_RESPONSE_REPLACE_ALL:
-			{
-				document_replace_all(idx, find, replace, search_flags_re, search_replace_escape_re);
-				if (close_window) gtk_widget_hide(widgets.replace_dialog);
-				break;
-			}
-			case GEANY_RESPONSE_REPLACE_SEL:
-			{
-				document_replace_sel(idx, find, replace, search_flags_re, search_replace_escape_re);
-				if (close_window) gtk_widget_hide(widgets.replace_dialog);
-				break;
-			}
+			document_replace_sel(idx, find, replace, search_flags_re, search_replace_escape_re);
+			if (close_window) gtk_widget_hide(widgets.replace_dialog);
+			break;
 		}
+		// Note: we can get GTK_RESPONSE_DELETE_EVENT responses when ESC is pressed
 	}
 	g_free(find);
 	g_free(replace);
 }
 
 
-static void
-on_replace_entry_activate(GtkEntry *entry, gpointer user_data)
-{
-	on_replace_dialog_response(NULL, GEANY_RESPONSE_REPLACE, NULL);
-}
-
-
 static gboolean
-on_combo_entry_key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
+on_widget_key_pressed_set_focus(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
 {
 	// catch tabulator key to set the focus in the replace entry instead of
 	// setting it to the combo box
@@ -800,11 +954,13 @@ on_find_in_files_dialog_response(GtkDialog *dialog, gint response, gpointer user
 	{
 		const gchar *search_text =
 			gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(user_data))));
+		GtkWidget *dir_combo =
+			lookup_widget(widgets.find_in_files_dialog, "dir_combo");
 		const gchar *utf8_dir =
-			gtk_entry_get_text(GTK_ENTRY(lookup_widget(widgets.find_in_files_dialog, "entry_dir")));
+			gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(dir_combo))));
 
-		if (utf8_dir == NULL || utils_strcmp(utf8_dir, ""))
-			msgwin_status_add(_("Invalid directory for find in files."));
+		if (utf8_dir == NULL || utils_str_equal(utf8_dir, ""))
+			ui_set_statusbar(_("Invalid directory for find in files."));
 		else if (search_text && *search_text)
 		{
 			gchar *locale_dir;
@@ -827,13 +983,14 @@ on_find_in_files_dialog_response(GtkDialog *dialog, gint response, gpointer user
 
 			if (search_find_in_files(search_text, locale_dir, regex_opt, opts))
 			{
-				gtk_combo_box_prepend_text(GTK_COMBO_BOX(user_data), search_text);
+				ui_combo_box_add_to_history(GTK_COMBO_BOX(user_data), search_text);
+				ui_combo_box_add_to_history(GTK_COMBO_BOX(dir_combo), utf8_dir);
 				gtk_widget_hide(widgets.find_in_files_dialog);
 			}
 			g_free(locale_dir);
 		}
 		else
-			msgwin_status_add(_("No text to find."));
+			ui_set_statusbar(_("No text to find."));
 	}
 	else
 		gtk_widget_hide(widgets.find_in_files_dialog);
@@ -1040,3 +1197,72 @@ static void search_close_pid(GPid child_pid, gint status, gpointer user_data)
 }
 
 
+static gint find_document_usage(gint idx, const gchar *search_text, gint flags)
+{
+	gchar *buffer, *short_file_name, *string;
+	gint pos, line, count = 0;
+	struct TextToFind ttf;
+
+	g_return_val_if_fail(DOC_IDX_VALID(idx), 0);
+
+	ttf.chrg.cpMin = 0;
+	ttf.chrg.cpMax = sci_get_length(doc_list[idx].sci);
+	ttf.lpstrText = (gchar *)search_text;
+	while (1)
+	{
+		pos = sci_find_text(doc_list[idx].sci, flags, &ttf);
+		if (pos == -1) break;
+
+		line = sci_get_line_from_position(doc_list[idx].sci, pos);
+		buffer = sci_get_line(doc_list[idx].sci, line);
+
+		if (doc_list[idx].file_name == NULL)
+			short_file_name = g_strdup(GEANY_STRING_UNTITLED);
+		else
+			short_file_name = g_path_get_basename(doc_list[idx].file_name);
+		string = g_strdup_printf("%s:%d : %s", short_file_name, line + 1, g_strstrip(buffer));
+		msgwin_msg_add(line + 1, idx, string);
+
+		g_free(buffer);
+		g_free(short_file_name);
+		g_free(string);
+		ttf.chrg.cpMin = ttf.chrgText.cpMax + 1;
+		count++;
+	}
+	return count;
+}
+
+
+void search_find_usage(const gchar *search_text, gint flags, gboolean in_session)
+{
+	gint idx;
+	gboolean found = FALSE;
+
+	idx = document_get_cur_idx();
+	g_return_if_fail(DOC_IDX_VALID(idx));
+
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_MESSAGE);
+	gtk_list_store_clear(msgwindow.store_msg);
+
+	if (! in_session)
+	{	// use current document
+		found = (find_document_usage(idx, search_text, flags) > 0);
+	}
+	else
+	{
+		guint i;
+		for (i = 0; i < doc_array->len; i++)
+		{
+			if (doc_list[i].is_valid)
+				if (find_document_usage(i, search_text, flags) > 0) found = TRUE;
+		}
+	}
+
+	if (! found) // no matches were found
+	{
+		gchar *text = g_strdup_printf(_("No matches found for '%s'."), search_text);
+		ui_set_statusbar("%s", text);
+		msgwin_msg_add(-1, -1, text);
+		g_free(text);
+	}
+}
