@@ -1,7 +1,8 @@
 /*
  *      utils.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: utils.c 836 2006-09-26 16:54:20Z eht16 $
+ * $Id: utils.c 1270 2007-02-12 18:42:15Z eht16 $
  */
 
 
@@ -43,8 +44,6 @@
 #include "sciwrappers.h"
 #include "dialogs.h"
 #include "win32.h"
-#include "main.h"
-#include "encodings.h"
 
 #include "utils.h"
 
@@ -191,71 +190,6 @@ gboolean utils_is_opening_brace(gchar c)
 }
 
 
-const GList *utils_get_tag_list(gint idx, guint tag_types)
-{
-	static GList *tag_names = NULL;
-
-	if (idx >= 0 && doc_list[idx].is_valid && doc_list[idx].tm_file &&
-		doc_list[idx].tm_file->tags_array)
-	{
-		TMTag *tag;
-		guint i;
-		GeanySymbol *symbol;
-		gboolean doc_is_utf8 = FALSE;
-		gchar *utf8_name;
-
-		if (tag_names)
-		{
-			GList *tmp;
-			for (tmp = tag_names; tmp; tmp = g_list_next(tmp))
-			{
-				g_free(((GeanySymbol*)tmp->data)->str);
-				g_free(tmp->data);
-			}
-			g_list_free(tag_names);
-			tag_names = NULL;
-		}
-
-		// do this comparison only once
-		if (utils_strcmp(doc_list[idx].encoding, "UTF-8")) doc_is_utf8 = TRUE;
-
-		for (i = 0; i < (doc_list[idx].tm_file)->tags_array->len; ++i)
-		{
-			tag = TM_TAG((doc_list[idx].tm_file)->tags_array->pdata[i]);
-			if (tag == NULL)
-				return NULL;
-
-			if (tag->type & tag_types)
-			{
-				if (! doc_is_utf8) utf8_name = utils_convert_to_utf8_from_charset(tag->name, -1,
-															doc_list[idx].encoding, TRUE);
-				else utf8_name = tag->name;
-				if ((tag->atts.entry.scope != NULL) && isalpha(tag->atts.entry.scope[0]))
-				{
-					symbol = g_new0(GeanySymbol, 1);
-					symbol->str = g_strdup_printf("%s::%s [%ld]", tag->atts.entry.scope, utf8_name, tag->atts.entry.line);
-					symbol->type = tag->type;
-					symbol->line = tag->atts.entry.line;
-					tag_names = g_list_prepend(tag_names, symbol);
-				}
-				else
-				{
-					symbol = g_new0(GeanySymbol, 1);
-					symbol->str = g_strdup_printf("%s [%ld]", utf8_name, tag->atts.entry.line);
-					symbol->type = tag->type;
-					symbol->line = tag->atts.entry.line;
-					tag_names = g_list_prepend(tag_names, symbol);
-				}
-				if (! doc_is_utf8) g_free(utf8_name);
-			}
-		}
-		tag_names = g_list_sort(tag_names, (GCompareFunc) utils_compare_symbol);
-		return tag_names;
-	}
-	else
-		return NULL;
-}
-
 /* returns the line of the given tag */
 gint utils_get_local_tag(gint idx, const gchar *qual_name)
 {
@@ -300,7 +234,8 @@ gboolean utils_goto_line(gint idx, gint line)
 	sci_marker_delete_all(doc_list[idx].sci, 0);
 	sci_set_marker_at_line(doc_list[idx].sci, line, TRUE, 0);
 
-	sci_goto_line_scroll(doc_list[idx].sci, line, 0.25);
+	sci_goto_line(doc_list[idx].sci, line, TRUE);
+	doc_list[idx].scroll_percent = 0.25F;
 
 	// finally switch to the page
 	page_num = gtk_notebook_page_num(GTK_NOTEBOOK(app->notebook), GTK_WIDGET(doc_list[idx].sci));
@@ -344,95 +279,6 @@ gint utils_write_file(const gchar *filename, const gchar *text)
 }
 
 
-/* Converts a string from the given charset to UTF-8.
- * If fast is set, no further checks are performed. */
-gchar *utils_convert_to_utf8_from_charset(const gchar *buffer, gsize size, const gchar *charset,
-										  gboolean fast)
-{
-	gchar *utf8_content = NULL;
-	GError *conv_error = NULL;
-	gchar* converted_contents = NULL;
-	gsize bytes_written;
-
-	g_return_val_if_fail(buffer != NULL, NULL);
-	g_return_val_if_fail(charset != NULL, NULL);
-
-	converted_contents = g_convert(buffer, size, "UTF-8", charset, NULL,
-								   &bytes_written, &conv_error);
-
-	if (fast)
-	{
-		utf8_content = converted_contents;
-		if (conv_error != NULL) g_error_free(conv_error);
-	}
-	else if (conv_error != NULL || ! g_utf8_validate(converted_contents, bytes_written, NULL))
-	{
-		if (conv_error != NULL)
-		{
-			geany_debug("Couldn't convert from %s to UTF-8 (%s).", charset, conv_error->message);
-			g_error_free(conv_error);
-			conv_error = NULL;
-		}
-		else
-			geany_debug("Couldn't convert from %s to UTF-8.", charset);
-
-		utf8_content = NULL;
-		if (converted_contents != NULL) g_free(converted_contents);
-	}
-	else
-	{
-		geany_debug("Converted from %s to UTF-8.", charset);
-		utf8_content = converted_contents;
-	}
-
-	return utf8_content;
-}
-
-
-gchar *utils_convert_to_utf8(const gchar *buffer, gsize size, gchar **used_encoding)
-{
-	gchar *locale_charset = NULL;
-	gchar *utf8_content;
-	gchar *charset;
-	gboolean check_current = FALSE;
-	guint i;
-
-	// current locale is not UTF-8, we have to check this charset
-	check_current = ! g_get_charset((const gchar**)&locale_charset);
-
-	for (i = 0; i < GEANY_ENCODINGS_MAX; i++)
-	{
-		if (check_current)
-		{
-			check_current = FALSE;
-			charset = locale_charset;
-			i = -1;
-		}
-		else
-			charset = encodings[i].charset;
-
-		geany_debug("Trying to convert %d bytes of data from %s into UTF-8.", size, charset);
-		utf8_content = utils_convert_to_utf8_from_charset(buffer, size, charset, FALSE);
-
-		if (utf8_content != NULL)
-		{
-			if (used_encoding != NULL)
-			{
-				if (*used_encoding != NULL)
-				{
-					g_free(*used_encoding);
-					geany_debug("%s:%d", __FILE__, __LINE__);
-				}
-				*used_encoding = g_strdup(charset);
-			}
-			return utf8_content;
-		}
-	}
-
-	return NULL;
-}
-
-
 /**
  * (stolen from anjuta and modified)
  * Search backward through size bytes looking for a '<', then return the tag if any
@@ -479,28 +325,31 @@ gchar *utils_find_open_xml_tag(const gchar sel[], gint size, gboolean check_tag)
 }
 
 
-gboolean utils_check_disk_status(gint idx)
+/* Set force to force a disk check, otherwise it is ignored if there was a check
+ * in the last GEANY_CHECK_FILE_DELAY seconds. */
+gboolean utils_check_disk_status(gint idx, gboolean force)
 {
 	struct stat st;
 	time_t t;
 	gchar *locale_filename;
+	gboolean ret = FALSE;
 
 	if (idx == -1 || doc_list[idx].file_name == NULL) return FALSE;
 
 	t = time(NULL);
 
-	if (doc_list[idx].last_check > (t - GEANY_CHECK_FILE_DELAY)) return FALSE;
+	if (! force && doc_list[idx].last_check > (t - GEANY_CHECK_FILE_DELAY)) return FALSE;
 
 	locale_filename = utils_get_locale_from_utf8(doc_list[idx].file_name);
-	if (stat(locale_filename, &st) != 0) return FALSE;
-
-	if (doc_list[idx].mtime > t || st.st_mtime > t)
+	if (stat(locale_filename, &st) != 0)
+	{
+		// TODO: warn user file on disk is missing
+	}
+	else if (doc_list[idx].mtime > t || st.st_mtime > t)
 	{
 		geany_debug("Strange: Something is wrong with the time stamps.");
-		return FALSE;
 	}
-
-	if (doc_list[idx].mtime < st.st_mtime)
+	else if (doc_list[idx].mtime < st.st_mtime)
 	{
 		gchar *basename = g_path_get_basename(doc_list[idx].file_name);
 
@@ -516,91 +365,193 @@ gboolean utils_check_disk_status(gint idx)
 			doc_list[idx].mtime = st.st_mtime;
 
 		g_free(basename);
-		return TRUE; //file has changed
+		ret = TRUE; // file has changed
 	}
-	return FALSE;
+	g_free(locale_filename);
+	return ret;
+}
+
+
+/* Should be used only with utils_get_current_function. */
+static gboolean current_function_changed(gint cur_idx, gint cur_line, gint fold_level)
+{
+	static gint old_line = -2;
+	static gint old_idx = -1;
+	static gint old_fold_level = -1;
+	gboolean ret;
+
+	// check if the cached line and file index have changed since last time:
+	if (cur_idx != old_idx)
+		ret = TRUE;
+	else
+	if (cur_line == old_line)
+		ret = FALSE;
+	else
+	{
+		// if the line has only changed by 1 and fold_level is the same, return unchanged.
+		if (abs(cur_line - old_line) == 1)
+		{
+			ret = (fold_level != old_fold_level);
+		}
+		else ret = TRUE;
+	}
+
+	//record current line and file index for next time
+	old_line = cur_line;
+	old_idx = cur_idx;
+	old_fold_level = fold_level;
+	return ret;
+}
+
+
+/* Parse the function name up to 2 lines before tag_line.
+ * C++ like syntax should be parsed by parse_cpp_function_at_line, otherwise the return
+ * type or argument names can be confused with the function name. */
+static gchar *parse_function_at_line(ScintillaObject *sci, gint tag_line)
+{
+	gint start, end, max_pos;
+	gchar *cur_tag;
+	gint fn_style;
+
+	switch (sci_get_lexer(sci))
+	{
+		case SCLEX_RUBY:	fn_style = SCE_RB_DEFNAME; break;
+		case SCLEX_PYTHON:	fn_style = SCE_P_DEFNAME; break;
+		default: fn_style = SCE_C_IDENTIFIER;	// several lexers use SCE_C_IDENTIFIER
+	}
+	start = sci_get_position_from_line(sci, tag_line - 2);
+	max_pos = sci_get_position_from_line(sci, tag_line + 1);
+	while (sci_get_style_at(sci, start) != fn_style
+		&& start < max_pos) start++;
+
+	end = start;
+	while (sci_get_style_at(sci, end) == fn_style
+		&& end < max_pos) end++;
+
+	if (start == end) return NULL;
+	cur_tag = g_malloc(end - start + 1);
+	sci_get_text_range(sci, start, end, cur_tag);
+	return cur_tag;
+}
+
+
+// Parse the function name
+static gchar *parse_cpp_function_at_line(ScintillaObject *sci, gint tag_line)
+{
+	gint start, end, first_pos, max_pos;
+	gint tmp;
+	gchar c;
+	gchar *cur_tag;
+
+	first_pos = end = sci_get_position_from_line(sci, tag_line);
+	max_pos = sci_get_position_from_line(sci, tag_line + 1);
+	tmp = 0;
+	// goto the begin of function body
+	while (end < max_pos &&
+		(tmp = sci_get_char_at(sci, end)) != '{' &&
+		tmp != 0) end++;
+	if (tmp == 0) end --;
+
+	// go back to the end of function identifier
+	while (end > 0 && end > first_pos - 500 &&
+		(tmp = sci_get_char_at(sci, end)) != '(' &&
+		tmp != 0) end--;
+	end--;
+	if (end < 0) end = 0;
+
+	// skip whitespaces between identifier and (
+	while (end > 0 && isspace(sci_get_char_at(sci, end))) end--;
+
+	start = end;
+	c = 0;
+	// Use tmp to find SCE_C_IDENTIFIER or SCE_C_GLOBALCLASS chars
+	while (start >= 0 && ((tmp = sci_get_style_at(sci, start)) == SCE_C_IDENTIFIER
+		 ||  tmp == SCE_C_GLOBALCLASS
+		 || (c = sci_get_char_at(sci, start)) == '~'
+		 ||  c == ':'))
+		start--;
+	if (start != 0 && start < end) start++;	// correct for last non-matching char
+
+	if (start == end) return NULL;
+	cur_tag = g_malloc(end - start + 2);
+	sci_get_text_range(sci, start, end + 1, cur_tag);
+	return cur_tag;
 }
 
 
 gint utils_get_current_function(gint idx, const gchar **tagname)
 {
 	static gint tag_line = -1;
-	gint line;
-	static gint old_line = -1;
-	static gint old_idx = -1;
 	static gchar *cur_tag = NULL;
+	gint line;
 	gint fold_level;
-	gint start, end, last_pos;
-	gint tmp;
-	const GList *tags;
+	TMWorkObject * tm_file = doc_list[idx].tm_file;
 
 	line = sci_get_current_line(doc_list[idx].sci, -1);
+	fold_level = sci_get_fold_level(doc_list[idx].sci, line);
 	// check if the cached line and file index have changed since last time:
-	if (line == old_line && idx == old_idx)
+	if (! current_function_changed(idx, line, fold_level))
 	{
 		// we can assume same current function as before
 		*tagname = cur_tag;
 		return tag_line;
 	}
 	g_free(cur_tag); // free the old tag, it will be replaced.
-	//record current line and file index for next time
-	old_line = line;
-	old_idx = idx;
 
-	// look first in the tag list
-	tags = utils_get_tag_list(idx, tm_tag_max_t);
-	for (; tags; tags = g_list_next(tags))
+	// if line is at base fold level, we're not in a function
+	if ((fold_level & SC_FOLDLEVELNUMBERMASK) == SC_FOLDLEVELBASE)
 	{
-		tag_line = ((GeanySymbol*)tags->data)->line;
-		if (line + 1 == tag_line)
+		cur_tag = g_strdup(_("unknown"));
+		*tagname = cur_tag;
+		tag_line = -1;
+		return tag_line;
+	}
+
+	// if the document has no changes, get the previous function name from TM
+	if(! doc_list[idx].changed && tm_file != NULL && tm_file->tags_array != NULL)
+	{
+		TMTag *tag = TM_TAG(tm_get_current_function(tm_file->tags_array, line));
+
+		if (tag != NULL)
 		{
-			cur_tag = g_strdup(strtok(((GeanySymbol*)tags->data)->str, " "));
+			gchar *tmp;
+			tmp = tag->atts.entry.scope;
+			cur_tag = tmp ? g_strconcat(tmp, "::", tag->name, NULL) : g_strdup(tag->name);
 			*tagname = cur_tag;
+			tag_line = tag->atts.entry.line;
 			return tag_line;
 		}
 	}
 
+	/* parse the current function name here because TM line numbers may have changed,
+	 * and it would take too long to reparse the whole file. */
 	if (doc_list[idx].file_type != NULL &&
-		doc_list[idx].file_type->id != GEANY_FILETYPES_JAVA &&
 		doc_list[idx].file_type->id != GEANY_FILETYPES_ALL)
 	{
-		fold_level = sci_get_fold_level(doc_list[idx].sci, line);
+		// for Java the functions are one fold level above the class scope
+		gint top_fold_level = (doc_list[idx].file_type->id == GEANY_FILETYPES_JAVA) ?
+			SC_FOLDLEVELBASE + 1 : SC_FOLDLEVELBASE;
 
-		if ((fold_level & 0xFF) != 0)
+		tag_line = line;
+		do	// find the top level fold point
 		{
-			tag_line = line;
-			while((fold_level & SC_FOLDLEVELNUMBERMASK) != SC_FOLDLEVELBASE && tag_line >= 0)
+			tag_line = sci_get_fold_parent(doc_list[idx].sci, tag_line);
+			fold_level = sci_get_fold_level(doc_list[idx].sci, tag_line);
+		} while (tag_line >= 0 &&
+			(fold_level & SC_FOLDLEVELNUMBERMASK) != top_fold_level);
+
+		if (tag_line >= 0)
+		{
+			if (sci_get_lexer(doc_list[idx].sci) == SCLEX_CPP)
+				cur_tag = parse_cpp_function_at_line(doc_list[idx].sci, tag_line);
+			else
+				cur_tag = parse_function_at_line(doc_list[idx].sci, tag_line);
+
+			if (cur_tag != NULL)
 			{
-				fold_level = sci_get_fold_level(doc_list[idx].sci, --tag_line);
+				*tagname = cur_tag;
+				return tag_line;
 			}
-
-			start = sci_get_position_from_line(doc_list[idx].sci, tag_line - 2);
-			last_pos = sci_get_length(doc_list[idx].sci);
-			tmp = 0;
-			while (sci_get_style_at(doc_list[idx].sci, start) != SCE_C_IDENTIFIER
-				&& sci_get_style_at(doc_list[idx].sci, start) != SCE_C_GLOBALCLASS
-				&& start < last_pos) start++;
-			end = start;
-			// Use tmp to find SCE_C_IDENTIFIER or SCE_C_GLOBALCLASS chars
-			// this fails on C++ code like 'Vek3 Vek3::mul(double s)' this code returns
-			// "Vek3" because the return type of the prototype is also a GLOBALCLASS,
-			// no idea how to fix at the moment
-			// fails also in C with code like
-			// typedef void viod;
-			// viod do_nothing() {}  -> return viod instead of do_nothing
-			// perhaps: get the first colon, read forward the second colon and then method
-			// name, then go from the first colon backwards and read class name until space
-			while (((tmp = sci_get_style_at(doc_list[idx].sci, end)) == SCE_C_IDENTIFIER
-				 || tmp == SCE_C_GLOBALCLASS
-				 || sci_get_char_at(doc_list[idx].sci, end) == '~'
-				 || sci_get_char_at(doc_list[idx].sci, end) == ':')
-				 && end < last_pos) end++;
-
-			cur_tag = g_malloc(end - start + 1);
-			sci_get_text_range(doc_list[idx].sci, start, end, cur_tag);
-			*tagname = cur_tag;
-
-			return tag_line;
 		}
 	}
 
@@ -737,7 +688,7 @@ gdouble utils_scale_round (gdouble val, gdouble factor)
  * NULL-safe string comparison. Returns TRUE if both a and b are
  * NULL or if a and b refer to valid strings which are equal.
  */
-gboolean utils_strcmp(const gchar *a, const gchar *b)
+gboolean utils_str_equal(const gchar *a, const gchar *b)
 {
 	if (a == NULL && b == NULL) return TRUE;
 	else if (a == NULL || b == NULL) return FALSE;
@@ -1242,20 +1193,6 @@ gchar *utils_get_current_time_string()
 }
 
 
-TMTag *utils_find_tm_tag(const GPtrArray *tags, const gchar *tag_name)
-{
-	guint i;
-	g_return_val_if_fail(tags != NULL, NULL);
-
-	for (i = 0; i < tags->len; ++i)
-	{
-		if (utils_strcmp(TM_TAG(tags->pdata[i])->name, tag_name))
-			return TM_TAG(tags->pdata[i]);
-	}
-	return NULL;
-}
-
-
 GIOChannel *utils_set_up_io_channel(gint fd, GIOCondition cond, GIOFunc func, gpointer data)
 {
 	GIOChannel *ioc;
@@ -1434,50 +1371,6 @@ gboolean utils_str_replace_escape(gchar *string)
 }
 
 
-gchar *utils_scan_unicode_bom(const gchar *string)
-{
-	if ((unsigned char)string[0] == 0xef && (unsigned char)string[1] == 0xbb &&
-		(unsigned char)string[2] == 0xbf)
-	{
-		return g_strdup("UTF-8");
-	}
-	else if ((unsigned char)string[0] == 0x00 && (unsigned char)string[1] == 0x00 &&
-			 (unsigned char)string[2] == 0xfe && (unsigned char)string[3] == 0xff)
-	{
-		return g_strdup("UTF-32BE"); // Big endian
-	}
-	else if ((unsigned char)string[0] == 0xff && (unsigned char)string[1] == 0xfe &&
-			 (unsigned char)string[2] == 0x00 && (unsigned char)string[3] == 0x00)
-	{
-		return g_strdup("UTF-32LE"); // Little endian
-	}
-	else if ((unsigned char)string[0]==0xfe && (unsigned char)string[1] == 0xff)
-	{
-		return g_strdup("UTF-16BE"); // Big endian
-	}
-	else if ((unsigned char)string[0] == 0xff && (unsigned char)string[1] == 0xfe)
-	{
-		return g_strdup("UTF-16LE"); // Little endian
-	}
-	else if ((string[0] == 0x2b && string[1] == 0x2f && string[2] == 0x76) &&
-			 (string[3] == 0x38 || string[3] == 0x39 || string[3] == 0x2b || string[3] == 0x2f))
-	{
-		 return g_strdup("UTF-7");
-	}
-	return NULL;
-}
-
-
-gboolean utils_is_unicode_charset(const gchar *string)
-{
-	if (string != NULL && (strncmp(string, "UTF", 3) == 0 || strncmp(string, "UCS", 3) == 0))
-	{
-		return TRUE;
-	}
-	return FALSE;
-}
-
-
 /* Wraps a string in place, replacing a space with a newline character.
  * wrapstart is the minimum position to start wrapping or -1 for default */
 gboolean utils_wrap_string(gchar *string, gint wrapstart)
@@ -1516,3 +1409,23 @@ gchar *utils_get_utf8_from_locale(const gchar *locale_text)
 }
 
 
+/* frees all passed pointers if they are non-NULL, the first argument is nothing special,
+ * it will also be freed, the list should be ended with NULL */
+void utils_free_pointers(gpointer first, ...)
+{
+	va_list a;
+	gpointer sa;
+
+    for (va_start(a, first);  (sa = va_arg(a, gpointer), sa!=NULL);)
+    {
+    	if (sa != NULL)
+    		g_free(sa);
+	}
+	va_end(a);
+
+    if (first != NULL)
+    	g_free(first);
+}
+
+
+
