@@ -1,7 +1,8 @@
 /*
  *      utils.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006 Enrico Troeger <enrico.troeger@uvena.de>
+ *      Copyright 2005-2007 Enrico Tröger <enrico.troeger@uvena.de>
+ *      Copyright 2006-2007 Nick Treleaven <nick.treleaven@btinternet.com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: utils.h 794 2006-09-06 17:31:53Z eht16 $
+ * $Id: utils.h 1270 2007-02-12 18:42:15Z eht16 $
  */
 
 
@@ -34,8 +35,6 @@ gboolean utils_isbrace(gchar c);
 
 gboolean utils_is_opening_brace(gchar c);
 
-const GList *utils_get_tag_list(gint idx, guint tag_types);
-
 gint utils_get_local_tag(gint idx, const gchar *qual_name);
 
 gboolean utils_goto_file_line(const gchar *file, gboolean is_tm_filename, gint line);
@@ -44,13 +43,6 @@ gboolean utils_goto_line(gint idx, gint line);
 
 gint utils_write_file(const gchar *filename, const gchar *text);
 
-gchar *utils_convert_to_utf8(const gchar *buffer, gsize size, gchar **used_encoding);
-
-/* Converts a string from the given charset to UTF-8.
- * If fast is set, no further checks are performed. */
-gchar *utils_convert_to_utf8_from_charset(const gchar *buffer, gsize size, const gchar *charset,
-										  gboolean fast);
-
 /**
  * (stolen from anjuta and modified)
  * Search backward through size bytes looking for a '<', then return the tag if any
@@ -58,7 +50,7 @@ gchar *utils_convert_to_utf8_from_charset(const gchar *buffer, gsize size, const
  */
 gchar *utils_find_open_xml_tag(const gchar sel[], gint size, gboolean check_tag);
 
-gboolean utils_check_disk_status(gint idx);
+gboolean utils_check_disk_status(gint idx, gboolean force);
 
 //gchar *utils_get_current_tag(gint idx, gint direction);
 gint utils_get_current_function(gint idx, const gchar **tagname);
@@ -84,7 +76,7 @@ gdouble utils_scale_round(gdouble val, gdouble factor);
  * NULL-safe string comparison. Returns TRUE if both a and b are
  * NULL or if a and b refer to valid strings which are equal.
  */
-gboolean utils_strcmp(const gchar *a, const gchar *b);
+gboolean utils_str_equal(const gchar *a, const gchar *b);
 
 /* removes the extension from filename and return the result in
  * a newly allocated string */
@@ -136,8 +128,6 @@ gint utils_strtod(const gchar *source, gchar **end, gboolean with_route);
 // returned string must be freed.
 gchar *utils_get_current_time_string();
 
-TMTag *utils_find_tm_tag(const GPtrArray *tags, const gchar *tag_name);
-
 GIOChannel *utils_set_up_io_channel(gint fd, GIOCondition cond, GIOFunc func, gpointer data);
 
 gchar **utils_read_file_in_array(const gchar *filename);
@@ -146,10 +136,6 @@ gchar **utils_read_file_in_array(const gchar *filename);
  * Replaces \\, \r, \n, \t and \uXXX by their real counterparts */
 gboolean utils_str_replace_escape(gchar *string);
 
-gchar *utils_scan_unicode_bom(const gchar *string);
-
-gboolean utils_is_unicode_charset(const gchar *string);
-
 /* Wraps a string in place, replacing a space with a newline character.
  * wrapstart is the minimum position to start wrapping or -1 for default */
 gboolean utils_wrap_string(gchar *string, gint wrapstart);
@@ -160,4 +146,8 @@ gchar *utils_get_locale_from_utf8(const gchar *utf8_text);
 /* Simple wrapper for g_locale_to_utf8; returns a copy of locale_text on failure. */
 gchar *utils_get_utf8_from_locale(const gchar *locale_text);
 
+/* frees all passed pointers if they are non-NULL, the first argument is nothing special,
+ * it will also be freed, the list should be ended with NULL */
+void utils_free_pointers(gpointer first, ...);
+
 #endif
