@@ -17,7 +17,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: win32.c 810 2006-09-11 07:41:37Z eht16 $
+ *  $Id: win32.c 972 2006-11-07 11:24:22Z ntrel $
  */
 
 
@@ -83,7 +83,8 @@ static gchar *win32_get_filters(gboolean exe)
 }
 
 
-void win32_show_file_dialog(gboolean file_open)
+// return TRUE if the dialog was not cancelled.
+gboolean win32_show_file_dialog(gboolean file_open)
 {
 	OPENFILENAME of;
 	gint retval;
@@ -134,7 +135,7 @@ void win32_show_file_dialog(gboolean file_open)
 			win32_message_dialog(GTK_MESSAGE_ERROR, error);
 		}
 		g_free(fname);
-		return;
+		return FALSE;
 	}
 
 	if (file_open)
@@ -145,16 +146,7 @@ void win32_show_file_dialog(gboolean file_open)
 		x = of.nFileOffset - 1;
 		if (x != strlen(fname))
 		{	// open a single file
-			if (gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)) >= GEANY_MAX_OPEN_FILES)
-			{
-				dialogs_show_error(
-			_("You have opened too many files. There is a limit of %d concurrent open files."),
-			GEANY_MAX_OPEN_FILES);
-			}
-			else
-			{
-				document_open_file(-1, fname, 0, of.Flags & OFN_READONLY, NULL, NULL);
-			}
+			document_open_file(-1, fname, 0, of.Flags & OFN_READONLY, NULL, NULL);
 		}
 		else
 		{	// open mutiple files
@@ -162,9 +154,7 @@ void win32_show_file_dialog(gboolean file_open)
 			{
 				if (! fname[x])
 				{
-					if (! fname[x+1] &&	(
-						gtk_notebook_get_n_pages(GTK_NOTEBOOK(app->notebook)) < GEANY_MAX_OPEN_FILES))
-						break;
+					if (! fname[x+1]) break;
 
 					g_snprintf(file_name, 254, "%s\\%s", fname, fname + x + 1);
 					document_open_file(-1, file_name, 0, of.Flags & OFN_READONLY, NULL, NULL);
@@ -180,6 +170,7 @@ void win32_show_file_dialog(gboolean file_open)
 		document_save_file(idx, TRUE);
 	}
 	g_free(fname);
+	return (retval != 0);
 }
 
 
@@ -348,6 +339,12 @@ gboolean win32_message_dialog(GtkMessageType type, const gchar *msg)
 			title = _("Question");
 			break;
 		}
+		case GTK_MESSAGE_WARNING:
+		{
+			t = MB_OK | MB_ICONWARNING;
+			title = _("Warning");
+			break;
+		}
 		default:
 		{
 			t = MB_OK | MB_ICONINFORMATION;
