@@ -34,7 +34,9 @@
     ShParser, \
     DParser, \
     FortranParser, \
-    FeriteParser
+    FeriteParser, \
+    DiffParser, \
+    VhdlParser
 
 /*
 langType of each parser
@@ -58,6 +60,8 @@ langType of each parser
 17	DParser
 18	FortranParser
 19	FeriteParser
+20	DiffParser
+21	VhdlParser
 */
 #endif	/* _PARSERS_H */
 
